// "Build Engine & Tools" Copyright (c) 1993-1997 Ken Silverman
// Ken Silverman's official web site: "http://www.advsys.net/ken"
// See the included license file "BUILDLIC.TXT" for license info.

//   This module keeps track of a 1-D cacheing system.
//   To use this module, here's all you need to do:
//
//   Step 1: Allocate a nice BIG buffer, like from 1MB-4MB and
//           Call initcache(long cachestart, long cachesize) where
//
//              cachestart = (long)(pointer to start of BIG buffer)
//              cachesize = length of BIG buffer
//
//   Step 2: Call allocache(long ptr, long siz) whenever
//              you need to allocate a rectangular region, where
//
//              ptr = (long)(pointer to (4-byte pointer to thing))
//                 Confused?  Using this method, cache2d can take previously
//                 allocated things out of the cache safely.  For each thing
//                 you allocate with this function, you should have some
//                 form of a 4-byte pointer.  This pointer will be the start
//                 of the thing if it is in cache, otherwise it should be 0.
//                 Given the pointer to this variables, Cache2d can now set
//                 it to 0 for you if it needs to remove that thing from the
//                 cache.
//              siz = number of bytes
//
//   Step 3: If you need to re-size the cacheing buffer then your should
//           Call uninitcache() just before changing the buffer since it
//           try to remove everything from the cache by setting all pointers
//           to 0.  You only need to call uninitcache() if you are changing
//           the BIG buffer somehow or need to call initcache(?,?) again.

#define MAXCACHEOBJECTS 4096

static long cachestart = 0, cachesize = 0, cacheplc = 0;
static long cachelistplc = 0, cachelistend = 0;
static long cacheptr[MAXCACHEOBJECTS];
static long cacheleng[MAXCACHEOBJECTS];

initcache(long dacachestart, long dacachesize)
{
	uninitcache();
	cachestart = dacachestart;
	cachesize = dacachesize;
	cacheplc = 0;
	cachelistplc = 0;
	cachelistend = 0;
}

uninitcache()
{
	long *longptr;

		//Suck everything out of cache
	while (cachelistplc != cachelistend)
	{
		longptr = (long *)cacheptr[cachelistplc]; *longptr = 0;
		cachelistplc = ((cachelistplc+1)&(MAXCACHEOBJECTS-1));
	}
}

allocache (long dacacheptr, long siz)
{
	long *longptr, z;

	if (cachesize <= 0) return;

	if (cacheplc+siz <= cachesize)
	{
		while (cachelistplc != cachelistend)
		{
			longptr = (long *)cacheptr[cachelistplc]; z = (*longptr)-cachestart;
			if ((z < cacheplc) || (z >= cacheplc+siz)) break;
			*longptr = 0; cachelistplc = ((cachelistplc+1)&(MAXCACHEOBJECTS-1));
		}
	}
	else
	{
		while (cachelistplc != cachelistend)
		{
			longptr = (long *)cacheptr[cachelistplc];
			if ((*longptr)-cachestart < cacheplc) break;
			*longptr = 0; cachelistplc = ((cachelistplc+1)&(MAXCACHEOBJECTS-1));
		}
		while (cachelistplc != cachelistend)
		{
			longptr = (long *)cacheptr[cachelistplc];
			if ((*longptr)-cachestart >= siz) break;
			*longptr = 0; cachelistplc = ((cachelistplc+1)&(MAXCACHEOBJECTS-1));
		}
		cacheplc = 0;
	}

	longptr = (long *)dacacheptr; *longptr = cachestart+cacheplc;
	cacheptr[cachelistend] = dacacheptr;
	cacheleng[cachelistend] = siz;
	cachelistend = ((cachelistend+1)&(MAXCACHEOBJECTS-1));

	cacheplc += siz;
}
