// "Build Engine & Tools" Copyright (c) 1993-1997 Ken Silverman
// Ken Silverman's official web site: "http://www.advsys.net/ken"
// See the included license file "BUILDLIC.TXT" for license info.

//I support 320*200, 640*480, 800*600, 1024*768, 1280*1024, 1600*1200

#include <i86.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#define MAXXDIM 1600
#define MAXYDIM 1200

#define MAXTILES 4096
#define MAXTILEFILES 256
#define MAXPALOOKUPS 64
#define MAXPOINTS 4096
#define MAXLINES 16384
#define MAXPLANES 4096
#define MAXREGIONS (MAXPLANES<<1)
#define MAXGROUPS 1024
#define MAXSTATUS 1024
#define MAXCLIPLANES 256
#define MAXCLIPOINTS 512
#define NUMOPTIONS 6
#define NUMKEYS 17
#define MAXDIAG MAXXDIM+MAXXDIM+MAXYDIM
#define MAXBUNCHES 256
#define MAXSORTNUM 64
#define AVERAGEFRAMES 16

#define MAXSPOINTS 4096
#define MAXSPLANES 4096
#define MAXSOBJECTS 256
#define MAXSPRIS 4096

#define MAXSECTORS 1024
#define MAXWALLS 4096
#define MAXSPRITES 4096

static long xdim = 320, ydim = 200, brinum = 4, curbrightness;
static long xdimup15, ydimup15, xpan = 0, ypan = 0;
static long xdimscale, ydimscale, xreszoom, zoom = 65536;
static long xdimscale1, xdimscale2, ydimscale1, ydimscale2;

static long px[MAXPOINTS], py[MAXPOINTS], pz[MAXPOINTS];
static long rx[MAXPOINTS], ry[MAXPOINTS], rz[MAXPOINTS];
static long rx2[MAXPOINTS], ry2[MAXPOINTS], rz2[MAXPOINTS], point22[MAXPOINTS];
static long nxi[MAXPLANES], nyi[MAXPLANES], nzi[MAXPLANES];
static long rxi[MAXPOINTS], ryi[MAXPOINTS], rzi[MAXPOINTS];
static long sx[MAXPOINTS], sy[MAXPOINTS];
static long bxv[MAXPLANES][3], byv[MAXPLANES][3], bzv[MAXPLANES][3];
static short numplanes, numlines, numpoints;
static short lin[MAXLINES], lin2[MAXLINES];
static short linoff[MAXPLANES], linnum[MAXPLANES];
static short split[MAXPOINTS], splitcnt;
static char got[MAXPOINTS];

static long pxmin[MAXPLANES], pxmax[MAXPLANES];
static long pymin[MAXPLANES], pymax[MAXPLANES];
static long pzmin[MAXPLANES], pzmax[MAXPLANES];

	//Polygon Sprite variables
static char spritedrawmode = 2;
static long snumpoints = 0, snumplanes = 0, snumobjects = 0, snumgroups = 0;
static long spx[MAXSPOINTS], spy[MAXSPOINTS], spz[MAXSPOINTS];
static short sp1[MAXSPLANES], sp2[MAXSPLANES], sp3[MAXSPLANES];
static long snx[MAXSPLANES], sny[MAXSPLANES], snz[MAXSPLANES];
static short sox[MAXSOBJECTS], soy[MAXSOBJECTS], soz[MAXSOBJECTS];
static short planeoff[MAXSOBJECTS], pointoff[MAXSOBJECTS];
static short groupoff[MAXGROUPS];

static short headspritesect[MAXREGIONS+1], headspritestat[MAXSTATUS+1];
static short prevspritesect[MAXSPRITES], prevspritestat[MAXSPRITES];
static short nextspritesect[MAXSPRITES], nextspritestat[MAXSPRITES];

static long objdist[MAXSPRIS], objsort[MAXSPRIS];
static struct spritype
{
	long x, y, z;
	long a1, a2, a3;
	short xvel, yvel, zvel;
	short a1vel, a2vel, a3vel;
	short objnum, stat;
	short sectnum, statnum;
	char xscale, yscale, zscale;
	short lotag, hitag, extra;
};
static struct spritype spri[MAXSPRIS];

static long ylookup[MAXYDIM];

static signed char shade[MAXPLANES];
static short picnum[MAXPLANES];
static char planestat[MAXPLANES];
	//Bit 0 = Parallax
	//Bit 1 = Maskable
	//Bit 2 = Walk-through

static short cliplinoff[MAXCLIPLANES], cliplinnum[MAXCLIPLANES];
static short cliplanum[MAXCLIPLANES];
static long clipoints, cliplanes;
static long clipx[MAXCLIPOINTS], clipy[MAXCLIPOINTS], clipz[MAXCLIPOINTS];

static short plist[MAXPLANES];
static short parent[MAXPLANES], front[MAXPLANES], back[MAXPLANES];
static long head;

static long globm;
static long globdax, globday;
static long getd, getdxinc, getdyinc;
static long gett;
static long getu, getuxinc, getuyinc, globuadd;
static long getv, getvxinc, getvyinc, globvadd;

static long doti[MAXDIAG][16], thedot[MAXPOINTS], lastx[MAXDIAG];
static short dotc[MAXDIAG];

static long slist[MAXSORTNUM], snum;
static long cy[MAXSORTNUM*MAXXDIM], cyy[MAXSORTNUM];
static long hplc[MAXXDIM], point2[MAXPOINTS];
static short bunchstart[MAXBUNCHES], bunchend[MAXBUNCHES], numbunches;

char picsiz[MAXTILES], *pic, tilefilenum[MAXTILES];
short tilesizx[MAXTILES], tilesizy[MAXTILES];
static long tileoffx[MAXTILES], tileoffy[MAXTILES];
long artversion, numtiles, picanm[MAXTILES], waloff[MAXTILES];
static char artfilename[20];
static long localtilestart[MAXTILEFILES], localtileend[MAXTILEFILES];
static long numtilefiles, picsloaded;
static long artfil, artfilnum;

static char boardfilename[13];

long cachesize = 0, artsize = 0;
static long cachelistplc, cachelistend, cacheplc;
static long cachelist[MAXTILES], cachelistoffs[MAXTILES];
static long walsiz[MAXTILES], tilefileoffs[MAXTILES];

char option[NUMOPTIONS], keys[NUMKEYS];
char palette[768], paletteloaded = 0;
short whitecol, blackcol;
short palookup[MAXPALOOKUPS][256];
char *transluc, translucloaded = 0;
short numpalookups;

static short sintable[2048];
static long sqrtable[2048];
static char britable[16][64];
static char textfont[1024], smalltextfont[1024];

static long stereopage = 0, stereowidth = 4096;

static char *screen;
short page;
static long frameplace, drawmode = 0, reversemode = 0;
static long xres;

static short mousx, mousy, bstatus, obstatus, moustat = 0;

static char vesapageshift, curpag = 0, tsengdriver = 0;

static long searchx, searchy;
static long searchitx, searchity, searchitz;
static short searchpoint, searchplane;

long asm1, asm2, asm3, asm4;
long haddtable[MAXXDIM];

char globstat;
long globplane, globshade;
long g, g2;

static long mapversion;
static long posx, posy, posz;
static short a1, a2, a3;
static short a1vel = 0, a2vel = 0, a3vel = 0, vel = 0, svel = 0, hvel = 0;

static long gxx, gxy, gxz, gyx, gyy, gyz, gzx, gzy, gzz;
static long hxx, hxy, hxz, hyx, hyy, hyz, hzx, hzy, hzz;
//static long cornx[4], corny[4], cornz[4];

static unsigned char tempbuf[4096];
static short tempshort[MAXSPOINTS];

static short capturecount;
static char pcxheader[128] =
{
	0xa,0x5,0x1,0x8,0x0,0x0,0x0,0x0,0x3f,0x1,0xc7,0x0,
	0x40,0x1,0xc8,0x0,0x0,0x0,0x0,0x8,0x8,0x8,0x10,0x10,
	0x10,0x18,0x18,0x18,0x20,0x20,0x20,0x28,0x28,0x28,0x30,0x30,
	0x30,0x38,0x38,0x38,0x40,0x40,0x40,0x48,0x48,0x48,0x50,0x50,
	0x50,0x58,0x58,0x58,0x60,0x60,0x60,0x68,0x68,0x68,0x70,0x70,
	0x70,0x78,0x78,0x78,0x0,0x1,0x40,0x1,0x0,0x0,0x0,0x0,
	0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
	0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
	0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
	0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
	0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
};
static char vgapal16[48] =
{
	00,00,00,00,00,42,00,42,00,00,42,42,42,00,00,42,00,42,42,21,00,42,42,42,
	21,21,21,21,21,63,21,63,21,21,63,63,63,21,21,63,21,63,63,63,21,63,63,63,
};

static struct sectortype
{
	unsigned short wallptr, wallnum;
	short ceilingpicnum, floorpicnum;
	short ceilingheinum, floorheinum;
	long ceilingz, floorz;
	signed char ceilingshade, floorshade;
	char ceilingxpanning, floorxpanning;
	char ceilingypanning, floorypanning;
	char ceilingstat, floorstat;
	char ceilingpal, floorpal;
	char visibility;
	short lotag, hitag, extra;
};

static struct walltype
{
	long x, y;
	short point2, nextsector, nextwall;
	short picnum, overpicnum;
	signed char shade;
	char pal;
	short cstat;
	unsigned char xrepeat, yrepeat, xpanning, ypanning;
	short lotag, hitag, extra;
};

static struct spritetype
{
	long x, y, z;
	short cstat;
	signed char shade;
	char pal, clipdist;
	unsigned char xrepeat, yrepeat;
	signed char xoffset, yoffset;
	short picnum, ang, xvel, yvel, zvel, owner;
	short sectnum, statnum;
	short lotag, hitag, extra;
};

static struct sectortype sector[MAXSECTORS];
static struct walltype wall[MAXWALLS];
static struct spritetype sprite[MAXSPRITES];

static long frameval[AVERAGEFRAMES], framecnt = 0;
long totalclock, lockclock, synctics, numframes;
volatile char keystatus[256];
volatile unsigned char readch, oldreadch, extended;
static void (__interrupt __far *oldtimerhandler)();
static void __interrupt __far timerhandler(void);
static void (__interrupt __far *oldkeyhandler)();
static void __interrupt __far keyhandler(void);

extern long dline(long,long,long,long,long,long);
#pragma aux dline parm [eax][ebx][ecx][edx][esi][edi];
extern long vline(long,long,long,long,long,long);
#pragma aux vline parm [eax][ebx][ecx][edx][esi][edi];
extern long hline(long,long,long,long,long,long);
#pragma aux hline parm [eax][ebx][ecx][edx][esi][edi];

extern long mdline(long,long,long,long,long,long);
#pragma aux mdline parm [eax][ebx][ecx][edx][esi][edi];
extern long mvline(long,long,long,long,long,long);
#pragma aux mvline parm [eax][ebx][ecx][edx][esi][edi];
extern long mhline(long,long,long,long,long,long);
#pragma aux mhline parm [eax][ebx][ecx][edx][esi][edi];

extern long setupvline(long,long,long);
#pragma aux setupvline parm [eax][ebx][ecx];
extern long setuphline(long,long,long);
#pragma aux setuphline parm [eax][ebx][ecx];
extern long setupdline(long,long,long);
#pragma aux setupdline parm [eax][ebx][ecx];

extern long msetupvline(long,long,long);
#pragma aux msetupvline parm [eax][ebx][ecx];
extern long msetuphline(long,long,long);
#pragma aux msetuphline parm [eax][ebx][ecx];
extern long msetupdline(long,long,long);
#pragma aux msetupdline parm [eax][ebx][ecx];

extern long setxdim(long);
#pragma aux setxdim parm [eax];

#pragma aux setvmode =\
	"int 0x10",\
	parm [eax]\

#pragma aux getpixel =\
	"mov al, byte ptr [edi]",\
	parm [edi]\

#pragma aux drawpixel =\
	"mov byte ptr [edi], al",\
	parm [edi][eax]\

#pragma aux drawpixels =\
	"mov word ptr [edi], ax",\
	parm [edi][eax]\

#pragma aux drawpixelses =\
	"mov dword ptr [edi], eax",\
	parm [edi][eax]\

#pragma aux clearbuf =\
	"rep stosd",\
	parm [edi][ecx][eax]\

#pragma aux copybuf =\
	"rep movsd",\
	parm [esi][edi][ecx]\

#pragma aux copybufbyte =\
	"shr ecx, 1",\
	"jnc skip1",\
	"movsb",\
	"skip1: shr ecx, 1",\
	"jnc skip2",\
	"movsw",\
	"skip2: test ecx, ecx",\
	"jz skip3",\
	"rep movsd",\
	"skip3:",\
	parm [esi][edi][ecx]\

#pragma aux scale =\
	"imul ebx",\
	"idiv ecx",\
	parm [eax][ebx][ecx]\
	modify [eax edx]\

#pragma aux mulscale =\
	"imul ebx",\
	"shrd eax, edx, cl",\
	parm [eax][ebx][ecx]\
	modify [edx]\

#pragma aux mulscale32 =\
	"imul ebx",\
	"mov eax, edx",\
	parm [eax][ebx]\
	modify [edx]\

#pragma aux doublemulscale =\
	"imul ebx",\
	"xchg esi, eax",\
	"mov ebx, edx",\
	"imul edi",\
	"add eax, esi",\
	"adc edx, ebx",\
	"shrd eax, edx, cl",\
	parm [eax][ebx][esi][edi][ecx]\
	modify [edx]\

#pragma aux triplemulscale8 =\
	"imul edx",\
	"xchg eax, ebx",\
	"xchg edx, ecx",\
	"imul edx",\
	"add ebx, eax",\
	"adc ecx, edx",\
	"mov eax, esi",\
	"imul edi",\
	"add eax, ebx",\
	"adc edx, ecx",\
	"shrd eax, edx, 13",\
	parm [eax][edx][ebx][ecx][esi][edi]\

#pragma aux triplemulscale28 =\
	"imul edx",\
	"xchg eax, ebx",\
	"xchg edx, ecx",\
	"imul edx",\
	"add ebx, eax",\
	"adc ecx, edx",\
	"mov eax, esi",\
	"imul edi",\
	"add eax, ebx",\
	"adc edx, ecx",\
	"shrd eax, edx, 28",\
	parm [eax][edx][ebx][ecx][esi][edi]\

#pragma aux divscale =\
	"cdq",\
	"shld edx, eax, cl",\
	"sal eax, cl",\
	"idiv ebx",\
	parm [eax][ebx][ecx]\
	modify [edx]\

#pragma aux divscale32 =\
	"mov edx, eax",\
	"xor eax, eax",\
	"idiv ebx",\
	parm [eax][ebx]\
	modify [edx]\

#pragma aux getdat =\
	"shr eax, 26",\
	"shld eax, ebx, 6",\
	"mov al, byte ptr [eax+esi]",\
	"xor ah, ah",\
	"mov al, byte ptr [eax+edi]",\
	parm [eax][ebx][esi][edi]\

#pragma aux readkey =\
	"in al, 0x60",\
	"mov readch, al",\
	"in al, 0x61",\
	"or al, 0x80",\
	"out 0x61, al",\
	"and al, 0x7f",\
	"out 0x61, al",\
	modify [eax]\

#pragma aux setupmouse =\
	"mov ax, 0",\
	"int 33h",\
	"and eax, 0x0000ffff",\
	modify [eax]\

#pragma aux readmousexy =\
	"mov ax, 11d",\
	"int 33h",\
	"mov [esi], cx",\
	"mov [edi], dx",\
	parm [esi][edi]\
	modify [eax ebx ecx edx]\

#pragma aux readmousebstatus =\
	"mov ax, 5d",\
	"int 33h",\
	"mov [esi], ax",\
	parm [esi]\
	modify [eax ebx ecx edx]\

#pragma aux vesasetmode =\
	"mov ax, 0x4f02",\
	"int 0x10",\
	"and eax, 0x0000ffff",\
	parm [ebx]\
	modify [eax ebx]\

#pragma aux vesasetpage =\
	"mov cl, vesapageshift",\
	"shl dx, cl",\
	"cmp dl, curpag",\
	"je skipsetpage",\
	"mov curpag, dl",\
	"mov ax, 0x4f05",\
	"xor bx, bx",\
	"int 0x10",\
	"skipsetpage:",\
	parm [edx]\
	modify [eax ebx ecx]\

#pragma aux koutp =\
	"out dx, al",\
	parm [edx][eax]\

#pragma aux koutpw =\
	"out dx, ax",\
	parm [edx][eax]\

#pragma aux kinp =\
	"in al, dx",\
	parm [edx]\

#pragma aux klabs =\
	"or eax, eax",\
	"jns skipnegate",\
	"neg eax",\
	"skipnegate:",\
	parm [eax]\

#pragma aux ksgn =\
	"add ebx, ebx",\
	"sbb eax, eax",\
	"cmp eax, ebx",\
	"adc eax, 0",\
	parm [ebx]\

// 11,12 is 2,1-10
// 13,14 is 4,2-10
// 15,16 is 6,3-10
#pragma aux ksqrtasm =\
	"or eax, eax",\
	"jz endksqrtasm",\
	"jns skipnegate",\
	"neg eax",\
	"skipnegate: bsr ecx, eax",\
	"cmp ecx, 11",\
	"jge over2048",\
	"mov eax, dword ptr sqrtable[eax*4]",\
	"shr eax, 10",\
	"jmp endksqrtasm",\
	"over2048:",\
	"sub cl, 9",\
	"and cl, 0xfe",\
	"shr eax, cl",\
	"mov eax, dword ptr sqrtable[eax*4]",\
	"shr cl, 1",\
	"shl eax, cl",\
	"shr eax, 10",\
	"endksqrtasm:",\
	parm [eax]\
	modify [eax ebx ecx edx]\

#pragma aux qinterpolatedown16 =\
	"mov ebx, ecx",\
	"shr ecx, 1",\
	"jz skipbegcalc",\
	"begqcalc: lea edi, [edx+esi]",\
	"sar edx, 16",\
	"mov dword ptr [eax], edx",\
	"lea edx, [edi+esi]",\
	"sar edi, 16",\
	"mov dword ptr [eax+4], edi",\
	"add eax, 8",\
	"dec ecx",\
	"jnz begqcalc",\
	"test ebx, 1",\
	"jz skipbegqcalc2",\
	"skipbegcalc: sar edx, 16",\
	"mov dword ptr [eax], edx",\
	"skipbegqcalc2:",\
	parm [eax][ecx][edx][esi]\
	modify [ebx edi]\

#pragma aux redblueblit = \
	"xor ecx, ecx",\
	"begblit: mov eax, dword ptr [edx+ecx]",\
	"shl eax, 4",\
	"add eax, dword ptr [ebx+ecx]",\
	"mov dword ptr [ecx+0xa0000], eax",\
	"add ecx, 4",\
	"cmp ecx, esi",\
	"jb begblit",\
	parm [ebx][edx][esi]\
	modify [eax ecx]\

	//eax = (unsigned min)umin(eax,ebx)
#pragma aux umin =\
	"sub eax,ebx",\
	"sbb ecx,ecx",\
	"and eax,ecx",\
	"add eax,ebx",\
	parm [eax][ebx]\
	modify [ecx]\

	//eax = (unsigned max)umax(eax,ebx)
#pragma aux umax =\
	"sub eax,ebx",\
	"sbb ecx,ecx",\
	"not ecx",\
	"and eax,ecx",\
	"add eax,ebx",\
	parm [eax][ebx]\
	modify [ecx]\

#pragma aux hicolorblit = \
	"sub edi, 8",\
	"begblit: mov eax, dword ptr [esi]",\
	"add edi, 8",\
	"mov ebx, eax",\
	"rol eax, 16",\
	"add esi, 4",\
	"xchg ax, bx",\
	"mov dword ptr [edi], eax",\
	"mov dword ptr [edi+4], ebx",\
	"dec ecx",\
	"jnz begblit",\
	parm [esi][edi][ecx]\
	modify [eax ebx]\

void __interrupt __far timerhandler()
{
	totalclock++;
	keytimerstuff();

	//_chain_intr(oldtimerhandler);
	koutp(0x20,0x20);
}

keytimerstuff()
{
	if (keystatus[keys[10]] > 0)
	{
		a2vel -= 16;
		if (a2vel < -128) a2vel = -128;
	}
	if (keystatus[keys[11]] > 0)
	{
		a2vel += 16;
		if (a2vel > 127) a2vel = 127;
	}
	if (keystatus[keys[5]] == 0)
	{
		if (keystatus[keys[2]] > 0)
		{
			a3vel -= 16;
			if (a3vel < -128) a3vel = -128;
		}
		if (keystatus[keys[3]] > 0)
		{
			a3vel += 16;
			if (a3vel > 127) a3vel = 127;
		}
	}
	else
	{
		if (keystatus[keys[2]] > 0)
		{
			svel += 8;
			if (svel > 127) svel = 127;
		}
		if (keystatus[keys[3]] > 0)
		{
			svel -= 8;
			if (svel < -128) svel = -128;
		}
	}
	if (keystatus[keys[0]] > 0)
	{
		vel += 8;
		if (vel > 127) vel = 127;
	}
	if (keystatus[keys[1]] > 0)
	{
		vel -= 8;
		if (vel < -128) vel = -128;
	}

	if (a2vel < 0)
	{
		a2vel += 12;
		if (a2vel > 0) a2vel = 0;
	}
	if (a2vel > 0)
	{
		a2vel -= 12;
		if (a2vel < 0) a2vel = 0;
	}
	if (a3vel < 0)
	{
		a3vel += 12;
		if (a3vel > 0) a3vel = 0;
	}
	if (a3vel > 0)
	{
		a3vel -= 12;
		if (a3vel < 0) a3vel = 0;
	}
	if (svel < 0)
	{
		svel += 2;
		if (svel > 0) svel = 0;
	}
	if (svel > 0)
	{
		svel -= 2;
		if (svel < 0) svel = 0;
	}
	if (vel < 0)
	{
		vel += 2;
		if (vel > 0) vel = 0;
	}
	if (vel > 0)
	{
		vel -= 2;
		if (vel < 0) vel = 0;
	}
}

void __interrupt __far keyhandler()
{
	oldreadch = readch;
	readkey();
	if ((readch|1) == 0xe1)
		extended = 128;
	else
	{
		if (oldreadch != readch)
			keystatus[(readch&127)+extended] = ((readch>>7)^1);
		extended = 0;
	}
	koutp(0x20,0x20);
}

main(short int argc,char **argv)
{
	long i, j, k, nextj, ox, oy, oz, templong, dist, nx1, ny1, nz1;
	long z0, z1, z2, temppicnum = -1, tempshade = -1, dragpoint;
	short joinplane = -1, joinstart, bad, shiftstate, splitplane;
	long dax, day, daz, xvect, yvect, zvect;

	if (argc >= 2)
	{
		strcpy(&boardfilename,argv[1]);
		if (strchr(boardfilename,'.') == 0)
			strcat(boardfilename,".map");
	}
	else
		strcpy(&boardfilename,"polytex.map");

	loadtables();
	if ((option[0] == 5) || (option[0] == 6))
	{
		xpan = 13824;
		ypan = 0;
		zoom = 80000;
		brinum = 12;
	}

	moustat = 0;  //Must initialize mouse stuff before timer!  But why?
	if (option[3] == 1)
		if (setupmouse() != 0)
			moustat = 1;

	if ((screen = (char *)malloc((MAXXDIM+4)*MAXYDIM)) == NULL)
		if ((screen = (char *)malloc((640+4)*480)) == NULL)
			if ((screen = (char *)malloc((320+4)*200)) == NULL)
			{
				printf("Not enough memory for screen buffer\n");
				exit(0);
			}

	loadpics("tiles000.art");

	for(i=0;i<MAXDIAG;i++)
		lastx[i] = 0x7fffffff;

	xres = (xdim>>1);
	setgamemode();
	calcscreenconsts();

	oldkeyhandler = _dos_getvect(0x9);
	_disable(); _dos_setvect(0x9, keyhandler); _enable();

	load3d("prmtvs01.3d");
	load3d("50pman.3d");

	loadboard(boardfilename);

	oldtimerhandler = _dos_getvect(0x8);
	_dos_setvect(0x8, timerhandler);
	koutp(0x43,54); koutp(0x40,9942&255); koutp(0x40,9942>>8);
	totalclock = 0L;
	lockclock = 0L;
	numframes = 0;
	synctics = 0;

	dragpoint = -1;
	splitplane = -1;

	while (keystatus[1] == 0)
	{
		setuprotate3d(a1,a2,a3);
		hxx = gxx; hxy = gxy; hxz = gxz;
		hyx = gyx; hyy = gyy; hyz = gyz;
		hzx = gzx; hzy = gzy; hzz = gzz;

		drawscreen();
		if ((option[0] == 5) || (option[0] == 6)) drawscreen();

		if (splitplane >= 0)
		{
			for(i=linoff[numplanes];i<linoff[numplanes]+linnum[numplanes]-1;i++)
			{
				j = lin[i]; k = lin[i+1];
				draw3dclipline(px[j]-posx,py[j]-posy,pz[j]-posz,px[k]-posx,py[k]-posy,pz[k]-posz,whitecol);
			}
		}

		obstatus = bstatus, getmousevalues(&mousx,&mousy,&bstatus);
		searchx += mousx;
		searchy += mousy;
		if (searchx < 4) searchx = 4;
		if (searchx >= xdim-4) searchx = xdim-4-1;
		if (searchy < 4) searchy = 4;
		if (searchy >= ydim-4) searchy = ydim-4-1;

		overwritemousecursor();

		i = totalclock;
		if (i != frameval[framecnt])
		{
			sprintf(tempbuf,"%ld",(120*AVERAGEFRAMES)/(i-frameval[framecnt]));
			printext256(0L,0L,31,-1,tempbuf,0);
			frameval[framecnt] = i;
		}
		framecnt = ((framecnt+1)&(AVERAGEFRAMES-1));

		nextpage();

		if (keystatus[0x1d] > 0)    //Shoot a bullet
		{
			keystatus[0x1d] = 0;

			if ((j = insertsprite(getregion(posx,posy,posz),1)) >= 0)
			{
				spri[j].x = posx;
				spri[j].y = posy;
				spri[j].z = posz;
				spri[j].a1 = (rand()&2047);
				spri[j].a2 = (rand()&2047);
				spri[j].a3 = (rand()&2047);

				xvect = 0; yvect = 0; zvect = 64;
				backrotate3d(&xvect,&yvect,&zvect);
				spri[j].xvel = xvect;
				spri[j].yvel = yvect;
				spri[j].zvel = zvect;

				spri[j].a1vel = (rand()&31)-16;
				spri[j].a2vel = (rand()&31)-16;
				spri[j].a3vel = (rand()&31)-16;
				spri[j].objnum = 1;
				spri[j].stat = 0;
				spri[j].xscale = 64;
				spri[j].yscale = 64;
				spri[j].zscale = 64;

				spri[j].lotag = 0;
				spri[j].hitag = 0;
				spri[j].extra = 480;
			}
		}

		for(j=headspritestat[0];j>=0;j=nextspritestat[j])
		{
			spri[j].a1 += spri[j].a1vel;
			spri[j].a2 += spri[j].a2vel;
			spri[j].a3 += spri[j].a3vel;
		}

		for(j=headspritestat[1];j>=0;j=nextj)
		{
			nextj = nextspritestat[j];

			spri[j].x += (long)spri[j].xvel*synctics;
			spri[j].y += (long)spri[j].yvel*synctics;
			spri[j].z += (long)spri[j].zvel*synctics;

			k = spri[j].objnum;
			k = getregion(sox[k]+spri[j].x,soy[k]+spri[j].y,soz[k]+spri[j].z);
			if (k < 0) { deletesprite((short)j); continue; }
			changespritesect((short)j,getregion(sox[k]+spri[j].x,soy[k]+spri[j].y,soz[k]+spri[j].z));

			templong = -1;
			for(k=headspritesect[spri[j].sectnum];k>=0;k=nextspritesect[k])
				if (j != k)
					if (klabs(spri[j].x-spri[k].x) < 384)
						if (klabs(spri[j].y-spri[k].y) < 384)
							if (klabs(spri[j].z-spri[k].z) < 384)
							{
								templong = k;
								break;
							}
			if (templong >= 0)
			{
				if (spri[templong].xscale < 240) spri[templong].xscale += 16;
				if (spri[templong].yscale < 240) spri[templong].yscale += 16;
				if (spri[templong].zscale < 240) spri[templong].zscale += 16;
				deletesprite((short)j); continue;
			}

			spri[j].a1 += spri[j].a1vel*synctics;
			spri[j].a2 += spri[j].a2vel*synctics;
			spri[j].a3 += spri[j].a3vel*synctics;
			spri[j].extra -= synctics;
			if (spri[j].extra < 0) { deletesprite((short)j); continue; }
		}

		if (keystatus[0xc] > 0)  // - (left)
		{
			if ((keystatus[0x2a]|keystatus[0x36]) > 0)
			{
				if (zoom > 16384) zoom -= (zoom>>5);
				calcscreenconsts();
			}
			else
			{
				if (xres > 16)
				{
					i = (xres>>5)+1;
					if ((xres > 160) && (xres-i <= 160)) xdim = 320, ydim = 200, setgamemode(), keystatus[0xc] = 0;
					if ((xres > 320) && (xres-i <= 320)) xdim = 640, ydim = 480, setgamemode(), keystatus[0xc] = 0;
					if ((xres > 400) && (xres-i <= 400)) xdim = 800, ydim = 600, setgamemode(), keystatus[0xc] = 0;
					if ((xres > 512) && (xres-i <= 512)) xdim = 1024, ydim = 768, setgamemode(), keystatus[0xc] = 0;
					if ((xres > 640) && (xres-i <= 640)) xdim = 1280, ydim = 1024, setgamemode(), keystatus[0xc] = 0;

					clearbuf(frameplace,(xdim*ydim)>>2,0L);
					xres -= i;
					if (xres < 16) xres = 16;

					calcscreenconsts();
				}
			}
		}
		if (keystatus[0xd] > 0)  // + (left)
		{
			if ((keystatus[0x2a]|keystatus[0x36]) > 0)
			{
				if (zoom < 262144) zoom += (zoom>>5);

				calcscreenconsts();
			}
			else
			{
				if (xres < (MAXXDIM>>1))
				{
					i = (xres>>5)+1;
					if ((xres <= 160) && (xres+i > 160)) xdim = 640, ydim = 480, setgamemode();
					if ((xres <= 320) && (xres+i > 320)) xdim = 800, ydim = 600, setgamemode();
					if ((xres <= 400) && (xres+i > 400)) xdim = 1024, ydim = 768, setgamemode();
					if ((xres <= 512) && (xres+i > 512)) xdim = 1280, ydim = 1024, setgamemode();
					if ((xres <= 640) && (xres+i > 640)) xdim = 1600, ydim = 1200, setgamemode();
					clearbuf(frameplace,(xdim*ydim)>>2,0L);
					xres += i;
					if (xres > (MAXXDIM>>1)) xres = (MAXXDIM>>1);

					i = (xres>>5)+1;
					if ((xres <= 160) && (xres+i > 160)) xdim = 320, ydim = 200, setxdim(xdim), keystatus[0xd] = 0;
					if ((xres <= 320) && (xres+i > 320)) xdim = 640, ydim = 480, setxdim(xdim), keystatus[0xd] = 0;
					if ((xres <= 400) && (xres+i > 400)) xdim = 800, ydim = 600, setxdim(xdim), keystatus[0xd] = 0;
					if ((xres <= 512) && (xres+i > 512)) xdim = 1024, ydim = 768, setxdim(xdim), keystatus[0xd] = 0;
					if ((xres <= 640) && (xres+i > 640)) xdim = 1280, ydim = 1024, setxdim(xdim), keystatus[0xd] = 0;

					calcscreenconsts();
				}
			}
		}
		if (keystatus[0x31] > 0)     // N (new board)
		{
			keystatus[0x31] = 0;
			newboard();
		}
		if (keystatus[0x26] > 0)     // L (load)
		{
			keystatus[0x26] = 0;
			loadboard(boardfilename);
		}
		if ((keystatus[0x1f]&(keystatus[0x38]|keystatus[0xb8])) > 0)     // Alt-S (save)
		{
			keystatus[0x1f] = 0;
			saveboard(boardfilename);
		}
		if (keystatus[0x2f] > 0)  // V
		{
			searchit();
			if (searchplane >= 0)
				picnum[searchplane] = gettile(picnum[searchplane]);
			keystatus[0x2f] = 0;
		}
		if (keystatus[0xf] > 0)   // Tab
		{
			keystatus[0xf] = 0;
			searchit();
			if (searchplane >= 0)
			{
				temppicnum = picnum[searchplane];
				tempshade = shade[searchplane];
			}
		}
		if (keystatus[0x1c] > 0)  // L. Enter
		{
			keystatus[0x1c] = 0;
			if (temppicnum >= 0)
			{
				searchit();
				if (searchplane >= 0)
				{
					picnum[searchplane] = temppicnum;
					shade[searchplane] = tempshade;
				}
			}
		}
		if (keystatus[0x4a] > 0)  // - (keypad)
		{
			keystatus[0x4a] = 0;
			if ((keystatus[0x38]|keystatus[0xb8]) > 0)  //ALT
			{
				 if (brinum > 0) brinum--;
				 setpalette(brinum);
			}
			else
			{
				searchit();
				if (searchplane >= 0) shade[searchplane]++;
			}
		}
		if (keystatus[0x4e] > 0)  // + (keypad)
		{
			keystatus[0x4e] = 0;
			if ((keystatus[0x38]|keystatus[0xb8]) > 0)  //ALT
			{
				if (brinum < 15) brinum++;
				setpalette(brinum);
			}
			else
			{
				searchit();
				if (searchplane >= 0) shade[searchplane]--;
			}
		}
		if (keystatus[0x51] > 0)  //Keyp. 3
		{
			searchit();
			if (searchplane >= 0)
			{
				nx1 = bxv[searchplane][1] - bxv[searchplane][0];
				ny1 = byv[searchplane][1] - byv[searchplane][0];
				nz1 = bzv[searchplane][1] - bzv[searchplane][0];
				while ((klabs(nx1) >= 32768) || (klabs(ny1) >= 32768) || (klabs(nz1) >= 32768))
					nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;
				dist = ksqrtasm(nx1*nx1+ny1*ny1+nz1*nz1);

				z0 = lin[linoff[searchplane]]; z1 = lin[linoff[searchplane]+1]; z2 = lin[linoff[searchplane]+linnum[searchplane]-1];
				bxv[searchplane][0] = px[z0]; byv[searchplane][0] = py[z0]; bzv[searchplane][0] = pz[z0];
				bxv[searchplane][1] = px[z1]; byv[searchplane][1] = py[z1]; bzv[searchplane][1] = pz[z1];
				bxv[searchplane][2] = px[z2]; byv[searchplane][2] = py[z2]; bzv[searchplane][2] = pz[z2];

				squareaspect((long)searchplane,(dist*63)>>6);
			}
		}
		if (keystatus[0x47] > 0)  //Keyp. 7
		{
			searchit();
			if (searchplane >= 0)
			{
				nx1 = bxv[searchplane][1] - bxv[searchplane][0];
				ny1 = byv[searchplane][1] - byv[searchplane][0];
				nz1 = bzv[searchplane][1] - bzv[searchplane][0];
				while ((klabs(nx1) >= 32768) || (klabs(ny1) >= 32768) || (klabs(nz1) >= 32768))
					nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;
				dist = ksqrtasm(nx1*nx1+ny1*ny1+nz1*nz1);

				z0 = lin[linoff[searchplane]]; z1 = lin[linoff[searchplane]+1]; z2 = lin[linoff[searchplane]+linnum[searchplane]-1];
				bxv[searchplane][0] = px[z0]; byv[searchplane][0] = py[z0]; bzv[searchplane][0] = pz[z0];
				bxv[searchplane][1] = px[z1]; byv[searchplane][1] = py[z1]; bzv[searchplane][1] = pz[z1];
				bxv[searchplane][2] = px[z2]; byv[searchplane][2] = py[z2]; bzv[searchplane][2] = pz[z2];

				squareaspect((long)searchplane,(dist*65)>>6);
			}
		}
		if (keystatus[0x21] > 0)  //F
		{
			keystatus[0x21] = 0;
			searchit();
			if (searchplane >= 0)
			{
				nx1 = bxv[searchplane][1] - bxv[searchplane][0];
				ny1 = byv[searchplane][1] - byv[searchplane][0];
				nz1 = bzv[searchplane][1] - bzv[searchplane][0];
				while ((klabs(nx1) >= 32768) || (klabs(ny1) >= 32768) || (klabs(nz1) >= 32768))
					nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;
				dist = ksqrtasm(nx1*nx1+ny1*ny1+nz1*nz1);

				templong = lin[linoff[searchplane]];
				for(i=0;i<linnum[searchplane]-1;i++)
					lin[linoff[searchplane]+i] = lin[linoff[searchplane]+i+1];
				lin[linoff[searchplane]+linnum[searchplane]-1] = templong;


				z0 = lin[linoff[searchplane]]; z1 = lin[linoff[searchplane]+1]; z2 = lin[linoff[searchplane]+linnum[searchplane]-1];
				bxv[searchplane][0] = px[z0]; byv[searchplane][0] = py[z0]; bzv[searchplane][0] = pz[z0];
				bxv[searchplane][1] = px[z1]; byv[searchplane][1] = py[z1]; bzv[searchplane][1] = pz[z1];
				bxv[searchplane][2] = px[z2]; byv[searchplane][2] = py[z2]; bzv[searchplane][2] = pz[z2];

				squareaspect((long)searchplane,dist);
			}
		}
		if (keystatus[0x13] > 0)  //R (Reverse mode! Inside/Outside)
		{
			keystatus[0x13] = 0;
			reversemode = 1-reversemode;
		}
		if (keystatus[0x14] > 0)  //T
		{
			keystatus[0x14] = 0;
			searchit();
			if ((searchplane >= 0) && (linnum[searchplane] >= 3))
			{
				px[numpoints] = searchitx;
				py[numpoints] = searchity;
				pz[numpoints] = searchitz;
				numpoints++;

				for(i=linoff[searchplane]+linnum[searchplane]-1;i>=linoff[searchplane];i--)
				{
					linoff[numplanes] = numlines;
					linnum[numplanes] = 3;
					lin2[numlines+0] = numlines+1;
					lin2[numlines+1] = numlines+2;
					lin2[numlines+2] = numlines+0;
					lin[numlines++] = lin[i];
					lin[numlines++] = lin[lin2[i]];
					lin[numlines++] = numpoints-1;

					for(j=0;j<3;j++)
					{
						bxv[numplanes][j] = bxv[searchplane][j];
						byv[numplanes][j] = byv[searchplane][j];
						bzv[numplanes][j] = bzv[searchplane][j];
					}
					nxi[numplanes] = nxi[searchplane];
					nyi[numplanes] = nyi[searchplane];
					nzi[numplanes] = nzi[searchplane];
					picnum[numplanes] = picnum[searchplane];
					shade[numplanes] = shade[searchplane];
					planestat[numplanes] = planestat[searchplane];

					numplanes++;
				}

				resizelinnum((long)searchplane,3L);          //Set new size
				numplanes--; numlines -= linnum[numplanes];  //Delete last plane
				j = linoff[searchplane]-linoff[numplanes];   //Copy back
				for(i=linnum[numplanes]-1;i>=0;i--)
				{
					lin[i+linoff[searchplane]] = lin[i+linoff[numplanes]];
					lin2[i+linoff[searchplane]] = lin2[i+linoff[numplanes]]+j;
				}

				makebsp();
			}
		}
		if (keystatus[0x39] > 0)  //Space (split plane)
		{
			keystatus[0x39] = 0;
			searchit();
			if (searchplane >= 0)
			{
				if (splitplane == -1)
				{
					if (searchpoint >= 0)
					{
						splitplane = searchplane;            //Start splitting

						j = numplanes;
						bxv[j][0] = bxv[searchplane][0]; byv[j][0] = byv[searchplane][0]; bzv[j][0] = bzv[searchplane][0];
						bxv[j][1] = bxv[searchplane][1]; byv[j][1] = byv[searchplane][1]; bzv[j][1] = bzv[searchplane][1];
						bxv[j][2] = bxv[searchplane][2]; byv[j][2] = byv[searchplane][2]; bzv[j][2] = bzv[searchplane][2];
						nxi[j] = nxi[searchplane];
						nyi[j] = nyi[searchplane];
						nzi[j] = nzi[searchplane];
						picnum[j] = picnum[searchplane];
						shade[j] = shade[searchplane];
						planestat[j] = planestat[searchplane];

						lin[numlines] = searchpoint;
						linoff[numplanes] = numlines;
						linnum[numplanes] = 1;
					}
				}
				else                                       //Already splitting
				{
					if (searchplane == splitplane)
					{
						if (searchpoint < 0)                    //Continue splitting
						{
							lin[numlines+linnum[numplanes]] = numpoints;
							linnum[numplanes]++;

							px[numpoints] = searchitx;
							py[numpoints] = searchity;
							pz[numpoints] = searchitz;
							numpoints++;
						}
						else                                    //Finish splitting
						{
								//Back up backwards splitting start index
							j = numlines+linnum[numplanes];

							lin[numlines+linnum[numplanes]] = searchpoint;
							linnum[numplanes]++;

							i = linoff[searchplane];
							while (lin[i] != lin[numlines+linnum[numplanes]-1])
								i++;

							i++;
							if (i >= linoff[searchplane]+linnum[searchplane])
								i = linoff[searchplane];

							while (lin[i] != lin[numlines])
							{
								lin[numlines+linnum[numplanes]] = lin[i];
								linnum[numplanes]++;

								i++;
								if (i >= linoff[searchplane]+linnum[searchplane])
									i = linoff[searchplane];
							}
							numlines += linnum[numplanes];
							numplanes++;                        //First half done


								//Put second half in fake slot, then re-size&copy
							linoff[numplanes] = numlines;
							linnum[numplanes] = 0;

							while (lin[i] != searchpoint)
							{
								lin[numlines+linnum[numplanes]] = lin[i];
								linnum[numplanes]++;

								i++;
								if (i >= linoff[searchplane]+linnum[searchplane])
									i = linoff[searchplane];
							}

							while (lin[j] != lin[linoff[numplanes-1]])
							{
								lin[numlines+linnum[numplanes]] = lin[j];
								linnum[numplanes]++;

								j--;
							}

								//Fake out resizelinnum
							numlines += linnum[numplanes]; numplanes++;
							resizelinnum((long)splitplane,(long)linnum[numplanes-1]);
							numplanes--; numlines -= linnum[numplanes];

							for(i=0;i<linnum[numplanes];i++)
								lin[i+linoff[splitplane]] = lin[i+linoff[numplanes]];

							splitplane = -1;

							makebsp();
						}
					}
					else
					{
						splitplane = -1;
					}
				}
			}
		}
		if (keystatus[0xe] > 0)  //Backspace
		{
			keystatus[0xe] = 0;
			if (linnum[numplanes] > 0)
			{
				linnum[numplanes]--;
				if (linnum[numplanes] <= 0)
					splitplane = -1;
			}
		}

		/*          Screwed up because copying lin[] to lin[] without resizing
		if (keystatus[0x24] > 0)  //J
		{
			keystatus[0x24] = 0;
			searchit();
			if (searchplane >= 0)
			{
				if (joinplane == -1)
				{
					joinplane = searchplane;
				}
				else
				{
					if (joinplane != searchplane)
					{
					  linoff[numplanes] = numlines;

					  joinstart = 0;
					  for(i=0;i<linnum[searchplane];i++)
						  for(j=0;j<linnum[joinplane];j++)
							  if (lin[linoff[searchplane]+i] == lin[linoff[joinplane]+j])
								  joinstart = i;

						k = joinstart; linnum[numplanes] = 0;
						for(i=0;i<linnum[searchplane];i++)
						{
							bad = 0;
							for(j=0;j<linnum[joinplane];j++)
								if (lin[linoff[searchplane]+k] == lin[linoff[joinplane]+j])
								{
									bad = 1;
									break;
								}

							if (bad == 0)
							{
								lin[linoff[numplanes]+linnum[numplanes]] = lin[linoff[searchplane]+k];
								joinstart = k;
								linnum[numplanes]++;
							}

							k = ((k+1)%linnum[searchplane]);
						}

						i = lin[linoff[searchplane]+((joinstart+1)%linnum[searchplane])];

						lin[linoff[numplanes]+linnum[numplanes]] = i;
						linnum[numplanes]++;

						for(j=0;j<linnum[joinplane];j++)
							if (lin[linoff[joinplane]+j] == i)
							{
								k = j;
								break;
							}

						bad = 0;
						while (bad == 0)
						{
							k = ((k+1)%linnum[joinplane]);

							for(i=0;i<linnum[searchplane];i++)
								if (lin[linoff[searchplane]+i] == lin[linoff[joinplane]+k])
								{
									bad = 1;
									break;
								}

							lin[linoff[numplanes]+linnum[numplanes]] = lin[linoff[joinplane]+k];
							linnum[numplanes]++;
						}

						linnum[searchplane] = linnum[numplanes];
						for(i=0;i<linnum[searchplane];i++)
							lin[linoff[searchplane]+i] = lin[linoff[numplanes]+i];

						numplanes--;
						for(i=0;i<linnum[numplanes];i++)
							lin[linoff[joinplane]+i] = lin[linoff[numplanes]+i];
						linnum[joinplane] = linnum[numplanes];

						//bxv[joinplane][0] = bxv[numplanes][0]; byv[joinplane][0] = byv[numplanes][0]; bzv[joinplane][0] = bzv[numplanes][0];
						//bxv[joinplane][1] = bxv[numplanes][1]; byv[joinplane][1] = byv[numplanes][1]; bzv[joinplane][1] = bzv[numplanes][1];
						//bxv[joinplane][2] = bxv[numplanes][2]; byv[joinplane][2] = byv[numplanes][2]; bzv[joinplane][2] = bzv[numplanes][2];
						//nxi[joinplane] = nxi[numplanes];
						//nyi[joinplane] = nyi[numplanes];
						//nzi[joinplane] = nzi[numplanes];
						//picnum[joinplane] = picnum[numplanes];
						//shade[joinplane] = shade[numplanes];
						//planestat[joinplane] = planestat[numplanes];

					}
					joinplane = -1;
					makebsp();
				}
			}
		}*/

		if (keystatus[0x19] > 0)          //P
		{
			keystatus[0x19] = 0;
			searchit();
			if (searchplane >= 0) planestat[searchplane] ^= 1;
		}
		if (keystatus[0x32] > 0)          //M
		{
			keystatus[0x32] = 0;
			searchit();
			if (searchplane >= 0) planestat[searchplane] ^= 2;
		}
		if (keystatus[0x30] > 0)          //B
		{
			keystatus[0x30] = 0;
			searchit();
			if (searchplane >= 0) planestat[searchplane] ^= 4;
		}

		if ((bstatus&1) > 0)
		{
			if ((obstatus&1) == 0)
			{
				if (dragpoint < 0)
				{
					searchit();
					dragpoint = searchpoint;
				}
			}

			if (dragpoint > 0)
			{
				if ((bstatus&6) > 0)
				{
					if (((a3+256)&1536) == 0) pz[dragpoint] -= (mousy<<3);
					if (((a3+256)&1536) == 512) px[dragpoint] -= (mousy<<3);
					if (((a3+256)&1536) == 1024) pz[dragpoint] += (mousy<<3);
					if (((a3+256)&1536) == 1536) px[dragpoint] += (mousy<<3);
				}
				else
					py[dragpoint] += (mousy<<3);

				if (((a3+256)&1536) == 0) px[dragpoint] += (mousx<<3);
				if (((a3+256)&1536) == 512) pz[dragpoint] -= (mousx<<3);
				if (((a3+256)&1536) == 1024) px[dragpoint] -= (mousx<<3);
				if (((a3+256)&1536) == 1536) pz[dragpoint] += (mousx<<3);

				for(i=0;i<numplanes;i++)
					for(j=0;j<linnum[i];j++)
						if (lin[linoff[i]+j] == dragpoint)
						{
							z0 = lin[linoff[i]+0]; z1 = lin[linoff[i]+1]; z2 = lin[linoff[i]+linnum[i]-1];
							bxv[i][0] = px[z0]; byv[i][0] = py[z0]; bzv[i][0] = pz[z0];
							bxv[i][1] = px[z1]; byv[i][1] = py[z1]; bzv[i][1] = pz[z1];
							bxv[i][2] = px[z2]; byv[i][2] = py[z2]; bzv[i][2] = pz[z2];
							calcnormal(i);
							squareaspect(i,1024L);
						}
			}
		}
		else if ((obstatus&1) > 0)
		{
			px[dragpoint] = ((px[dragpoint]+128)&0xffffff00);
			py[dragpoint] = ((py[dragpoint]+128)&0xffffff00);
			pz[dragpoint] = ((pz[dragpoint]+128)&0xffffff00);

			for(i=0;i<numplanes;i++)
				for(j=0;j<linnum[i];j++)
					if (lin[linoff[i]+j] == dragpoint)
					{
						z0 = lin[linoff[i]+0]; z1 = lin[linoff[i]+1]; z2 = lin[linoff[i]+linnum[i]-1];
						bxv[i][0] = px[z0]; byv[i][0] = py[z0]; bzv[i][0] = pz[z0];
						bxv[i][1] = px[z1]; byv[i][1] = py[z1]; bzv[i][1] = pz[z1];
						bxv[i][2] = px[z2]; byv[i][2] = py[z2]; bzv[i][2] = pz[z2];
						calcnormal(i);
						squareaspect(i,1024L);
					}

			dragpoint = -1;
			makebsp();
		}

		if (keystatus[0x58] > 0)   //F12
		{
			screencapture();
			keystatus[0x58] = 0;
		}

		if (keystatus[0x9c] > 0)
		{
			keystatus[0x9c] = 0;
			if ((keystatus[0x2a]|keystatus[0x36]) > 0)
			{
				if (spritedrawmode > 0)
					spritedrawmode--;
				else
					spritedrawmode = 2;
			}
			else
			{
				if ((keystatus[0x1d]|keystatus[0x9d]) > 0)
					drawmode = 1;
				else
				{
					drawmode++;
					if (drawmode == 1) drawmode = 2;
					if (drawmode == 4) drawmode = 0;
				}
			}
		}

		shiftstate = (keystatus[0x2a]|keystatus[0x36]);
		if ((keystatus[0x34] > 0) && (a1vel > (-128<<shiftstate))) a1vel -= (synctics<<(3+shiftstate));
		if ((keystatus[0x33] > 0) && (a1vel < (128<<shiftstate))) a1vel += (synctics<<(3+shiftstate));
		if ((keystatus[0x2c] > 0) && (hvel < (192<<shiftstate))) hvel += (synctics<<(2+shiftstate));
		if ((keystatus[0x1e] > 0) && (hvel > (-192<<shiftstate))) hvel -= (synctics<<(2+shiftstate));

		xvect = -(svel<<(shiftstate+1));
		yvect = hvel;
		zvect = (vel<<(shiftstate+1));
		if ((xvect|yvect|zvect) != 0)
		{
			xvect *= (synctics<<8);
			yvect *= (synctics<<8);
			zvect *= (synctics<<8);
			backrotate3d(&xvect,&yvect,&zvect);
			clipmove(&posx,&posy,&posz,xvect>>11,yvect>>11,zvect>>11);
		}

		if (keystatus[0x52] > 0)    //0 on keypad (return to normal angle)
		{
			if ((keystatus[0x1d]|keystatus[0x9d]) > 0)
			{
				a2 = ((a2+1536)&2047);
				if (a2 < 1024) { a2 -= (synctics<<1); if (a2 < 0) a2 = 0; }
				else           { a2 += (synctics<<1); if (a2 >= 2048) a2 = 0; }
				a2 = ((a2+512)&2047);
			}
			else
			{
				if (a2 < 1024) { a2 -= (synctics<<1); if (a2 < 0) a2 = 0; }
				else           { a2 += (synctics<<1); if (a2 >= 2048) a2 = 0; }
			}
			if (a1 < 1024) { a1 -= (synctics<<1); if (a1 < 0) a1 = 0; }
			else           { a1 += (synctics<<1); if (a1 >= 2048) a1 = 0; }
		}

		a1 = ((a1+((a1vel*synctics)>>5)+2048)&2047);
		a2 = ((a2+((a2vel*synctics)>>(4-shiftstate))+2048)&2047);
		a3 = ((a3+((a3vel*synctics)>>(4-shiftstate))+2048)&2047);

			//Auto-Tilting
		if (a3vel == 0)
		{
			if (a1 < 1024)
			{
				a1 -= (synctics<<1);
				if (a1 < 0) a1 = 0;
			}
			else
			{
				a1 += (synctics<<1);
				if (a1 >= 2048) a1 = 0;
			}
		}
		else
		{
			a1 = (((((((a1+1024)&2047)*7) + (1024-a3vel))>>3)+1024)&2047);
		}

		if (hvel > 0) {hvel -= (synctics<<1); if (hvel < 0) hvel = 0;}
		if (hvel < 0) {hvel += (synctics<<1); if (hvel > 0) hvel = 0;}
		if (a1vel > 0) {a1vel -= (synctics<<2); if (a1vel < 0) a1vel = 0;}
		if (a1vel < 0) {a1vel += (synctics<<2); if (a1vel > 0) a1vel = 0;}

		synctics = totalclock-lockclock;
		lockclock += synctics;
		numframes++;
	}

	if (artfil != -1) close(artfil);

	_dos_setvect(0x9, oldkeyhandler);
	_dos_setvect(0x8, oldtimerhandler);
	koutp(0x43,54); koutp(0x40,255); koutp(0x40,255);
	setvmode(0x3);
	if (totalclock != 0)
	{
		templong = (numframes*12000L)/totalclock;
		printf("%d.%1d%1d frames per second\n",(short)(templong/100),(short)((templong/10)%10),(short)(templong%10));
	}
	if ((option[0] == 5) || (option[0] == 6))
	{
		printf("stereowidth was: %ld\n",stereowidth);
		printf("xpan was %ld\n",xpan);
		printf("ypan was %ld\n",ypan);
		printf("zoom was: %ld\n",zoom);
	}
	free(pic);
}

loadtables()
{
	long i, fil;

	for(i=0;i<2048;i++)
		sqrtable[i] = calcksqrt(i<<20);

	if ((fil = open("tables.dat",O_BINARY|O_RDWR,S_IREAD)) != -1)
	{
		read(fil,&sintable[0],4096);
		lseek(fil,4096+4096+640,SEEK_SET);
		read(fil,&textfont[0],1024);
		read(fil,&smalltextfont[0],1024);
		read(fil,&britable[0],1024);
		close(fil);
	}
	if ((fil = open("setup.dat",O_BINARY|O_RDWR,S_IREAD)) != -1)
	{
		read(fil,&option[0],NUMOPTIONS);
		read(fil,&keys[0],NUMKEYS);
		close(fil);
	}
}

allocache(short tilenume)
{
	long i, thesiz;

	thesiz = tilesizx[tilenume];

	i = cachelist[(cachelistend+(MAXTILES-1))&(MAXTILES-1)];
	if ((tileoffx[i]+thesiz <= cacheplc) && (tileoffy[i]+tilesizy[i]+tilesizy[tilenume] <= 256))
	{
		tileoffx[tilenume] = tileoffx[i];
		tileoffy[tilenume] = tileoffy[i]+tilesizy[i];
	}
	else
	{
		if (cacheplc+thesiz > cachesize)
		{
			while ((cachelistoffs[cachelistplc] >= cacheplc) && (cachelistplc != cachelistend))
			{
				waloff[cachelist[cachelistplc]] = -1;
				cachelistplc = ((cachelistplc+1)&(MAXTILES-1));
			}
			while ((cachelistoffs[cachelistplc] < thesiz) && (cachelistplc != cachelistend))
			{
				waloff[cachelist[cachelistplc]] = -1;
				cachelistplc = ((cachelistplc+1)&(MAXTILES-1));
			}

			tileoffx[tilenume] = 0;
			cacheplc = thesiz;
		}
		else
		{
			while ((cachelistoffs[cachelistplc] < cacheplc+thesiz) && (cachelistoffs[cachelistplc] >= cacheplc) && (cachelistplc != cachelistend))
			{
				waloff[cachelist[cachelistplc]] = -1;
				cachelistplc = ((cachelistplc+1)&(MAXTILES-1));
			}

			tileoffx[tilenume] = cacheplc;
			cacheplc += thesiz;
		}

		tileoffy[tilenume] = 0;
	}

	cachelist[cachelistend] = tilenume;
	cachelistoffs[cachelistend] = tileoffx[tilenume];
	cachelistend = ((cachelistend+1)&(MAXTILES - 1));

	waloff[tilenume] = (tileoffx[tilenume]<<8)+tileoffy[tilenume]+FP_OFF(pic);
}

initcache()
{
	long i;

	for(i=0;i<MAXTILES;i++)
		waloff[i] = -1;

	cacheplc = 0;
	cachelistplc = 0;
	cachelistend = 0;
}

loadtile (short tilenume)
{
	char *ptr;
	long i, j, k, l, xsiz, ysiz;

	if ((walsiz[tilenume] <= 0) || (waloff[tilenume] >= 0))
		return;

	i = tilefilenum[tilenume];
	if (i != artfilnum)
	{
		if (artfil != -1) close(artfil);
		artfilnum = i;

		artfilename[7] = (i%10)+48;
		artfilename[6] = ((i/10)%10)+48;
		artfilename[5] = ((i/100)%10)+48;
		artfil = open(artfilename,O_BINARY|O_RDONLY,S_IREAD);
	}

	allocache(tilenume);
	lseek(artfil,tilefileoffs[tilenume],SEEK_SET);
	ptr = (char *)waloff[tilenume];

	xsiz = tilesizx[tilenume];
	ysiz = tilesizy[tilenume]; if (ysiz > 256) ysiz = 256;

	l = 16;
	if (ysiz <= 128)
	{
		l = 32;
		if (ysiz <= 64)
		{
			l = 64;
			if (ysiz <= 64) l = 128;
		}
	}
	for(i=0;i<xsiz;i+=l)
	{
		j = min(xsiz-i,l);
		read(artfil,tempbuf,j*ysiz);

		k = FP_OFF(tempbuf);
		while (j > 0)
		{
			copybufbyte(k,ptr,ysiz);
			j--; k += ysiz; ptr += 256;
		}
	}
}

loadpics(char filename[20])
{
	long offscount, siz;
	short int fil, i, j, k;

	for(i=0;i<MAXTILES;i++)
		walsiz[i] = 0;
	initcache();

	strcpy(artfilename,filename);

	for(i=0;i<MAXTILES;i++)
	{
		tilesizx[i] = 0;
		tilesizy[i] = 0;
		picanm[i] = 0L;
	}

	artsize = 0L;

	numtilefiles = 0;
	do
	{
		k = numtilefiles;

		artfilename[7] = (k%10)+48;
		artfilename[6] = ((k/10)%10)+48;
		artfilename[5] = ((k/100)%10)+48;
		if ((fil = open(artfilename,O_BINARY|O_RDONLY,S_IREAD)) != -1)
		{
			read(fil,&artversion,4);
			if (artversion != 1) return(-1);
			read(fil,&numtiles,4);
			read(fil,&localtilestart[k],4);
			read(fil,&localtileend[k],4);
			read(fil,&tilesizx[localtilestart[k]],(localtileend[k]-localtilestart[k]+1)<<1);
			read(fil,&tilesizy[localtilestart[k]],(localtileend[k]-localtilestart[k]+1)<<1);
			read(fil,&picanm[localtilestart[k]],(localtileend[k]-localtilestart[k]+1)<<2);

			offscount = 4+4+4+4+((localtileend[k]-localtilestart[k]+1)<<3);
			for(i=localtilestart[k];i<=localtileend[k];i++)
			{
				tilefilenum[i] = k;
				tilefileoffs[i] = offscount;
				walsiz[i] = (long)(tilesizx[i]*tilesizy[i]);
				offscount += walsiz[i];
				artsize += walsiz[i];
			}
			close(fil);

			numtilefiles++;
		}
	}
	while (k != numtilefiles);

	cachesize = ((artsize>>8)<<2);            //Need to get better estimate
	while ((pic = (char *)malloc((cachesize<<8))) == NULL)
		cachesize -= (65536L>>8);

	for(i=0;i<MAXTILES;i++)
	{
		j = 15;
		while ((j > 0) && ((1<<j) > tilesizx[i]))
			j--;
		picsiz[i] = ((char)j);
		j = 15;
		while ((j > 0) && ((1<<j) > tilesizy[i]))
			j--;
		picsiz[i] += ((char)(j<<4));
	}

	picsloaded = 1;

	artfil = -1;
	artfilnum = -1;

	return(0);
}

newboard()
{
	long i, j, k, z0, z1, z2;

	initspritelists();

	if ((keystatus[0x2a]|keystatus[0x36]) > 0)
	{
		posx = 0; posy = 0; posz = 0;
		a1 = 0; a2 = 0; a3 = 0;

		numpoints = 8;
		px[0] = -512; py[0] = -512; pz[0] = -512;
		px[1] = 512;  py[1] = -512; pz[1] = -512;
		px[2] = -512; py[2] = 512;  pz[2] = -512;
		px[3] = 512;  py[3] = 512;  pz[3] = -512;
		px[4] = -512; py[4] = -512; pz[4] = 512;
		px[5] = 512;  py[5] = -512; pz[5] = 512;
		px[6] = -512; py[6] = 512;  pz[6] = 512;
		px[7] = 512;  py[7] = 512;  pz[7] = 512;

		numplanes = 6;
		linnum[0] = 4; lin[0] = 4; lin[1] = 5; lin[2] = 7; lin[3] = 6;
		linnum[1] = 4; lin[4] = 5; lin[5] = 1; lin[6] = 3; lin[7] = 7;
		linnum[2] = 4; lin[8] = 1; lin[9] = 0; lin[10] = 2; lin[11] = 3;
		linnum[3] = 4; lin[12] = 0; lin[13] = 4; lin[14] = 6; lin[15] = 2;
		linnum[4] = 4; lin[16] = 0; lin[17] = 1; lin[18] = 5; lin[19] = 4;
		linnum[5] = 4; lin[20] = 6; lin[21] = 7; lin[22] = 3; lin[23] = 2;

		numlines = 24;

		j = 0;
		for(i=0;i<numplanes;i++)
		{
			linoff[i] = j;
			calcnormal(i);
			j += linnum[i];
		}

		for(i=0;i<numplanes;i++)
		{
			shade[i] = -4;
			picnum[i] = i+24;
			planestat[i] = 0;

			z0 = lin[linoff[i]]; z1 = lin[linoff[i]+1]; z2 = lin[linoff[i]+linnum[i]-1];
			bxv[i][0] = px[z0]; byv[i][0] = py[z0]; bzv[i][0] = pz[z0];
			bxv[i][1] = px[z1]; byv[i][1] = py[z1]; bzv[i][1] = pz[z1];
			bxv[i][2] = px[z2]; byv[i][2] = py[z2]; bzv[i][2] = pz[z2];

			squareaspect(i,1024L);
		}
	}
	else
	{
		posx = 0; posy = 0; posz = 0;
		a1 = 0; a2 = 0; a3 = 0;

		numpoints = 22;
		px[0] = 512;    py[0] = 512;    pz[0] = 512;
		px[1] = -512;   py[1] = 512;    pz[1] = 512;
		px[2] = -512;   py[2] = -256;   pz[2] = 512;
		px[3] = -256;   py[3] = -512;   pz[3] = 512;
		px[4] = 512;    py[4] = -512;   pz[4] = 512;
		px[5] = 512;    py[5] = 512;    pz[5] = -512;
		px[6] = -512;   py[6] = 512;    pz[6] = -512;
		px[7] = -512;   py[7] = -256;   pz[7] = -512;
		px[8] = -256;   py[8] = -512;   pz[8] = -512;
		px[9] = 512;    py[9] = -512;   pz[9] = -512;
		px[10] = -512;  py[10] = -512;  pz[10] = -256;
		px[11] = -512;  py[11] = -512;  pz[11] = 256;
		px[12] = 2560;  py[12] = 512;   pz[12] = 1536;
		px[13] = 512;   py[13] = 512;   pz[13] = 1536;
		px[14] = 512;   py[14] = -1536; pz[14] = 1536;
		px[15] = 1536;  py[15] = -2176; pz[15] = 1536;
		px[16] = 2560;  py[16] = -1536; pz[16] = 1536;
		px[17] = 2560;  py[17] = 512;   pz[17] = -1536;
		px[18] = 512;   py[18] = 512;   pz[18] = -1536;
		px[19] = 512;   py[19] = -1536; pz[19] = -1536;
		px[20] = 1536;  py[20] = -2176; pz[20] = -1536;
		px[21] = 2560;  py[21] = -1536; pz[21] = -1536;

		numplanes = 16;
		linnum[0] = 5; lin[0] = 3; lin[1] = 4; lin[2] = 0; lin[3] = 1; lin[4] = 2;
		linnum[1] = 5; lin[5] = 9; lin[6] = 8; lin[7] = 7; lin[8] = 6; lin[9] = 5;
		linnum[2] = 6; lin[10] = 9; lin[11] = 4; lin[12] = 3; lin[13] = 11; lin[14] = 10; lin[15] = 8;
		linnum[3] = 4; lin[16] = 0; lin[17] = 5; lin[18] = 6; lin[19] = 1;
		linnum[4] = 6; lin[20] = 10; lin[21] = 11; lin[22] = 2; lin[23] = 1; lin[24] = 6; lin[25] = 7;
		linnum[5] = 3; lin[26] = 7; lin[27] = 8; lin[28] = 10;
		linnum[6] = 3; lin[29] = 3; lin[30] = 2; lin[31] = 11;
		linnum[7] = 8; lin[32] = 19; lin[33] = 14; lin[34] = 13; lin[35] = 0; lin[36] = 4; lin[37] = 9; lin[38] = 5; lin[39] = 18;
		linnum[8] = 4; lin[40] = 20; lin[41] = 15; lin[42] = 14; lin[43] = 19;
		linnum[9] = 4; lin[44] = 15; lin[45] = 20; lin[46] = 21; lin[47] = 16;
		linnum[10] = 4; lin[48] = 16; lin[49] = 21; lin[50] = 17; lin[51] = 12;
		linnum[11] = 5; lin[52] = 14; lin[53] = 15; lin[54] = 16; lin[55] = 12; lin[56] = 13;
		linnum[12] = 5; lin[57] = 18; lin[58] = 17; lin[59] = 21; lin[60] = 20; lin[61] = 19;
		linnum[13] = 6; lin[62] = 12; lin[63] = 17; lin[64] = 18; lin[65] = 5; lin[66] = 0; lin[67] = 13;
		linnum[14] = 4; lin[68] = 9; lin[69] = 4; lin[70] = 0; lin[71] = 5;
		linnum[15] = 4; lin[72] = 4; lin[73] = 9; lin[74] = 5; lin[75] = 0;

		numlines = 76;

		j = 0;
		for(i=0;i<numplanes;i++)
		{
			linoff[i] = j;
			calcnormal(i);
			j += linnum[i];
		}

		for(i=0;i<numplanes;i++)
		{
			shade[i] = 0;
			picnum[i] = i;
			planestat[i] = 0;

			z0 = lin[linoff[i]]; z1 = lin[linoff[i]+1]; z2 = lin[linoff[i]+linnum[i]-1];
			bxv[i][0] = px[z0]; byv[i][0] = py[z0]; bzv[i][0] = pz[z0];
			bxv[i][1] = px[z1]; byv[i][1] = py[z1]; bzv[i][1] = pz[z1];
			bxv[i][2] = px[z2]; byv[i][2] = py[z2]; bzv[i][2] = pz[z2];
			squareaspect(i,1024L);
		}

		bxv[0][0] -= 256;  //Specific to my typed in board only!
		bxv[0][1] -= 256;
		bxv[0][2] -= 256;

		bzv[4][0] -= 256;  //Specific to my typed in board only!
		bzv[4][1] -= 256;
		bzv[4][2] -= 256;

		planestat[12] = 1;

		picnum[14] = 23; planestat[14] = 2;
		picnum[15] = 23; planestat[15] = 2;
	}

	for(i=0;i<numplanes;i++)
	{
		lin2[linoff[i]+linnum[i]-1] = linoff[i];
		for(k=linoff[i];k<linoff[i]+linnum[i]-1;k++)
			lin2[k] = k+1;
	}

	makebsp();
}

loadboard(char *filename)
{
	long fil, i, j, k, l, z0, z1, z2, startwall, endwall, tempx, tempy, tempz;
	short cursectnum, numsectors, numwalls, numsprites, datempshort;

	char cstat;
	long daz, daz2, tilenum, xoff, yoff, xspan, yspan, xrepeat, yrepeat;
	long cosang, sinang, dax, day, x1, y1, x2, y2, x3, y3, x4, y4;

	if ((fil = open(filename,O_BINARY|O_RDONLY,S_IREAD)) == -1)
	{
		setvmode(0x3);
		printf("File not found\n");
		exit(0);
	}

	initspritelists();

	read(fil,&mapversion,4);
	if (mapversion == 0x00020000)
	{
		read(fil,&posx,4);
		read(fil,&posy,4);
		read(fil,&posz,4);
		read(fil,&a1,2);
		read(fil,&a2,2);
		read(fil,&a3,2);
		read(fil,&numplanes,2);
		read(fil,&numlines,2);
		read(fil,&numpoints,2);
		read(fil,&px[0],numpoints<<2);
		read(fil,&py[0],numpoints<<2);
		read(fil,&pz[0],numpoints<<2);
		read(fil,&linnum[0],numplanes<<1);
		read(fil,&lin[0],numlines<<1);
		read(fil,&lin2[0],numlines<<1);
		read(fil,&bxv[0],numplanes*12);
		read(fil,&byv[0],numplanes*12);
		read(fil,&bzv[0],numplanes*12);
		read(fil,&shade[0],numplanes);
		read(fil,&picnum[0],numplanes<<1);
		read(fil,&planestat[0],numplanes);

		j = 0;
		for(i=0;i<numplanes;i++)
		{
			linoff[i] = j;
			calcnormal(i);
			j += linnum[i];
		}
	}
	else if (mapversion == 6L)                 //Convert Build map
	{
		read(fil,&posz,4);
		read(fil,&posx,4);
		read(fil,&posy,4);
		posx <<= 1;
		posy >>= 3;
		posz <<= 1;
		a1 = 0;
		a2 = 0;
		read(fil,&a3,2);
		read(fil,&cursectnum,2);

		read(fil,&numsectors,2);
		for(i=0;i<numsectors;i++)
			read(fil,&sector[i],sizeof(struct sectortype));

		read(fil,&numwalls,2);
		for(i=0;i<numwalls;i++)
			read(fil,&wall[i],sizeof(struct walltype));

		read(fil,&numsprites,2);
		for(i=0;i<numsprites;i++)
			read(fil,&sprite[i],sizeof(struct spritetype));

		numplanes = 0; numlines = 0; numpoints = 0;
		for(i=0;i<numsectors;i++)
		{
			startwall = sector[i].wallptr;
			endwall = startwall + sector[i].wallnum - 1;
			for(j=startwall;j<=endwall;j++)
			{
				if (wall[j].nextsector == -1)
				{
					linoff[numplanes] = numlines;
					linnum[numplanes] = 0;

					lin2[numlines+0] = numlines+1;
					lin2[numlines+1] = numlines+2;
					lin2[numlines+2] = numlines+3;
					lin2[numlines+3] = numlines+0;
					lin[numlines++] = newpoint(wall[j].y<<1,sector[i].ceilingz>>3,wall[j].x<<1);
					lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[i].ceilingz>>3,wall[wall[j].point2].x<<1);
					lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[i].floorz>>3,wall[wall[j].point2].x<<1);
					lin[numlines++] = newpoint(wall[j].y<<1,sector[i].floorz>>3,wall[j].x<<1);

					linnum[numplanes] = numlines-linoff[numplanes];
					picnum[numplanes] = wall[j].picnum;
					shade[numplanes] = wall[j].shade;
					planestat[numplanes] = ((wall[j].cstat&1)<<2);

					z0 = lin[linoff[numplanes]]; z1 = lin[linoff[numplanes]+1]; z2 = lin[linoff[numplanes]+3];
					bxv[numplanes][0] = px[z0]; byv[numplanes][0] = py[z0]; bzv[numplanes][0] = pz[z0];
					bxv[numplanes][1] = px[z1]; byv[numplanes][1] = py[z1]; bzv[numplanes][1] = pz[z1];
					bxv[numplanes][2] = px[z2]; byv[numplanes][2] = py[z2]; bzv[numplanes][2] = pz[z2];
					calcnormal((long)numplanes);

					if ((wall[j].xrepeat > 0) && (wall[j].yrepeat > 0))
					{
						bxv[numplanes][1] = bxv[numplanes][0] + divscale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);
						byv[numplanes][2] = byv[numplanes][0] + (256<<(picsiz[picnum[numplanes]]>>4)) / wall[j].yrepeat;
						bzv[numplanes][1] = bzv[numplanes][0] + divscale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);

						if (tilesizx[picnum[numplanes]] > 0)
						{
							tempx = -scale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
							tempz = -scale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
						}
						tempy = -mulscale(byv[numplanes][2]-byv[numplanes][0],wall[j].ypanning,8);
						if ((wall[j].cstat&4) > 0)
							tempy += ((sector[i].floorz-sector[i].ceilingz)>>3);
						bxv[numplanes][0] += tempx; byv[numplanes][0] += tempy; bzv[numplanes][0] += tempz;
						bxv[numplanes][1] += tempx; byv[numplanes][1] += tempy; bzv[numplanes][1] += tempz;
						bxv[numplanes][2] += tempx; byv[numplanes][2] += tempy; bzv[numplanes][2] += tempz;
					}

					drawbspbox((long)numplanes,whitecol);
					numplanes++;
				}
				else
				{
					if (sector[wall[j].nextsector].ceilingz > sector[i].ceilingz)
					{
						linoff[numplanes] = numlines;
						linnum[numplanes] = 0;

						lin2[numlines+0] = numlines+1;
						lin2[numlines+1] = numlines+2;
						lin2[numlines+2] = numlines+3;
						lin2[numlines+3] = numlines+0;
						lin[numlines++] = newpoint(wall[j].y<<1,sector[i].ceilingz>>3,wall[j].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[i].ceilingz>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[wall[j].nextsector].ceilingz>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[j].y<<1,sector[wall[j].nextsector].ceilingz>>3,wall[j].x<<1);

						linnum[numplanes] = numlines-linoff[numplanes];
						picnum[numplanes] = wall[j].picnum;
						shade[numplanes] = wall[j].shade;
						planestat[numplanes] = ((wall[j].cstat&1)<<2);

						z0 = lin[linoff[numplanes]]; z1 = lin[linoff[numplanes]+1]; z2 = lin[linoff[numplanes]+3];
						bxv[numplanes][0] = px[z0]; byv[numplanes][0] = py[z0]; bzv[numplanes][0] = pz[z0];
						bxv[numplanes][1] = px[z1]; byv[numplanes][1] = py[z1]; bzv[numplanes][1] = pz[z1];
						bxv[numplanes][2] = px[z2]; byv[numplanes][2] = py[z2]; bzv[numplanes][2] = pz[z2];
						calcnormal((long)numplanes);

						if ((wall[j].xrepeat > 0) && (wall[j].yrepeat > 0))
						{
							bxv[numplanes][1] = bxv[numplanes][0] + divscale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);
							byv[numplanes][2] = byv[numplanes][0] + (256<<(picsiz[picnum[numplanes]]>>4)) / wall[j].yrepeat;
							bzv[numplanes][1] = bzv[numplanes][0] + divscale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);

							if (tilesizx[picnum[numplanes]] > 0)
							{
								tempx = -scale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
								tempz = -scale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
							}
							tempy = -mulscale(byv[numplanes][2]-byv[numplanes][0],wall[j].ypanning,8);
							if ((wall[j].cstat&4) == 0)
								tempy += ((sector[wall[j].nextsector].ceilingz-sector[i].ceilingz)>>3);
							bxv[numplanes][0] += tempx; byv[numplanes][0] += tempy; bzv[numplanes][0] += tempz;
							bxv[numplanes][1] += tempx; byv[numplanes][1] += tempy; bzv[numplanes][1] += tempz;
							bxv[numplanes][2] += tempx; byv[numplanes][2] += tempy; bzv[numplanes][2] += tempz;
						}

						drawbspbox((long)numplanes,whitecol);
						numplanes++;
					}
					if (sector[wall[j].nextsector].floorz < sector[i].floorz)
					{
						linoff[numplanes] = numlines;
						linnum[numplanes] = 0;

						lin2[numlines+0] = numlines+1;
						lin2[numlines+1] = numlines+2;
						lin2[numlines+2] = numlines+3;
						lin2[numlines+3] = numlines+0;
						lin[numlines++] = newpoint(wall[j].y<<1,sector[wall[j].nextsector].floorz>>3,wall[j].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[wall[j].nextsector].floorz>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,sector[i].floorz>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[j].y<<1,sector[i].floorz>>3,wall[j].x<<1);

						linnum[numplanes] = numlines-linoff[numplanes];
						picnum[numplanes] = wall[j].picnum;
						shade[numplanes] = wall[j].shade;
						planestat[numplanes] = ((wall[j].cstat&1)<<2);

						z0 = lin[linoff[numplanes]]; z1 = lin[linoff[numplanes]+1]; z2 = lin[linoff[numplanes]+3];
						bxv[numplanes][0] = px[z0]; byv[numplanes][0] = py[z0]; bzv[numplanes][0] = pz[z0];
						bxv[numplanes][1] = px[z1]; byv[numplanes][1] = py[z1]; bzv[numplanes][1] = pz[z1];
						bxv[numplanes][2] = px[z2]; byv[numplanes][2] = py[z2]; bzv[numplanes][2] = pz[z2];
						calcnormal((long)numplanes);

						if ((wall[j].xrepeat > 0) && (wall[j].yrepeat > 0))
						{
							bxv[numplanes][1] = bxv[numplanes][0] + divscale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);
							byv[numplanes][2] = byv[numplanes][0] + (256<<(picsiz[picnum[numplanes]]>>4)) / wall[j].yrepeat;
							bzv[numplanes][1] = bzv[numplanes][0] + divscale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);

							if (tilesizx[picnum[numplanes]] > 0)
							{
								tempx = -scale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
								tempz = -scale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
							}
							tempy = -mulscale(byv[numplanes][2]-byv[numplanes][0],wall[j].ypanning,8);
							if ((wall[j].cstat&4) > 0)
								tempy += ((sector[i].ceilingz-sector[wall[j].nextsector].floorz)>>3);
							bxv[numplanes][0] += tempx; byv[numplanes][0] += tempy; bzv[numplanes][0] += tempz;
							bxv[numplanes][1] += tempx; byv[numplanes][1] += tempy; bzv[numplanes][1] += tempz;
							bxv[numplanes][2] += tempx; byv[numplanes][2] += tempy; bzv[numplanes][2] += tempz;
						}

						drawbspbox((long)numplanes,whitecol);
						numplanes++;
					}
					if ((wall[j].cstat&16) > 0)
					{
						linoff[numplanes] = numlines;
						linnum[numplanes] = 0;

						z1 = sector[i].ceilingz;
						if (sector[wall[j].nextsector].ceilingz > z1)
							z1 = sector[wall[j].nextsector].ceilingz;
						z2 = sector[i].floorz;
						if (sector[wall[j].nextsector].floorz < z2)
							z2 = sector[wall[j].nextsector].floorz;

						lin2[numlines+0] = numlines+1;
						lin2[numlines+1] = numlines+2;
						lin2[numlines+2] = numlines+3;
						lin2[numlines+3] = numlines+0;
						lin[numlines++] = newpoint(wall[j].y<<1,z1>>3,wall[j].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,z1>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[wall[j].point2].y<<1,z2>>3,wall[wall[j].point2].x<<1);
						lin[numlines++] = newpoint(wall[j].y<<1,z2>>3,wall[j].x<<1);

						linnum[numplanes] = numlines-linoff[numplanes];
						picnum[numplanes] = wall[j].overpicnum;
						shade[numplanes] = wall[j].shade;
						planestat[numplanes] = 2 + ((wall[j].cstat&1^1)<<2);  //Reverse bit 2 for masked walls

						z0 = lin[linoff[numplanes]]; z1 = lin[linoff[numplanes]+1]; z2 = lin[linoff[numplanes]+3];
						bxv[numplanes][0] = px[z0]; byv[numplanes][0] = py[z0]; bzv[numplanes][0] = pz[z0];
						bxv[numplanes][1] = px[z1]; byv[numplanes][1] = py[z1]; bzv[numplanes][1] = pz[z1];
						bxv[numplanes][2] = px[z2]; byv[numplanes][2] = py[z2]; bzv[numplanes][2] = pz[z2];
						calcnormal((long)numplanes);

						if ((wall[j].xrepeat > 0) && (wall[j].yrepeat > 0))
						{
							bxv[numplanes][1] = bxv[numplanes][0] + divscale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);
							byv[numplanes][2] = byv[numplanes][0] + (256<<(picsiz[picnum[numplanes]]>>4)) / wall[j].yrepeat;
							bzv[numplanes][1] = bzv[numplanes][0] + divscale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xrepeat<<3,picsiz[picnum[numplanes]]&15);

							if (tilesizx[picnum[numplanes]] > 0)
							{
								tempx = -scale(bxv[numplanes][1]-bxv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
								tempz = -scale(bzv[numplanes][1]-bzv[numplanes][0],wall[j].xpanning,tilesizx[picnum[numplanes]]);
							}
							tempy = -mulscale(byv[numplanes][2]-byv[numplanes][0],wall[j].ypanning,8);
							bxv[numplanes][0] += tempx; byv[numplanes][0] += tempy; bzv[numplanes][0] += tempz;
							bxv[numplanes][1] += tempx; byv[numplanes][1] += tempy; bzv[numplanes][1] += tempz;
							bxv[numplanes][2] += tempx; byv[numplanes][2] += tempy; bzv[numplanes][2] += tempz;
						}

						drawbspbox((long)numplanes,whitecol);
						numplanes++;
					}
				}
			}
		}

		for(i=0;i<numsectors;i++)
		{
			startwall = sector[i].wallptr;
			endwall = startwall + sector[i].wallnum - 1;

				//Ceiling
			linoff[numplanes] = numlines;
			linnum[numplanes] = 0;

			for(j=startwall;j<=endwall;j++)
			{
				lin2[numlines] = wall[j].point2-j+numlines;
				lin[numlines] = newpoint(wall[j].y<<1,sector[i].ceilingz>>3,wall[j].x<<1);
				numlines++;
			}
			linnum[numplanes] = numlines-linoff[numplanes];

				//Reverse order for ceilings
			k = linoff[numplanes];
			for(j=linoff[numplanes];j<numlines;j++)
				if (lin2[j] < j)
				{
					for(l=((j-k+1)>>1)-1;l>=0;l--)
					{
						datempshort = lin[k+l];
						lin[k+l] = lin[j-l];
						lin[j-l] = datempshort;
					}
					k = j+1;
				}

			picnum[numplanes] = sector[i].ceilingpicnum;
			shade[numplanes] = sector[i].ceilingshade;
			planestat[numplanes] = (sector[i].ceilingstat&1);
			j = lin[linoff[numplanes]];
			bxv[numplanes][0] = (px[j]&0xfffff000);
			byv[numplanes][0] = py[j];
			bzv[numplanes][0] = (pz[j]&0xfffff000);
			bxv[numplanes][1] = bxv[numplanes][0]+(1<<(5+(picsiz[picnum[numplanes]]&15)));
			byv[numplanes][1] = byv[numplanes][0];
			bzv[numplanes][1] = bzv[numplanes][0];
			bxv[numplanes][2] = bxv[numplanes][0];
			byv[numplanes][2] = byv[numplanes][0];
			bzv[numplanes][2] = bzv[numplanes][0]+(1<<(5+(picsiz[picnum[numplanes]]>>4)));
			nxi[numplanes] = 0;
			nyi[numplanes] = -65536;
			nzi[numplanes] = 0;
			drawbspbox((long)numplanes,whitecol);
			numplanes++;

				//Floor
			linoff[numplanes] = numlines;
			linnum[numplanes] = 0;

			k = -1;
			for(j=startwall;j<=endwall;j++)
			{
				lin2[numlines] = wall[j].point2-j+numlines;
				lin[numlines] = newpoint(wall[j].y<<1,sector[i].floorz>>3,wall[j].x<<1);
				numlines++;
			}

			linnum[numplanes] = numlines-linoff[numplanes];
			picnum[numplanes] = sector[i].floorpicnum;
			shade[numplanes] = sector[i].floorshade;
			planestat[numplanes] = (sector[i].floorstat&1);
			j = lin[linoff[numplanes]];
			bxv[numplanes][0] = (px[j]&0xfffff000);
			byv[numplanes][0] = py[j];
			bzv[numplanes][0] = (pz[j]&0xfffff000);
			bxv[numplanes][1] = bxv[numplanes][0];
			byv[numplanes][1] = byv[numplanes][0];
			bzv[numplanes][1] = bzv[numplanes][0]+(1<<(5+(picsiz[picnum[numplanes]]&15)));
			bxv[numplanes][2] = bxv[numplanes][0]+(1<<(5+(picsiz[picnum[numplanes]]>>4)));
			byv[numplanes][2] = byv[numplanes][0];
			bzv[numplanes][2] = bzv[numplanes][0];
			nxi[numplanes] = 0;
			nyi[numplanes] = 65536;
			nzi[numplanes] = 0;
			drawbspbox((long)numplanes,whitecol);
			numplanes++;
		}

		for(i=0;i<numsprites;i++)
		{
			cstat = sprite[i].cstat;
				//Sprite
			linoff[numplanes] = numlines;
			linnum[numplanes] = 4;

			tempx = (sprite[i].y<<1);
			tempy = (sprite[i].z>>3);
			tempz = (sprite[i].x<<1);

			switch (sprite[i].cstat&48)
			{
				case 0:
					if (!(sprite[i].cstat&128))
						tempy -= ((tilesizy[sprite[i].picnum]*sprite[i].yrepeat)>>2);

					k = sprite[i].picnum;
					if ((k == 110) || (k == 113) || (k == 114) || (k == 115) || (k == 120))
					{
						j = insertsprite(0,0);
						spri[j].x = tempx; spri[j].y = tempy; spri[j].z = tempz;
						spri[j].a1 = 0; spri[j].a2 = 0; spri[j].a3 = 0;
						spri[j].xvel = 0; spri[j].yvel = 0; spri[j].zvel = 0;
						spri[j].a1vel = 0;
						spri[j].a2vel = 0;
						spri[j].a3vel = (rand()&63)-32;
						spri[j].objnum = 0;
						spri[j].stat = 0;
						spri[j].xscale = 64; spri[j].yscale = 64; spri[j].zscale = 64;
						spri[j].lotag = 0;
						spri[j].hitag = 0;
						spri[j].extra = 0;
					}
					else
					{
						j = insertsprite(0,0);
						spri[j].x = tempx; spri[j].y = tempy; spri[j].z = tempz;
						spri[j].a1 = 0; spri[j].a2 = 0; spri[j].a3 = 0;
						spri[j].xvel = 0; spri[j].yvel = 0; spri[j].zvel = 0;
						spri[j].a1vel = (rand()&31)-16;
						spri[j].a2vel = (rand()&31)-16;
						spri[j].a3vel = (rand()&31)-16;
						spri[j].objnum = 2;
						spri[j].stat = 0;
						spri[j].xscale = 64; spri[j].yscale = 64; spri[j].zscale = 64;
						spri[j].lotag = 0;
						spri[j].hitag = 0;
						spri[j].extra = 0;
					}
					continue;
				case 16:
					daz = sprite[i].z;
					daz2 = sprite[i].z-((tilesizy[sprite[i].picnum]*sprite[i].yrepeat)<<2);

						//These lines get the 2 points of the rotated sprite
						 //Given: (x1, y1) starts out as the center point
					tilenum = sprite[i].picnum;
					xoff = (long)((signed char)((picanm[tilenum]>>8)&255));
					if ((cstat&4) > 0) xoff = -xoff;
					k = sprite[i].ang; l = sprite[i].xrepeat;
					dax = sintable[k&2047]*l; day = sintable[(k+1536)&2047]*l;
					l = tilesizx[tilenum]; k = (l>>1)+xoff;
					x1 = sprite[i].x-mulscale(dax,k,16); x2 = x1+mulscale(dax,l,16);
					y1 = sprite[i].y-mulscale(day,k,16); y2 = y1+mulscale(day,l,16);

					lin2[numlines+0] = numlines+1;
					lin2[numlines+1] = numlines+2;
					lin2[numlines+2] = numlines+3;
					lin2[numlines+3] = numlines+0;
					lin[numlines++] = newpoint(y1<<1,daz2>>3,x1<<1);
					lin[numlines++] = newpoint(y2<<1,daz2>>3,x2<<1);
					lin[numlines++] = newpoint(y2<<1,daz>>3,x2<<1);
					lin[numlines++] = newpoint(y1<<1,daz>>3,x1<<1);
					break;
				case 32:
					tilenum = sprite[i].picnum;
					xoff = (long)((signed char)((picanm[tilenum]>>8)&255));
					yoff = (long)((signed char)((picanm[tilenum]>>16)&255));
					if ((cstat&4) > 0) xoff = -xoff;
					if ((cstat&8) > 0) yoff = -yoff;

					k = sprite[i].ang;
					cosang = sintable[(k+512)&2047]; sinang = sintable[k];
					xspan = tilesizx[tilenum]; xrepeat = sprite[i].xrepeat;
					yspan = tilesizy[tilenum]; yrepeat = sprite[i].yrepeat;

					dax = ((xspan>>1)+xoff)*xrepeat; day = ((yspan>>1)+yoff)*yrepeat;
					x1 = sprite[i].x+doublemulscale(sinang,dax,cosang,day,16);
					y1 = sprite[i].y+doublemulscale(sinang,day,-cosang,dax,16);
					l = xspan*xrepeat;
					x2 = x1-mulscale(sinang,l,16);
					y2 = y1+mulscale(cosang,l,16);
					l = yspan*yrepeat;
					k = -mulscale(cosang,l,16); x3 = x2+k; x4 = x1+k;
					k = -mulscale(sinang,l,16); y3 = y2+k; y4 = y1+k;

					lin2[numlines+0] = numlines+1;
					lin2[numlines+1] = numlines+2;
					lin2[numlines+2] = numlines+3;
					lin2[numlines+3] = numlines+0;
					lin[numlines++] = newpoint(y1<<1,tempy,x1<<1);
					lin[numlines++] = newpoint(y2<<1,tempy,x2<<1);
					lin[numlines++] = newpoint(y3<<1,tempy,x3<<1);
					lin[numlines++] = newpoint(y4<<1,tempy,x4<<1);
					break;
			}
			picnum[numplanes] = sprite[i].picnum;
			shade[numplanes] = sprite[i].shade;
			planestat[numplanes] = ((sprite[i].cstat&1^1)<<2)+2;
			bxv[numplanes][0] = px[lin[numlines-4]];
			byv[numplanes][0] = py[lin[numlines-4]];
			bzv[numplanes][0] = pz[lin[numlines-4]];
			bxv[numplanes][1] = px[lin[numlines-3]];
			byv[numplanes][1] = py[lin[numlines-3]];
			bzv[numplanes][1] = pz[lin[numlines-3]];
			bxv[numplanes][2] = px[lin[numlines-1]];
			byv[numplanes][2] = py[lin[numlines-1]];
			bzv[numplanes][2] = pz[lin[numlines-1]];
			calcnormal((long)numplanes);
			drawbspbox((long)numplanes,whitecol);
			numplanes++;

				//Sprite - double sided - must do back side also
			linoff[numplanes] = numlines;
			linnum[numplanes] = 4;
			lin2[numlines+0] = numlines+1;
			lin2[numlines+1] = numlines+2;
			lin2[numlines+2] = numlines+3;
			lin2[numlines+3] = numlines+0;
			lin[numlines++] = lin[linoff[numplanes-1]+1];
			lin[numlines++] = lin[linoff[numplanes-1]+0];
			lin[numlines++] = lin[linoff[numplanes-1]+3];
			lin[numlines++] = lin[linoff[numplanes-1]+2];
			picnum[numplanes] = sprite[i].picnum;
			shade[numplanes] = sprite[i].shade;
			planestat[numplanes] = ((sprite[i].cstat&1^1)<<2)+2;
			bxv[numplanes][0] = px[lin[numlines-4]];
			byv[numplanes][0] = py[lin[numlines-4]];
			bzv[numplanes][0] = pz[lin[numlines-4]];
			bxv[numplanes][1] = px[lin[numlines-3]];
			byv[numplanes][1] = py[lin[numlines-3]];
			bzv[numplanes][1] = pz[lin[numlines-3]];
			bxv[numplanes][2] = px[lin[numlines-1]];
			byv[numplanes][2] = py[lin[numlines-1]];
			bzv[numplanes][2] = pz[lin[numlines-1]];
			calcnormal((long)numplanes);
			drawbspbox((long)numplanes,whitecol);
			numplanes++;
		}

		for(i=0;i<numplanes;i++)
			if ((tilesizx[picnum[i]] <= 0) || (tilesizy[picnum[i]] <= 0))
				picnum[i] = 0;
	}
	else
	{
		setvmode(0x3);
		printf("Wrong map version\n");
		exit(0);
	}

	close(fil);

	makebsp();

	for(k=groupoff[1];k<groupoff[2];k++)
	{
		j = insertsprite(0,0);
		spri[j].x = posx; spri[j].y = posy; spri[j].z = posz;
		spri[j].a1 = 0; spri[j].a2 = 0; spri[j].a3 = 0;
		spri[j].xvel = 0; spri[j].yvel = 0; spri[j].zvel = 0;
		spri[j].a1vel = 0;
		spri[j].a2vel = 0;
		spri[j].a3vel = (rand()&63)-32;
		spri[j].objnum = k;
		spri[j].stat = 0;
		spri[j].xscale = 64; spri[j].yscale = 64; spri[j].zscale = 64;
		spri[j].lotag = 0;
		spri[j].hitag = 0;
		spri[j].extra = 0;
	}

	for(i=MAXSPRIS-1;i>=0;i--)
		if (spri[i].statnum < MAXSTATUS)
		{
			j = spri[i].objnum;
			changespritesect((short)i,getregion(sox[j]+spri[i].x,soy[j]+spri[i].y,soz[j]+spri[i].z));
		}
}

newpoint(long x, long y, long z)
{
	long i;

	for(i=0;i<numpoints;i++)
		if (px[i] == x)
			if (py[i] == y)
				if (pz[i] == z)
					return(i);

	px[numpoints] = x;
	py[numpoints] = y;
	pz[numpoints] = z;
	numpoints++;
	return(numpoints-1);
}

saveboard(char *filename)
{
	long fil, i;

	if ((fil = open(filename,O_BINARY|O_TRUNC|O_CREAT|O_WRONLY,S_IWRITE)) == -1)
		return(-1);

	mapversion = 0x00020000;
	write(fil,&mapversion,4);
	write(fil,&posx,4);
	write(fil,&posy,4);
	write(fil,&posz,4);
	write(fil,&a1,2);
	write(fil,&a2,2);
	write(fil,&a3,2);
	write(fil,&numplanes,2);
	write(fil,&numlines,2);
	write(fil,&numpoints,2);
	write(fil,&px[0],numpoints<<2);
	write(fil,&py[0],numpoints<<2);
	write(fil,&pz[0],numpoints<<2);
	write(fil,&linnum[0],numplanes<<1);
	write(fil,&lin[0],numlines<<1);
	write(fil,&lin2[0],numlines<<1);
	write(fil,&bxv[0],numplanes*12);
	write(fil,&byv[0],numplanes*12);
	write(fil,&bzv[0],numplanes*12);
	write(fil,&shade[0],numplanes);
	write(fil,&picnum[0],numplanes<<1);
	write(fil,&planestat[0],numplanes);
	close(fil);

	return(0);
}

drawscreen()
{
	long i, j, k, index, gap, temp, templong, farthest, bad;
	long oposx, oposy, oposz, oxpan, xvect, yvect, zvect;

	if ((option[0] == 5) || (option[0] == 6))
	{
		if ((keystatus[0x2a]|keystatus[0x36]) > 0)
		{
			if (keystatus[0x1a] > 0) xpan -= 256;   //Shift [
			if (keystatus[0x1b] > 0) xpan += 256;   //Shift ]
			calcscreenconsts();
		}
		else
		{
			if (keystatus[0x1a] > 0) stereowidth += 512;   //[
			if (keystatus[0x1b] > 0) stereowidth -= 512;   //]
		}

		oposx = posx; oposy = posy; oposz = posz;
		oxpan = xpan;

		if (stereopage == 0) xvect = -stereowidth; else xvect = stereowidth;
		yvect = 0; zvect = 0;
		backrotate3d(&xvect,&yvect,&zvect);
		posx += (xvect>>8); posy += (yvect>>8); posz += (zvect>>8);

		if (stereopage == 1) xpan = -xpan;
		calcscreenconsts();

		frameplace = FP_OFF(screen)+(xdim<<1);
		if (stereopage == 1) frameplace += xdim*ydim;
		stereopage ^= 1;
	}

	if (drawmode == 3)
	{
		clearbuf(frameplace,(xdim*ydim)>>2,0L);
		//if (drawmode == 3) drawgrid();
	}
	else if (checkinside(posx,posy,posz) == 0)
	{
		clearbuf(frameplace,(xdim*ydim)>>2,0L);
	}

	for(i=numpoints;i>=0;i--)
	{
		rxi[i] = ((px[i]-posx)<<8);
		ryi[i] = ((py[i]-posy)<<8);
		rzi[i] = ((pz[i]-posz)<<8);
		rotate3d(&rxi[i],&ryi[i],&rzi[i]);
	}

	recurseit(head);

	if ((option[0] == 5) || (option[0] == 6))
	{
		posx = oposx, posy = oposy, posz = oposz;
		xpan = oxpan; calcscreenconsts();
	}
}

squareaspect(long i, long boxsiz)
{
	long nx1, ny1, nz1, nx2, ny2, nz2, nx3, ny3, nz3, nx4, ny4, nz4;
	long dist, templong;

	nx1 = bxv[i][1]-bxv[i][0];
	ny1 = byv[i][1]-byv[i][0];
	nz1 = bzv[i][1]-bzv[i][0];
	while (klabs(nx1) >= 32768) nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;
	while (klabs(ny1) >= 32768) nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;
	while (klabs(nz1) >= 32768) nx1 >>= 1, ny1 >>= 1, nz1 >>= 1;

	dist = ksqrtasm(nx1*nx1+ny1*ny1+nz1*nz1);
	if (dist >= 16)
	{
		templong = (boxsiz<<16)/dist;
		nx1 = mulscale(nx1,templong,16);
		ny1 = mulscale(ny1,templong,16);
		nz1 = mulscale(nz1,templong,16);
	}

	nx2 = bxv[i][2]-bxv[i][0];
	ny2 = byv[i][2]-byv[i][0];
	nz2 = bzv[i][2]-bzv[i][0];
	while (klabs(nx2) >= 32768) nx2 >>= 1, ny2 >>= 1, nz2 >>= 1;
	while (klabs(ny2) >= 32768) nx2 >>= 1, ny2 >>= 1, nz2 >>= 1;
	while (klabs(nz2) >= 32768) nx2 >>= 1, ny2 >>= 1, nz2 >>= 1;

	dist = ksqrtasm(nx2*nx2+ny2*ny2+nz2*nz2);
	if (dist >= 16)
	{
		templong = (boxsiz<<16)/dist;
		nx2 = mulscale(nx2,templong,16);
		ny2 = mulscale(ny2,templong,16);
		nz2 = mulscale(nz2,templong,16);
	}

		//<b?v[1]-b?v[0]> x <b?v[2]-b?v[0]>
		//normalx = y0z1-y1z0, normaly = x1z0-x0z1, normalz = x0y1-x1y0
	nx3 = ny1*nz2 - ny2*nz1;
	ny3 = nx2*nz1 - nx1*nz2;
	nz3 = nx1*ny2 - nx2*ny1;
	while (klabs(nx3) >= 32768) nx3 >>= 1, ny3 >>= 1, nz3 >>= 1;
	while (klabs(ny3) >= 32768) nx3 >>= 1, ny3 >>= 1, nz3 >>= 1;
	while (klabs(nz3) >= 32768) nx3 >>= 1, ny3 >>= 1, nz3 >>= 1;

	dist = ksqrtasm(nx3*nx3+ny3*ny3+nz3*nz3);
	if (dist >= 16)
	{
		templong = (boxsiz<<16)/dist;
		nx3 = mulscale(nx3,templong,16);
		ny3 = mulscale(ny3,templong,16);
		nz3 = mulscale(nz3,templong,16);
	}

		//<b?v[1]-b?v[0]> x <n?>
		//normalx = y0z1-y1z0, normaly = x1z0-x0z1, normalz = x0y1-x1y0
	nx4 = ny1*nz3 - ny3*nz1;
	ny4 = nx3*nz1 - nx1*nz3;
	nz4 = nx1*ny3 - nx3*ny1;
	while (klabs(nx4) >= 32768) nx4 >>= 1, ny4 >>= 1, nz4 >>= 1;
	while (klabs(ny4) >= 32768) nx4 >>= 1, ny4 >>= 1, nz4 >>= 1;
	while (klabs(nz4) >= 32768) nx4 >>= 1, ny4 >>= 1, nz4 >>= 1;

	dist = ksqrtasm(nx4*nx4+ny4*ny4+nz4*nz4);
	if (dist >= 16)
	{
		templong = (boxsiz<<16)/dist;
		nx4 = mulscale(nx4,templong,16);
		ny4 = mulscale(ny4,templong,16);
		nz4 = mulscale(nz4,templong,16);
	}

	bxv[i][1] = bxv[i][0]+nx1;
	byv[i][1] = byv[i][0]+ny1;
	bzv[i][1] = bzv[i][0]+nz1;
	bxv[i][2] = bxv[i][0]-nx4;
	byv[i][2] = byv[i][0]-ny4;
	bzv[i][2] = bzv[i][0]-nz4;
}

calcnormal(long p)
{
	long i, i0, i1, i2, x, y, z, x1, y1, z1, x2, y2, z2, nx, ny, nz;

	nx = 0; ny = 0; nz = 0;
	for(i=linoff[p]+linnum[p]-1;i>=linoff[p];i--)
	{
		i0 = i; i1 = lin2[i]; i2 = lin2[lin2[i]];
		i0 = lin[i0]; i1 = lin[i1]; i2 = lin[i2];
		x = px[i1]; x1 = px[i2]-x; x2 = px[i0]-x;
		y = py[i1]; y1 = py[i2]-y; y2 = py[i0]-y;
		z = pz[i1]; z1 = pz[i2]-z; z2 = pz[i0]-z;
		nx += y1*z2 - y2*z1;
		ny += x2*z1 - x1*z2;
		nz += x1*y2 - x2*y1;
	}
	while (klabs(nx) >= 65536) nx >>= 1, ny >>= 1, nz >>= 1;
	while (klabs(ny) >= 65536) nx >>= 1, ny >>= 1, nz >>= 1;
	while (klabs(nz) >= 65536) nx >>= 1, ny >>= 1, nz >>= 1;
	nxi[p] = nx; nyi[p] = ny; nzi[p] = nz;
}

setgamemode()
{
	long i, j, r, g, b, fil;

	setvesamode(0x110);
	outp(0x3d4,0x9); outp(0x3d5,inp(0x3d5)|128);

	/*
	if (option[0] == 0) option[0] = 1;

	if ((xdim == 320) && (ydim == 200))
		setvmode(0x13);
	else if ((xdim > 320) && (ydim > 200))
	{
		i = -1;
		if ((xdim == 640) && (ydim == 400)) i = setvesamode(0x100);
		if ((xdim == 640) && (ydim == 480)) i = setvesamode(0x101);
		if ((xdim == 800) && (ydim == 600)) i = setvesamode(0x103);
		if ((xdim == 1024) && (ydim == 768)) i = setvesamode(0x105);
		if ((xdim == 1280) && (ydim == 1024)) i = setvesamode(0x107);
		if ((xdim == 1600) && (ydim == 1200)) i = setvesamode(0x120);

		if (i == -1)
		{
			if (option[0] == 2)            //Special Tseng driver
			{
				if ((xdim == 640) && (ydim == 480)) i = setvmode(0x2e);
				if ((xdim == 800) && (ydim == 600)) i = setvmode(0x30);
				if ((xdim == 1024) && (ydim == 768)) i = setvmode(0x38);

				vesapageshift = 6;
				tsengdriver = 1;
			}
			else
			{
				printf("Vesa NOT found / mode not supported\n");
				exit(0);
			}
		}
		if ((option[0] != 5) && (option[0] != 6)) option[0] = 1;
	} */

	setxdim(xdim);

	searchx = (xdim>>1);
	searchy = (ydim>>1);

	if (paletteloaded == 0)
	{
		if ((fil = open("palette.dat",O_BINARY|O_RDWR,S_IREAD)) != -1)
		{
			numpalookups = 32;
			read(fil,&palette[0],768);

			for(i=0;i<numpalookups;i++)
				for(j=0;j<256;j++)
				{
					r = ((palette[j*3+0]*(32-i))>>6);
					g = ((palette[j*3+1]*(32-i))>>6);
					b = ((palette[j*3+2]*(32-i))>>6);
					palookup[i][j] = (r<<10)+(g<<5)+b;
				}

			whitecol = 0x7fff;
			blackcol = 0;
			close(fil);
		}

		/*
		if ((fil = open("palette.dat",O_BINARY|O_RDWR,S_IREAD)) != -1)
		{
			numpalookups = ((filelength(fil)-768)>>7);
			if ((numpalookups&1) > 0)
			{   //Transluscent palette found in palette.dat
				numpalookups = ((numpalookups-255)>>1);

				if (translucloaded == 0)
					if ((transluc = (char *)malloc(65536L)) != 0)
					{
						//fixtransluscence(FP_OFF(transluc));
						translucloaded = 1;
					}
			}
			else
			{    //No transluscent palette in palette.dat
				numpalookups >>= 1;
			}
			read(fil,&palette[0],768);
			read(fil,&palookup[0][0],(numpalookups<<8));
			if (translucloaded == 1)
			{
				for(i=0;i<255;i++)
				{
					read(fil,&transluc[(i<<8)+i+1],255-i);
					for(j=i+1;j<256;j++)
						transluc[(j<<8)+i] = transluc[(i<<8)+j];
				}
				for(i=0;i<256;i++)
					transluc[(i<<8)+i] = i;
			}

			blackcol = 0; j = 0x7fffffff;
			for(i=0;i<256;i++)
				if (palette[i+i+i]+palette[i+i+i+1]+palette[i+i+i+2] < j)
				{
					j = palette[i+i+i]+palette[i+i+i+1]+palette[i+i+i+2];
					blackcol = i;
				}

			whitecol = 0; j = 0x80000000;
			for(i=0;i<256;i++)
				if (palette[i+i+i]+palette[i+i+i+1]+palette[i+i+i+2] > j)
				{
					j = palette[i+i+i]+palette[i+i+i+1]+palette[i+i+i+2];
					whitecol = i;
				}

			close(fil);
		}*/

		paletteloaded = 1;
	}
	setpalette(brinum);

	switch(option[0])
	{
		case 0:
			koutp(0x3c4,0x4); koutp(0x3c5,0x6);
			koutp(0x3d4,0x14); koutp(0x3d5,0x0);
			koutp(0x3d4,0x17); koutp(0x3d5,0xe3);

			koutp(0x3c4,2); koutp(0x3c5,15);
			clearbuf(0xa0000,4096L+4096L+4096L+4096L,0L);

			frameplace = 0xa0000;
			//fixvlinesforchain();
			page = 0;
			break;
		case 1:
			frameplace = FP_OFF(screen)+(xdim<<1);
			clearbuf(frameplace,(xdim*ydim)>>2,0L);
			break;
		case 2:
			koutp(0x3cd,0), clearbuf(0xa0000,16000L,0L);
			koutp(0x3cd,1), clearbuf(0xa0000,16000L,0L);
			koutp(0x3cd,2), clearbuf(0xa0000,16000L,0L);
			koutp(0x3cd,3), clearbuf(0xa0000,16000L,0L);

			frameplace = 0xa0000;
			page = 0; koutp(0x3cd,page+(page<<4));
			break;
		case 3:
			koutp(0x3ce,0xf); koutp(0x3cf,0x5);
			koutp(0x3d4,0x29); koutp(0x3d5,0x85);
			koutp(0x3c4,0x6); koutp(0x3c5,0x48);
			koutp(0x3d4,0x2f); koutp(0x3d5,0);   //set mod = 256k

			koutp(0x3ce,0x9); koutp(0x3cf,0<<4); clearbuf(0xa0000,16000L,0L);
			koutp(0x3ce,0x9); koutp(0x3cf,1<<4); clearbuf(0xa0000,16000L,0L);
			koutp(0x3ce,0x9); koutp(0x3cf,2<<4); clearbuf(0xa0000,16000L,0L);
			koutp(0x3ce,0x9); koutp(0x3cf,3<<4); clearbuf(0xa0000,16000L,0L);

			frameplace = 0xa0000;
			page = 0; koutp(0x3ce,0x9); koutp(0x3cf,page<<4);
			break;
		case 4:
			koutp(0x3d4,0x38); koutp(0x3d5,0x48);           //S3 Extensions enable #1
			koutp(0x3d4,0x39); koutp(0x3d5,0xa5);           //S3 Extensions enable #2
			koutp(0x3d4,0x31); koutp(0x3d5,kinp(0x3d5)|9);   //Init for start address

			koutp(0x3d4,0x35); koutp(0x3d5,0); clearbuf(0xa0000,16000L,0L);
			koutp(0x3d4,0x35); koutp(0x3d5,1); clearbuf(0xa0000,16000L,0L);
			koutp(0x3d4,0x35); koutp(0x3d5,2); clearbuf(0xa0000,16000L,0L);
			koutp(0x3d4,0x35); koutp(0x3d5,3); clearbuf(0xa0000,16000L,0L);

			frameplace = 0xa0000;
			page = 0; koutp(0x3d4,0x35); koutp(0x3d5,page);
			break;
		case 5:
			frameplace = FP_OFF(screen)+(xdim<<1);
			clearbuf(frameplace,(xdim*ydim)>>2,0L);
			stereoinit();
			break;
		case 6:
			frameplace = FP_OFF(screen)+(xdim<<1);
			clearbuf(frameplace,(xdim*ydim)>>2,0L);
			break;
	}
}

nextpage()
{
	long i, totbytes, s1, s2;

	totbytes = 320*240*2*2;
	for(i=0;i<totbytes;i+=65536)
	{
		vesasetpage(i>>16);

		if (i+65536 <= totbytes)
			hicolorblit(frameplace+(i>>1),0xa0000,8192);
		else
			hicolorblit(frameplace+(i>>1),0xa0000,(totbytes-i)>>3);
	}
	return;

	switch(option[0])
	{
		case 0:
			koutp(0x3d4,0xc); koutp(0x3d5,page<<6);
			page = ((page+1)&3);
			break;
		case 1:
			totbytes = xdim*ydim;
			for(i=0;i<totbytes;i+=65536)
			{
				if (tsengdriver == 1) koutp(0x3cd,i>>16); else vesasetpage(i>>16);

				if (i+65536 <= totbytes)
					copybuf(frameplace+i,0xa0000,16384);
				else
					copybuf(frameplace+i,0xa0000,(totbytes-i)>>2);
			}
			break;
		case 2:
			koutp(0x3d4,0xc); koutp(0x3d5,page<<6);
			page = ((page+1)&3);
			koutp(0x3cd,page+(page<<4));
			break;
		case 3:
			koutp(0x3d4,0xc); koutp(0x3d5,page<<6);
			page = ((page+1)&3);
			koutp(0x3ce,0x9); koutp(0x3cf,page<<4);
			break;
		case 4:
			koutp(0x3d4,0xc); koutp(0x3d5,page<<6);
			page = ((page+1)&3);
			koutp(0x3d4,0x35); koutp(0x3d5,page);
			break;
		case 5:
			totbytes = xdim*ydim;
			s1 = (long)(&screen[xdim<<1]); s2 = s1+totbytes;
			stereoblit(s1,s2);
			break;
		case 6:
			totbytes = xdim*ydim;
			s1 = (long)(&screen[xdim<<1]); s2 = s1+totbytes;
			for(i=0;i<totbytes;i+=65536)
			{
				if (tsengdriver == 1) koutp(0x3cd,i>>16); else vesasetpage(i>>16);

				if (i+65536 <= totbytes)
					redblueblit(s1+i,s2+i,65536);
				else
					redblueblit(s1+i,s2+i,totbytes-i);
			}
			break;
	}
}

drawplane(long plane)
{
	long i, j, index, z, zz, z0, z1, z2, npoints, bad;

	globplane = plane;
	globstat = planestat[globplane];
	globshade = shade[globplane];

	bad = 0;
	npoints = linnum[globplane];
	i = linoff[globplane]+npoints-1;
	for(z=npoints-1;z>=0;z--)
	{
		point2[z] = lin2[i]-linoff[globplane];
		index = lin[i--];
		rx[z] = rxi[index]; ry[z] = ryi[index]; rz[z] = rzi[index];
		if (rz[z] > 0) bad = 1;
	}
	if (bad == 0) return;

	if (reversemode == 1)
	{
		for(j=npoints-1;j>=0;j--) point2[j+npoints] = point2[j];
		for(j=0;j<npoints;j++) point2[point2[j+npoints]] = j;
	}

	if ((npoints = clippoly(npoints)) <= 2L) return;

	for(z=npoints-1;z>=0;z--)
	{
		if (klabs(rz[z]) < 16) return;
		project(rx[z],ry[z],rz[z],&sx[z],&sy[z]);

		if (sx[z] < 0) sx[z] = 0;
		if (sx[z] > (xdim<<16)) sx[z] = (xdim<<16);
		if (sy[z] < 0) sy[z] = 0;
		if (sy[z] > (ydim<<16)) sy[z] = (ydim<<16);
	}

	switch(drawmode)
	{
		case 0:
			drawfasttextureplane(npoints);
			break;
		case 1:
			drawslowtextureplane(npoints);
			break;
		case 2:
			drawsolidplane(npoints);
			break;
		case 3:
			drawwireplane(npoints);
			break;
	}
}

drawline(long x1, long y1, long x2, long y2, char col)
{
	long dx, dy, x, y, xinc, yinc, d1, d2, t, p, cnt;

	col = palookup[0][col];

	dx = x2-x1; dy = y2-y1;
	if (klabs(dx) > klabs(dy))
	{
		if (x1 > x2)
		{
			t = x1, x1 = x2, x2 = t;
			t = y1, y1 = y2, y2 = t;
		}
		d1 = ((x1+32768)>>16); d2 = ((x2+32768)>>16);
		p = d1+frameplace; cnt = d2-d1;
		yinc = divscale(dy,dx,16);
		y = y1 + mulscale((32768-x1)&65535,yinc,16);
		while (cnt > 0)
		{
			drawpixel(ylookup[y>>16]+p,col);
			y += yinc; p++; cnt--;
		}
	}
	else
	{
		if (dy == 0) return;

		if (y1 > y2)
		{
			t = x1, x1 = x2, x2 = t;
			t = y1, y1 = y2, y2 = t;
		}
		d1 = ((y1+32768)>>16); d2 = ((y2+32768)>>16);
		p = ylookup[d1]+frameplace; cnt = d2-d1;
		xinc = divscale(dx,dy,16);
		x = x1 + mulscale((32768-y1)&65535,xinc,16);
		while (cnt > 0)
		{
			drawpixel(p+(x>>16),col);
			x += xinc; p += xdim;
			cnt--;
		}
	}
}

setuprotate3d(short da1, short da2, short da3)
{
	long cosa1, sina1, cosa2, sina2, cosa3, sina3, c1c3, c1s3, s1c3, s1s3;

	cosa1 = sintable[(da1+512)&2047]; sina1 = sintable[da1&2047];
	cosa2 = sintable[(da2+512)&2047]; sina2 = sintable[da2&2047];
	cosa3 = sintable[(da3+512)&2047]; sina3 = sintable[da3&2047];

	c1c3 = cosa1*cosa3; c1s3 = cosa1*sina3;
	s1c3 = sina1*cosa3; s1s3 = sina1*sina3;
	gxx = mulscale(s1s3,sina2,14) + c1c3;
	gxy = -mulscale(c1s3,sina2,14) + s1c3;
	gxz = sina3*cosa2;
	gyx = -cosa2*sina1;
	gyy = cosa2*cosa1;
	gyz = (sina2<<14);
	gzx = mulscale(s1c3,sina2,14) - c1s3;
	gzy = -mulscale(c1c3,sina2,14) - s1s3;
	gzz = cosa3*cosa2;

	/*ox = -(xdim<<7); oy = -(ydim<<7); oz = (xdim<<7);
	cornx[0] = mulscale(ox,gxx,28) + mulscale(oy,gxy,28) + mulscale(oz,gxz,28);
	corny[0] = mulscale(ox,gyx,28) + mulscale(oy,gyy,28) + mulscale(oz,gyz,28);
	cornz[0] = mulscale(ox,gzx,28) + mulscale(oy,gzy,28) + mulscale(oz,gzz,28);
	ox = (xdim<<7); oy = -(ydim<<7); oz = (xdim<<7);
	cornx[1] = mulscale(ox,gxx,28) + mulscale(oy,gxy,28) + mulscale(oz,gxz,28);
	corny[1] = mulscale(ox,gyx,28) + mulscale(oy,gyy,28) + mulscale(oz,gyz,28);
	cornz[1] = mulscale(ox,gzx,28) + mulscale(oy,gzy,28) + mulscale(oz,gzz,28);
	ox = (xdim<<7); oy = (ydim<<7); oz = (xdim<<7);
	cornx[2] = mulscale(ox,gxx,28) + mulscale(oy,gxy,28) + mulscale(oz,gxz,28);
	corny[2] = mulscale(ox,gyx,28) + mulscale(oy,gyy,28) + mulscale(oz,gyz,28);
	cornz[2] = mulscale(ox,gzx,28) + mulscale(oy,gzy,28) + mulscale(oz,gzz,28);
	ox = -(xdim<<7); oy = (ydim<<7); oz = (xdim<<7);
	cornx[3] = mulscale(ox,gxx,28) + mulscale(oy,gxy,28) + mulscale(oz,gxz,28);
	corny[3] = mulscale(ox,gyx,28) + mulscale(oy,gyy,28) + mulscale(oz,gyz,28);
	cornz[3] = mulscale(ox,gzx,28) + mulscale(oy,gzy,28) + mulscale(oz,gzz,28);*/
}

addrotate3d(short da1, short da2, short da3, long daindex)
{
	long cosa1, sina1, cosa2, sina2, cosa3, sina3, c1c3, c1s3, s1c3, s1s3;
	long fxx, fxy, fxz, fyx, fyy, fyz, fzx, fzy, fzz, ox, oy, oz;

	cosa1 = sintable[(da1+512)&2047]; sina1 = sintable[da1&2047];
	cosa2 = sintable[(da2+512)&2047]; sina2 = sintable[da2&2047];
	cosa3 = sintable[(da3+512)&2047]; sina3 = sintable[da3&2047];

	c1c3 = cosa1*cosa3; c1s3 = cosa1*sina3;
	s1c3 = sina1*cosa3; s1s3 = sina1*sina3;
	fxx = mulscale(s1s3,sina2,14) + c1c3;
	fxy = -mulscale(c1s3,sina2,14) + s1c3;
	fxz = sina3*cosa2;
	fyx = -cosa2*sina1;
	fyy = cosa2*cosa1;
	fyz = (sina2<<14);
	fzx = mulscale(s1c3,sina2,14) - c1s3;
	fzy = -mulscale(c1c3,sina2,14) - s1s3;
	fzz = cosa3*cosa2;

	if (daindex >= 0)
	{
		if (spri[daindex].xscale != 64)
		{
			fxx = mulscale(fxx,spri[daindex].xscale,6);
			fyx = mulscale(fyx,spri[daindex].xscale,6);
			fzx = mulscale(fzx,spri[daindex].xscale,6);
		}
		if (spri[daindex].yscale != 64)
		{
			fxy = mulscale(fxy,spri[daindex].yscale,6);
			fyy = mulscale(fyy,spri[daindex].yscale,6);
			fzy = mulscale(fzy,spri[daindex].yscale,6);
		}
		if (spri[daindex].zscale != 64)
		{
			fxz = mulscale(fxz,spri[daindex].zscale,6);
			fyz = mulscale(fyz,spri[daindex].zscale,6);
			fzz = mulscale(fzz,spri[daindex].zscale,6);
		}
	}

	ox = gxx; oy = gyx; oz = gzx;
	gxx = triplemulscale28(ox,fxx,oy,fyx,oz,fzx);
	gyx = triplemulscale28(ox,fxy,oy,fyy,oz,fzy);
	gzx = triplemulscale28(ox,fxz,oy,fyz,oz,fzz);
	ox = gxy; oy = gyy; oz = gzy;
	gxy = triplemulscale28(ox,fxx,oy,fyx,oz,fzx);
	gyy = triplemulscale28(ox,fxy,oy,fyy,oz,fzy);
	gzy = triplemulscale28(ox,fxz,oy,fyz,oz,fzz);
	ox = gxz; oy = gyz; oz = gzz;
	gxz = triplemulscale28(ox,fxx,oy,fyx,oz,fzx);
	gyz = triplemulscale28(ox,fxy,oy,fyy,oz,fzy);
	gzz = triplemulscale28(ox,fxz,oy,fyz,oz,fzz);
}

rotate3d (long *x, long *y, long *z)
{
	long ox, oy, oz;

	ox = *x; oy = *y; oz = *z;
	*x = triplemulscale28(ox,gxx,oy,gyx,oz,gzx);
	*y = triplemulscale28(ox,gxy,oy,gyy,oz,gzy);
	*z = triplemulscale28(ox,gxz,oy,gyz,oz,gzz);
}

backrotate3d (long *x, long *y, long *z)
{
	long ox, oy, oz;

	ox = *x; oy = *y; oz = *z;
	*x = triplemulscale28(ox,gxx,oy,gxy,oz,gxz);
	*y = triplemulscale28(ox,gyx,oy,gyy,oz,gyz);
	*z = triplemulscale28(ox,gzx,oy,gzy,oz,gzz);
}

project (long x, long y, long z, long *dasx, long *dasy)
{
	long t;

	t = divscale(xreszoom,z,8);
	*dasx = mulscale(x,t,8) + xdimup15;
	*dasy = mulscale(y,t,8) + ydimup15;
}

clippoly (long npoints)
{
	long z, zz, s1, s2, t, npoints2, start2, z1, z2, z3, z4;

	npoints2 = 0; start2 = 0; z = 0; splitcnt = 0;
	do
	{
		s2 = -mulscale(rz[z],xdimscale1,16)-rx[z];
		do
		{
			zz = point2[z]; point2[z] = -1;
			s1 = s2; s2 = -mulscale(rz[zz],xdimscale1,16)-rx[zz];
			if (s1 < 0)
			{
				rx2[npoints2] = rx[z]; ry2[npoints2] = ry[z]; rz2[npoints2] = rz[z];
				point22[npoints2] = npoints2+1; npoints2++;
			}
			if ((s1^s2) < 0)
			{
				if (s1 >= 0)
				{
					t = divscale(s1,s1-s2,30);
					rx2[npoints2] = rx[z]+mulscale(rx[zz]-rx[z],t,30);
					ry2[npoints2] = ry[z]+mulscale(ry[zz]-ry[z],t,30);
					rz2[npoints2] = rz[z]+mulscale(rz[zz]-rz[z],t,30);
				}
				else
				{
					t = divscale(s2,s2-s1,30);
					rx2[npoints2] = rx[zz]+mulscale(rx[z]-rx[zz],t,30);
					ry2[npoints2] = ry[zz]+mulscale(ry[z]-ry[zz],t,30);
					rz2[npoints2] = rz[zz]+mulscale(rz[z]-rz[zz],t,30);
					split[splitcnt++] = npoints2;
				}
				point22[npoints2] = npoints2+1;
				npoints2++;
			}
			z = zz;
		} while (point2[z] >= 0);

		if (npoints2 >= start2+3)
			point22[npoints2-1] = start2, start2 = npoints2;
		else
			npoints2 = start2;

		z = 1;
		while ((z < npoints) && (point2[z] < 0)) z++;
	} while (z < npoints);
	if (npoints2 <= 2) return(0);

	for(z=1;z<splitcnt;z++)
		for(zz=0;zz<z;zz++)
		{
			z1 = split[z]; z2 = point22[z1]; z3 = split[zz]; z4 = point22[z3];
			s1  = klabs(rx2[z1]-rx2[z2])+klabs(ry2[z1]-ry2[z2])+klabs(rz2[z1]-rz2[z2]);
			s1 += klabs(rx2[z3]-rx2[z4])+klabs(ry2[z3]-ry2[z4])+klabs(rz2[z3]-rz2[z4]);
			s2  = klabs(rx2[z1]-rx2[z4])+klabs(ry2[z1]-ry2[z4])+klabs(rz2[z1]-rz2[z4]);
			s2 += klabs(rx2[z3]-rx2[z2])+klabs(ry2[z3]-ry2[z2])+klabs(rz2[z3]-rz2[z2]);
			if (s2 < s1)
				{ t = point22[split[z]]; point22[split[z]] = point22[split[zz]]; point22[split[zz]] = t; }
		}


	npoints = 0; start2 = 0; z = 0; splitcnt = 0;
	do
	{
		s2 = rx2[z]-mulscale(rz2[z],xdimscale2,16);
		do
		{
			zz = point22[z]; point22[z] = -1;
			s1 = s2; s2 = rx2[zz]-mulscale(rz2[zz],xdimscale2,16);
			if (s1 < 0)
			{
				rx[npoints] = rx2[z]; ry[npoints] = ry2[z]; rz[npoints] = rz2[z];
				point2[npoints] = npoints+1; npoints++;
			}
			if ((s1^s2) < 0)
			{
				if (s1 >= 0)
				{
					t = divscale(s1,s1-s2,30);
					rx[npoints] = rx2[z]+mulscale(rx2[zz]-rx2[z],t,30);
					ry[npoints] = ry2[z]+mulscale(ry2[zz]-ry2[z],t,30);
					rz[npoints] = rz2[z]+mulscale(rz2[zz]-rz2[z],t,30);
				}
				else
				{
					t = divscale(s2,s2-s1,30);
					rx[npoints] = rx2[zz]+mulscale(rx2[z]-rx2[zz],t,30);
					ry[npoints] = ry2[zz]+mulscale(ry2[z]-ry2[zz],t,30);
					rz[npoints] = rz2[zz]+mulscale(rz2[z]-rz2[zz],t,30);
					split[splitcnt++] = npoints;
				}
				point2[npoints] = npoints+1;
				npoints++;
			}
			z = zz;
		} while (point22[z] >= 0);

		if (npoints >= start2+3)
			point2[npoints-1] = start2, start2 = npoints;
		else
			npoints = start2;

		z = 1;
		while ((z < npoints2) && (point22[z] < 0)) z++;
	} while (z < npoints2);
	if (npoints <= 2) return(0);

	for(z=1;z<splitcnt;z++)
		for(zz=0;zz<z;zz++)
		{
			z1 = split[z]; z2 = point2[z1]; z3 = split[zz]; z4 = point2[z3];
			s1  = klabs(rx[z1]-rx[z2])+klabs(ry[z1]-ry[z2])+klabs(rz[z1]-rz[z2]);
			s1 += klabs(rx[z3]-rx[z4])+klabs(ry[z3]-ry[z4])+klabs(rz[z3]-rz[z4]);
			s2  = klabs(rx[z1]-rx[z4])+klabs(ry[z1]-ry[z4])+klabs(rz[z1]-rz[z4]);
			s2 += klabs(rx[z3]-rx[z2])+klabs(ry[z3]-ry[z2])+klabs(rz[z3]-rz[z2]);
			if (s2 < s1)
				{ t = point2[split[z]]; point2[split[z]] = point2[split[zz]]; point2[split[zz]] = t; }
		}


	npoints2 = 0; start2 = 0; z = 0; splitcnt = 0;
	do
	{
		s2 = -mulscale(rz[z],ydimscale1,16)-ry[z];
		do
		{
			zz = point2[z]; point2[z] = -1;
			s1 = s2; s2 = -mulscale(rz[zz],ydimscale1,16)-ry[zz];
			if (s1 < 0)
			{
				rx2[npoints2] = rx[z]; ry2[npoints2] = ry[z]; rz2[npoints2] = rz[z];
				point22[npoints2] = npoints2+1; npoints2++;
			}
			if ((s1^s2) < 0)
			{
				if (s1 >= 0)
				{
					t = divscale(s1,s1-s2,30);
					rx2[npoints2] = rx[z]+mulscale(rx[zz]-rx[z],t,30);
					ry2[npoints2] = ry[z]+mulscale(ry[zz]-ry[z],t,30);
					rz2[npoints2] = rz[z]+mulscale(rz[zz]-rz[z],t,30);
				}
				else
				{
					t = divscale(s2,s2-s1,30);
					rx2[npoints2] = rx[zz]+mulscale(rx[z]-rx[zz],t,30);
					ry2[npoints2] = ry[zz]+mulscale(ry[z]-ry[zz],t,30);
					rz2[npoints2] = rz[zz]+mulscale(rz[z]-rz[zz],t,30);
					split[splitcnt++] = npoints2;
				}
				point22[npoints2] = npoints2+1;
				npoints2++;
			}
			z = zz;
		} while (point2[z] >= 0);

		if (npoints2 >= start2+3)
			point22[npoints2-1] = start2, start2 = npoints2;
		else
			npoints2 = start2;

		z = 1;
		while ((z < npoints) && (point2[z] < 0)) z++;
	} while (z < npoints);
	if (npoints2 <= 2) return(0);

	for(z=1;z<splitcnt;z++)
		for(zz=0;zz<z;zz++)
		{
			z1 = split[z]; z2 = point22[z1]; z3 = split[zz]; z4 = point22[z3];
			s1  = klabs(rx2[z1]-rx2[z2])+klabs(ry2[z1]-ry2[z2])+klabs(rz2[z1]-rz2[z2]);
			s1 += klabs(rx2[z3]-rx2[z4])+klabs(ry2[z3]-ry2[z4])+klabs(rz2[z3]-rz2[z4]);
			s2  = klabs(rx2[z1]-rx2[z4])+klabs(ry2[z1]-ry2[z4])+klabs(rz2[z1]-rz2[z4]);
			s2 += klabs(rx2[z3]-rx2[z2])+klabs(ry2[z3]-ry2[z2])+klabs(rz2[z3]-rz2[z2]);
			if (s2 < s1)
				{ t = point22[split[z]]; point22[split[z]] = point22[split[zz]]; point22[split[zz]] = t; }
		}


	npoints = 0; start2 = 0; z = 0; splitcnt = 0;
	do
	{
		s2 = ry2[z]-mulscale(rz2[z],ydimscale2,16);
		do
		{
			zz = point22[z]; point22[z] = -1;
			s1 = s2; s2 = ry2[zz]-mulscale(rz2[zz],ydimscale2,16);
			if (s1 < 0)
			{
				rx[npoints] = rx2[z]; ry[npoints] = ry2[z]; rz[npoints] = rz2[z];
				point2[npoints] = npoints+1; npoints++;
			}
			if ((s1^s2) < 0)
			{
				if (s1 >= 0)
				{
					t = divscale(s1,s1-s2,30);
					rx[npoints] = rx2[z]+mulscale(rx2[zz]-rx2[z],t,30);
					ry[npoints] = ry2[z]+mulscale(ry2[zz]-ry2[z],t,30);
					rz[npoints] = rz2[z]+mulscale(rz2[zz]-rz2[z],t,30);
				}
				else
				{
					t = divscale(s2,s2-s1,30);
					rx[npoints] = rx2[zz]+mulscale(rx2[z]-rx2[zz],t,30);
					ry[npoints] = ry2[zz]+mulscale(ry2[z]-ry2[zz],t,30);
					rz[npoints] = rz2[zz]+mulscale(rz2[z]-rz2[zz],t,30);
					split[splitcnt++] = npoints;
				}
				point2[npoints] = npoints+1;
				npoints++;
			}
			z = zz;
		} while (point22[z] >= 0);

		if (npoints >= start2+3)
			point2[npoints-1] = start2, start2 = npoints;
		else
			npoints = start2;

		z = 1;
		while ((z < npoints2) && (point22[z] < 0)) z++;
	} while (z < npoints2);
	if (npoints <= 2) return(0);

	for(z=1;z<splitcnt;z++)
		for(zz=0;zz<z;zz++)
		{
			z1 = split[z]; z2 = point2[z1]; z3 = split[zz]; z4 = point2[z3];
			s1  = klabs(rx[z1]-rx[z2])+klabs(ry[z1]-ry[z2])+klabs(rz[z1]-rz[z2]);
			s1 += klabs(rx[z3]-rx[z4])+klabs(ry[z3]-ry[z4])+klabs(rz[z3]-rz[z4]);
			s2  = klabs(rx[z1]-rx[z4])+klabs(ry[z1]-ry[z4])+klabs(rz[z1]-rz[z4]);
			s2 += klabs(rx[z3]-rx[z2])+klabs(ry[z3]-ry[z2])+klabs(rz[z3]-rz[z2]);
			if (s2 < s1)
				{ t = point2[split[z]]; point2[split[z]] = point2[split[zz]]; point2[split[zz]] = t; }
		}

	return(npoints);
}

clipline (long *x1, long *y1, long *z1, long *x2, long *y2, long *z2)
{
	long t, templong;

	if ((*x1 < -*z1) != (*x2 < -*z2))
	{
		if (*x2 < -*z2)
		{
			templong = *x1; *x1 = *x2; *x2 = templong;
			templong = *y1; *y1 = *y2; *y2 = templong;
			templong = *z1; *z1 = *z2; *z2 = templong;
		}
		t = divscale(*x1+*z1,*z1-*z2+*x1-*x2,16);
		*x1 = *x1+mulscale(t,*x2-*x1,16);
		*y1 = *y1+mulscale(t,*y2-*y1,16);
		*z1 = *z1+mulscale(t,*z2-*z1,16);
	}
	else if (*x1 < -*z1)
		return(0);
	if ((*x1 > *z1) != (*x2 > *z2))
	{
		if (*x1 > *z1)
		{
			templong = *x1; *x1 = *x2; *x2 = templong;
			templong = *y1; *y1 = *y2; *y2 = templong;
			templong = *z1; *z1 = *z2; *z2 = templong;
		}
		t = divscale(*x1-*z1,*z2-*z1+*x1-*x2,16);
		*x2 = *x1+mulscale(t,*x2-*x1,16);
		*y2 = *y1+mulscale(t,*y2-*y1,16);
		*z2 = *z1+mulscale(t,*z2-*z1,16);
	}
	else if (*x1 > *z1)
		return(0);

	if ((*y1 < mulscale(-*z1,ydimscale,16)) != (*y2 < mulscale(-*z2,ydimscale,16)))
	{
		if (*y2 < mulscale(-*z2,ydimscale,16))
		{
			templong = *x1; *x1 = *x2; *x2 = templong;
			templong = *y1; *y1 = *y2; *y2 = templong;
			templong = *z1; *z1 = *z2; *z2 = templong;
		}
		t = divscale(*y1+mulscale(*z1,ydimscale,16),mulscale(*z1-*z2,ydimscale,16)+*y1-*y2,16);
		*x1 = *x1+mulscale(t,*x2-*x1,16);
		*y1 = *y1+mulscale(t,*y2-*y1,16);
		*z1 = *z1+mulscale(t,*z2-*z1,16);
	}
	else if (*y1 < mulscale(-*z1,ydimscale,16))
		return(0);

	if ((*y1 > mulscale(*z1,ydimscale,16)) != (*y2 > mulscale(*z2,ydimscale,16)))
	{
		if (*y1 > mulscale(*z1,ydimscale,16))
		{
			templong = *x1; *x1 = *x2; *x2 = templong;
			templong = *y1; *y1 = *y2; *y2 = templong;
			templong = *z1; *z1 = *z2; *z2 = templong;
		}
		t = divscale(*y1-mulscale(*z1,ydimscale,16),mulscale(*z2-*z1,ydimscale,16)+*y1-*y2,16);
		*x2 = *x1+mulscale(t,*x2-*x1,16);
		*y2 = *y1+mulscale(t,*y2-*y1,16);
		*z2 = *z1+mulscale(t,*z2-*z1,16);
	}
	else if (*y1 > mulscale(*z1,ydimscale,16))
		return(0);

	return(1);
}

setupgetreadpos()
{
	long x0, y0, z0, x1, y1, z1, x2, y2, z2;
	long *ptr, offs;

	offs = ((globplane+globplane+globplane)<<2);
	ptr = (long *)(FP_OFF(bxv)+offs); x0 = *ptr++; x1 = *ptr++; x2 = *ptr++;
	ptr = (long *)(FP_OFF(byv)+offs); y0 = *ptr++; y1 = *ptr++; y2 = *ptr++;
	ptr = (long *)(FP_OFF(bzv)+offs); z0 = *ptr++; z1 = *ptr++; z2 = *ptr++;
	if ((globstat&1) == 0)
	{
		x0 -= posx, y0 -= posy, z0 -= posz;
		x1 -= posx, y1 -= posy, z1 -= posz;
		x2 -= posx, y2 -= posy, z2 -= posz;
	}
	x0 <<= 8; y0 <<= 8; z0 <<= 8;
	x1 <<= 8; y1 <<= 8; z1 <<= 8;
	x2 <<= 8; y2 <<= 8; z2 <<= 8;
	rotate3d(&x0,&y0,&z0);
	rotate3d(&x1,&y1,&z1);
	rotate3d(&x2,&y2,&z2);

	//(x1-x0)*t + (x2-x0)*u + xv*(-v) = -x0
	//(y1-y0)*t + (y2-y0)*u + yv*(-v) = -y0
	//(z1-z0)*t + (z2-z0)*u + zv*(-v) = -z0

	x1 -= x0; x2 -= x0;
	y1 -= y0; y2 -= y0;
	z1 -= z0; z2 -= z0;

	getdxinc = doublemulscale(y1,z2,-y2,z1,24);
	getuxinc = doublemulscale(y2,z0,-y0,z2,24);
	getvxinc = doublemulscale(y0,z1,-y1,z0,24);
	getdyinc = doublemulscale(x2,z1,-x1,z2,24);
	getuyinc = doublemulscale(x0,z2,-x2,z0,24);
	getvyinc = doublemulscale(x1,z0,-x0,z1,24);

	getd = doublemulscale(x1,y2,-x2,y1,24);
	getu = doublemulscale(x2,y0,-x0,y2,24);
	getv = doublemulscale(x0,y1,-x1,y0,24);
	gett = mulscale(x0,getdxinc,16) + mulscale(x1,getuxinc,16) + mulscale(x2,getvxinc,16);

	getd = mulscale(getd,xreszoom,16);
	getu = mulscale(getu,xreszoom,16);
	getv = mulscale(getv,xreszoom,16);
	gett = mulscale(gett,xreszoom,28);

	globdax = xdimup15; globday = ydimup15;
}

getreadpos(long scrx, long scry, long *bx, long *by)
{
	long thed, thet, theu;

	scrx -= globdax; scry -= globday;
	thed = getd + doublemulscale(getdxinc,scrx,getdyinc,scry,16);
	thet = getu + doublemulscale(getuxinc,scrx,getuyinc,scry,16);
	theu = getv + doublemulscale(getvxinc,scrx,getvyinc,scry,16);

	if (thed <= 0) return;
	if ((klabs(thet)>>5) >= thed) return;
	if ((klabs(theu)>>5) >= thed) return;

	*bx = divscale(thet,thed,26);
	*by = divscale(theu,thed,26);
}

ksqrt(long num)
{
	return(ksqrtasm(num));
}

calcksqrt(long num)
{
	long root, temp;

	if (num < 0)
		return(-1);
	root = 128;
	do
	{
		temp = root;
		root = ((root+(num/root))>>1);
	}
	while (klabs(temp-root) > 1);
	return(root);
}

drawfasttextureplane(long npoints)
{
	long i, ii, iii, z, zz, zx, zzx, x, y, x1, y1, x2, y2, dapicnum;
	long j, gap, templong, splitnum, bad, b1, b2, dahplc, g1, g2;
	long lastz, nx1, ny1, nx2, ny2, dax1, dax2, lx, rx, yplc, yinc;
	long dabunches, s, startz, *cyptr;

	setupgetreadpos();

	dapicnum = picnum[globplane]; dapicnum += animateoffs(dapicnum);
	if (waloff[dapicnum] < 0) loadtile(dapicnum);

	if (klabs(getdxinc) <= klabs(getdyinc))
	{
		if (getdxinc == 0)
			globm = 0;
		else
		{
			globm = divscale(getdxinc,getdyinc,16);   //d& = y& + mulscale&(x&, m&, 16)
			if (klabs(globm) < 64) globm = 0;
			if (globm > 65536-128) globm = 65536;
			if (globm < -65536+128) globm = -65536;
		}

		z = picsiz[picnum[globplane]];
		if (globm == 0)
		{
			if ((globstat&2) == 0)
				setuphline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
			else
				msetuphline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
		}
		else
		{
			if ((globstat&2) == 0)
				setupdline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
			else
				msetupdline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
		}

		globuadd = getuxinc-mulscale(globm,getuyinc,16);
		globvadd = getvxinc-mulscale(globm,getvyinc,16);
		globdax >>= 16;

		for(z=npoints-1;z>=0;z--)
			sx[z] += 32768, sy[z] += 32768, got[z] = 0;
		numbunches = 0; z = 0;
		do
		{
			dabunches = numbunches;

			while (sx[point2[z]] == sx[z]) z = point2[z];
			s = (sx[point2[z]]<sx[z]);

			startz = z;
			do
			{
				zz = point2[z]; got[z] = 1;
				if (sx[z] != sx[zz])
					if ((sx[zz]>sx[z]) == s)
						bunchstart[numbunches++] = z, s ^= 1;
				z = zz;
			} while (z != startz);

			if (((dabunches^numbunches)&1) > 0)
				bunchstart[numbunches++] = startz;

			for(zx=dabunches;zx<numbunches-1;zx++)
				bunchend[zx] = bunchstart[zx+1];
			bunchend[numbunches-1] = bunchstart[dabunches];

			z = npoints-1;
			while ((z > 0) && (got[z] == 1)) z--;
		} while (z > 0);

			//Sort bunches by x's
		gap = numbunches;
		do
		{
			gap >>= 1;
			for(z=0;z<numbunches-gap;z++)
			{
				zz = z;
				while (sx[bunchstart[zz]] > sx[bunchstart[zz+gap]])
				{
					templong = bunchstart[zz]; bunchstart[zz] = bunchstart[zz+gap]; bunchstart[zz+gap] = templong;
					templong = bunchend[zz]; bunchend[zz] = bunchend[zz+gap]; bunchend[zz+gap] = templong;
					zz -= gap; if (zz < 0) break;
				}
			}
		} while (gap > 1);

		lx = (sx[bunchstart[0]]>>16); if (lx < 0) lx = 0;
		rx = (sx[bunchstart[numbunches-1]]>>16); if (rx > xdim) rx = xdim;
		if (lx >= rx) return;

		//j = -lx*globm;
		//for(i=lx;i<rx;i++) { hplc[i] = (j>>16); j -= globm; }
		qinterpolatedown16((long)(&hplc[lx]),rx-lx,-lx*globm,-globm);

		if (globm != 0)
		{
			if (globm < 0) i = (xdim<<1); else i = -(xdim<<1);
			for(x=lx;x<rx;x++)
			{
				haddtable[x] = 2;
				if (hplc[x] != hplc[x+1]) haddtable[x] += i;
			}
			haddtable[rx] = 0;
		}

			//Sort bunches
		snum = 0;
		for(z=0;z<numbunches;z++)
		{
			if (sx[bunchend[z]] > sx[bunchstart[z]])
			{
				splitnum = snum; bad = 0;
				while ((splitnum > 0) && (bad == 0))
				{
					splitnum--;
					cyy[splitnum+2] = cyy[splitnum];
					zx = slist[splitnum]; slist[splitnum+2] = zx;
					b1 = bunchstart[zx]; b2 = bunchend[zx];
					zx = bunchstart[z]; x = sx[zx]; y = sy[zx];
					if ((sx[b1] <= x) == (x < sx[b2]))
					{
						i = b1; nx2 = sx[i];
						while (i != b2)
						{
							ii = point2[i];
							nx1 = nx2; nx2 = sx[ii];
							if ((sx[i] <= x) == (x < sx[ii]))
							{
								ny1 = sy[i]; ny2 = sy[ii];  //compare line(i%,ii%) with point(x&,y&)
								if (nx2 > nx1)
								{
									if (mulscale(ny2-ny1,x-nx1,22) <= mulscale(y-ny1,nx2-nx1,22))
										{ splitnum++; bad = 1; break; }
								}
								else
								{
									if (mulscale(ny1-ny2,x-nx2,22) <= mulscale(y-ny2,nx1-nx2,22))
										{ splitnum++; bad = 1; break; }
								}
							}
							i = ii;
						}
					}
				}
				snum += 2;

				lastz = 0;
				while (bunchend[lastz] != bunchstart[z]) lastz++;

				ii = bunchstart[z];
				i = bunchstart[lastz];
				while (point2[i] != ii) i = point2[i];
				iii = point2[ii];

				cyy[splitnum+0] = (sy[ii]>>16)-hplc[sx[ii]>>16];
				cyy[splitnum+1] = cyy[splitnum+0];

				if (mulscale(sy[iii]-sy[ii],sx[i]-sx[ii],22) <= mulscale(sy[i]-sy[ii],sx[iii]-sx[ii],22))
				{
					slist[splitnum+0] = z;
					slist[splitnum+1] = lastz;
					cyy[splitnum+1]--;
				}
				else
				{
					slist[splitnum+0] = lastz;
					slist[splitnum+1] = z;
					cyy[splitnum+0]--;
				}
			}
			else
			{
				zx = 0; while (slist[zx] != z) zx++;
				zzx = 0; while (bunchend[slist[zzx]] != bunchstart[z]) zzx++;

				if (zx < zzx)
				{
					for(y=cyy[zx]+1;y<cyy[zzx];y++)
						lastx[y+xdim] = x;
				}
				else
				{
					for(y=cyy[zzx];y<=cyy[zx];y++)
						diaghline(x-1,y);
				}

				if (z == numbunches-1) break;

				if (zzx >= zx) zzx--;
				while (zx < snum)
					slist[zx] = slist[zx+1], cyy[zx] = cyy[zx+1], zx++;
				while (zzx < snum)
					slist[zzx] = slist[zzx+1], cyy[zzx] = cyy[zzx+1], zzx++;
				snum -= 2;
			}

			lx = (sx[bunchstart[z+0]]>>16);
			rx = (sx[bunchstart[z+1]]>>16);
			cyptr = (long *)(FP_OFF(cy));
			for (zz=0;zz<snum;zz++)
			{
				i = bunchstart[slist[zz]];
				do
				{
					ii = point2[i];

					x1 = sx[i]; dax1 = (x1>>16);
					x2 = sx[ii]; dax2 = (x2>>16);
					if (dax1 != dax2)
					{
						y1 = sy[i]; y2 = sy[ii];
						if (dax1 > dax2)
						{
							templong = x1; x1 = x2; x2 = templong;
							templong = y1; y1 = y2; y2 = templong;
							templong = dax1; dax1 = dax2; dax2 = templong;
						}
						if ((dax1 < rx) && (lx < dax2))
						{
							yplc = y1-((zz&1)<<16);
							if (dax2 > dax1+1)
							{
								yinc = divscale(y2-y1,x2-x1,16);
								yplc += mulscale((-x1)&65535,yinc,16);
							}
							if (dax1 < lx) yplc += (lx-dax1)*yinc, dax1 = lx;
							if (dax2 > rx) dax2 = rx;

							if (dax2 > dax1)
								qinterpolatedown16(((long)cyptr)+(dax1<<2),dax2-dax1,yplc,yinc);

							//dax1 += zzoffs; dax2 += zzoffs;
							//for(x=dax1;x<dax2;x++)
							//{
							//   cy[x] = (yplc>>16);
							//   yplc += yinc;
							//}
						}
					}

					i = ii;
				} while (i != bunchend[slist[zz]]);
				cyptr += MAXXDIM;
			}

			if (snum == 2)
			{
				cyptr = (long *)(FP_OFF(cy)+(lx<<2));
				y1 = cyy[0]; y2 = cyy[1];
				for(x=lx;x<rx;x++)
				{
					dahplc = hplc[x];
					g1 = cyptr[0]-dahplc; g2 = cyptr[MAXXDIM]-dahplc;
					cyptr++;

					while (g1 < y1) { y1--; lastx[y1+xdim] = x; }
					while (g1 > y1) { diaghline(x-1,y1); y1++; }
					while (g2 > y2) { y2++; lastx[y2+xdim] = x; }
					while (g2 < y2) { diaghline(x-1,y2); y2--; }
				}
				cyy[0] = y1; cyy[1] = y2;
			}
			else if (snum > 2)
			{
				for(x=lx;x<rx;x++)
				{
					dahplc = hplc[x];
					cyptr = (long *)(FP_OFF(cy)+(x<<2));
					for(zz=0;zz<snum;zz+=2)
					{
						g1 = cyptr[0]-dahplc; g2 = cyptr[MAXXDIM]-dahplc;
						cyptr += (MAXXDIM<<1);
						y1 = cyy[zz]; y2 = cyy[zz+1];

						while (g1 < y1) { y1--; lastx[y1+xdim] = x; }
						while (g1 > y1) { diaghline(x-1,y1); y1++; }
						while (g2 > y2) { y2++; lastx[y2+xdim] = x; }
						while (g2 < y2) { diaghline(x-1,y2); y2--; }

						cyy[zz] = y1; cyy[zz+1] = y2;
					}
				}
			}
		}
	}
	else
	{
		if (getdyinc == 0)
			globm = 0;
		else
		{
			globm = divscale(getdyinc,getdxinc,16);
			if (klabs(globm) < 64) globm = 0;
			if (globm > 65536-128) globm = 65536;
			if (globm < -65536+128) globm = -65536;
		}

		z = picsiz[picnum[globplane]];
		if (globm == 0)
		{
			if ((globstat&2) == 0)
				setupvline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
			else
				msetupvline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
		}
		else
		{
			if ((globstat&2) == 0)
				setupdline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
			else
				msetupdline(waloff[dapicnum],(256<<(z&15))+(1<<(z>>4))-257,((z&15)<<8)+(z>>4));
		}

		globuadd = getuyinc-mulscale(globm,getuxinc,16);
		globvadd = getvyinc-mulscale(globm,getvxinc,16);
		globday >>= 16;

		for(z=npoints;z>=0;z--)
			sx[z] += 32768, sy[z] += 32768, got[z] = 0;
		numbunches = 0; z = 0;
		do
		{
			dabunches = numbunches;

			while (sy[point2[z]] == sy[z]) z = point2[z];
			s = (sy[point2[z]]<sy[z]);

			startz = z;
			do
			{
				zz = point2[z]; got[z] = 1;
				if (sy[z] != sy[zz])
					if ((sy[zz]>sy[z]) == s)
						bunchstart[numbunches++] = z, s ^= 1;
				z = zz;
			} while (z != startz);

			if (((dabunches^numbunches)&1) > 0)
				bunchstart[numbunches++] = startz;

			for(zx=dabunches;zx<numbunches-1;zx++)
				bunchend[zx] = bunchstart[zx+1];
			bunchend[numbunches-1] = bunchstart[dabunches];

			z = npoints-1;
			while ((z > 0) && (got[z] == 1)) z--;
		} while (z > 0);

			//Sort bunches by x's
		gap = numbunches;
		do
		{
			gap >>= 1;
			for(z=0;z<numbunches-gap;z++)
			{
				zz = z;
				while (sy[bunchstart[zz]] > sy[bunchstart[zz+gap]])
				{
					templong = bunchstart[zz]; bunchstart[zz] = bunchstart[zz+gap]; bunchstart[zz+gap] = templong;
					templong = bunchend[zz]; bunchend[zz] = bunchend[zz+gap]; bunchend[zz+gap] = templong;
					zz -= gap; if (zz < 0) break;
				}
			}
		} while (gap > 1);

		lx = (sy[bunchstart[0]]>>16); if (lx < 0) lx = 0;
		rx = (sy[bunchstart[numbunches-1]]>>16); if (rx > ydim) rx = ydim;
		if (lx >= rx) return;

		//j = -lx*globm;
		//for(i=lx;i<rx;i++) { hplc[i] = (j>>16); j -= globm; }
		qinterpolatedown16((long)(&hplc[lx]),rx-lx,-lx*globm,-globm);

		if (globm != 0)
		{
			if (globm < 0) i = 2; else i = -2;
			for(x=lx;x<rx;x++)
			{
				haddtable[x] = (xdim<<1);
				if (hplc[x] != hplc[x+1]) haddtable[x] += i;
			}
			haddtable[rx] = 0;
		}

			//Sort bunches
		snum = 0;
		for(z=0;z<numbunches;z++)
		{
			if (sy[bunchend[z]] > sy[bunchstart[z]])
			{
				splitnum = snum; bad = 0;
				while ((splitnum > 0) && (bad == 0))
				{
					splitnum--;
					cyy[splitnum+2] = cyy[splitnum];
					zx = slist[splitnum]; slist[splitnum+2] = zx;
					b1 = bunchstart[zx]; b2 = bunchend[zx];
					zx = bunchstart[z]; x = sy[zx]; y = sx[zx];
					if ((sy[b1] <= x) == (x < sy[b2]))
					{
						i = b1; nx2 = sy[i];
						while (i != b2)
						{
							ii = point2[i];
							nx1 = nx2; nx2 = sy[ii];
							if ((sy[i] <= x) == (x < sy[ii]))
							{
								ny1 = sx[i]; ny2 = sx[ii];  //compare line(i%,ii%) with point(x&,y&)
								if (nx2 > nx1)
								{
									if (mulscale(ny2-ny1,x-nx1,22) <= mulscale(y-ny1,nx2-nx1,22))
										{ splitnum++; bad = 1; break; }
								}
								else
								{
									if (mulscale(ny1-ny2,x-nx2,22) <= mulscale(y-ny2,nx1-nx2,22))
										{ splitnum++; bad = 1; break; }
								}
							}
							i = ii;
						}
					}
				}
				snum += 2;

				lastz = 0;
				while (bunchend[lastz] != bunchstart[z]) lastz++;

				ii = bunchstart[z];
				i = bunchstart[lastz];
				while (point2[i] != ii) i = point2[i];
				iii = point2[ii];

				cyy[splitnum+0] = (sx[ii]>>16)-hplc[sy[ii]>>16];
				cyy[splitnum+1] = cyy[splitnum+0];

				if (mulscale(sx[iii]-sx[ii],sy[i]-sy[ii],22) <= mulscale(sx[i]-sx[ii],sy[iii]-sy[ii],22))
				{
					slist[splitnum+0] = z;
					slist[splitnum+1] = lastz;
					cyy[splitnum+1]++;
				}
				else
				{
					slist[splitnum+0] = lastz;
					slist[splitnum+1] = z;
					cyy[splitnum+0]++;
				}
			}
			else
			{
				zx = 0; while (slist[zx] != z) zx++;
				zzx = 0; while (bunchend[slist[zzx]] != bunchstart[z]) zzx++;

				if (zx > zzx)
				{
					for(y=cyy[zzx]+1;y<cyy[zx];y++)
						lastx[y+xdim] = x;
				}
				else
				{
					for(y=cyy[zx];y<=cyy[zzx];y++)
						diagvline(x-1,y);
				}

				if (z == numbunches-1) break;

				if (zzx >= zx) zzx--;
				while (zx < snum)
					slist[zx] = slist[zx+1], cyy[zx] = cyy[zx+1], zx++;
				while (zzx < snum)
					slist[zzx] = slist[zzx+1], cyy[zzx] = cyy[zzx+1], zzx++;
				snum -= 2;
			}

			lx = (sy[bunchstart[z+0]]>>16);
			rx = (sy[bunchstart[z+1]]>>16);
			cyptr = (long *)(FP_OFF(cy));
			for (zz=0;zz<snum;zz++)
			{
				i = bunchstart[slist[zz]];
				do
				{
					ii = point2[i];

					x1 = sy[i]; dax1 = (x1>>16);
					x2 = sy[ii]; dax2 = (x2>>16);
					if (dax1 != dax2)
					{
						y1 = sx[i]; y2 = sx[ii];
						if (dax1 > dax2)
						{
							templong = x1; x1 = x2; x2 = templong;
							templong = y1; y1 = y2; y2 = templong;
							templong = dax1; dax1 = dax2; dax2 = templong;
						}
						if ((dax1 < rx) && (lx < dax2))
						{
							yplc = y1-((zz&1)<<16);
							if (dax2 > dax1+1)
							{
								yinc = divscale(y2-y1,x2-x1,16);
								yplc += mulscale((-x1)&65535,yinc,16);
							}
							if (dax1 < lx) yplc += (lx-dax1)*yinc, dax1 = lx;
							if (dax2 > rx) dax2 = rx;

							if (dax2 > dax1)
								qinterpolatedown16(((long)cyptr)+(dax1<<2),dax2-dax1,yplc,yinc);

							//dax1 += zzoffs; dax2 += zzoffs;
							//for(x=dax1;x<dax2;x++)
							//{
							//   cy[x] = (yplc>>16);
							//   yplc += yinc;
							//}
						}
					}

					i = ii;
				} while (i != bunchend[slist[zz]]);
				cyptr += MAXXDIM;
			}

			if (snum == 2)
			{
				y1 = cyy[0]; y2 = cyy[1];
				cyptr = (long *)(FP_OFF(cy)+(lx<<2));
				for(x=lx;x<rx;x++)
				{
					dahplc = hplc[x];
					g1 = cyptr[0]-dahplc; g2 = cyptr[MAXXDIM]-dahplc;
					cyptr++;

					while (g1 < y1) { y1--; lastx[y1+xdim] = x; }
					while (g1 > y1) { diagvline(x-1,y1); y1++; }
					while (g2 > y2) { y2++; lastx[y2+xdim] = x; }
					while (g2 < y2) { diagvline(x-1,y2); y2--; }
				}
				cyy[0] = y1; cyy[1] = y2;
			}
			else if (snum > 2)
			{
				for(x=lx;x<rx;x++)
				{
					dahplc = hplc[x];
					cyptr = (long *)(FP_OFF(cy)+(x<<2));
					for(zz=0;zz<snum;zz+=2)
					{
						g1 = cyptr[0]-dahplc; g2 = cyptr[MAXXDIM]-dahplc;
						cyptr += (MAXXDIM<<1);
						y1 = cyy[zz]; y2 = cyy[zz+1];

						while (g1 < y1) { y1--; lastx[y1+xdim] = x; }
						while (g1 > y1) { diagvline(x-1,y1); y1++; }
						while (g2 > y2) { y2++; lastx[y2+xdim] = x; }
						while (g2 < y2) { diagvline(x-1,y2); y2--; }

						cyy[zz] = y1; cyy[zz+1] = y2;
					}
				}
			}
		}
	}
}

diaghline (long x2, long y)
{
	long x1, y1, p, bx, by, t, scrx, scry;

	x1 = lastx[y+xdim]; if (x1 > x2) return;
	lastx[y+xdim] = 0x7fffffff;

	y1 = (y<<16) - x1*globm;

	scrx = x1-globdax;
	if (scrx != 0)
	{
		globdax = x1;
		getd += getdxinc*scrx;
		getu += getuxinc*scrx;
		getv += getvxinc*scrx;
	}
	scry = y1-globday;
	t = getd + mulscale(getdyinc,scry,16);
	if ((reversemode == 0) && (t <= 512)) return;
	if ((reversemode == 1) && (t >= -512)) return;
	t = divscale(65536,t,24);
	bx = (mulscale(t,getu + mulscale(getuyinc,scry,16),8)&0xffffffc0);
	by = (mulscale(t,getv + mulscale(getvyinc,scry,16),8)&0xffffffc0);
	asm1 = mulscale(t,globuadd,8);     //bxinc
	asm2 = mulscale(t,globvadd,8);     //byinc

	asm3 = mulscale(t,gett,31)+globshade;
	if (asm3 < 0) asm3 = 0;
	if (asm3 >= numpalookups) asm3 = numpalookups-1;
	asm3 = FP_OFF(palookup)+(asm3<<9);   //shadeoffs

	p = ((ylookup[y1>>16] + x1)<<1) + frameplace;

	if (globm == 0)
	{
		if ((globstat&2) == 0)
			hline(bx,by,x2-x1,0L,0L,p);
		else
			mhline(bx,by,x2-x1,0L,0L,p);
		return;
	}

	//yinc = -globm;
	//if (yinc < 0)
	//   yinc = -yinc, y1 ^= 65535;
	//y1 &= 65535;
	//if (yinc >= 65536)
	//   yinc = 65535, y1 = 65535;
	//asm4 = (yinc<<16);

	if ((globstat&2) == 0)
		dline(bx,by,x2-x1,x1<<2,0L,p);
	else
		mdline(bx,by,x2-x1,x1<<2,0L,p);

	//for(x=x1;x<=x2;x++)
	//{
	//   ptr = (char *)(picoffs+(((bx>>26)&63)<<6)+((by>>26)&63));
	//   drawpixel(p,palookup[shade][*ptr]);
	//   y += yinc;
	//   p += (pinc&(sign extend of carry));
	//   p++;
	//   bx += bxinc;
	//   by += byinc;
	//}
}

diagvline (long y2, long x)
{
	long x1, y1, p, bx, by, t, scrx, scry, xinc;

	y1 = lastx[x+xdim]; if (y1 > y2) return;
	lastx[x+xdim] = 0x7fffffff;

	x1 = (x<<16) - y1*globm;

	scry = y1-globday;
	if (scry != 0)
	{
		globday = y1;
		getd += getdyinc*scry;
		getu += getuyinc*scry;
		getv += getvyinc*scry;
	}
	scrx = x1-globdax;
	t = getd + mulscale(getdxinc,scrx,16);
	if ((reversemode == 0) && (t <= 512)) return;
	if ((reversemode == 1) && (t >= -512)) return;
	t = divscale(65536,t,24);
	bx = (mulscale(t,getu + mulscale(getuxinc,scrx,16),8)&0xffffffc0);
	by = (mulscale(t,getv + mulscale(getvxinc,scrx,16),8)&0xffffffc0);
	asm1 = mulscale(t,globuadd,8);                 //bxinc
	asm2 = mulscale(t,globvadd,8);                 //byinc

	asm3 = mulscale(t,gett,31)+globshade;
	if (asm3 < 0) asm3 = 0;
	if (asm3 >= numpalookups) asm3 = numpalookups-1;
	asm3 = FP_OFF(palookup)+(asm3<<9);   //shadeoffs

	p = ((ylookup[y1] + (x1>>16))<<1) + frameplace;

	if (globm == 0)
	{
		if ((globstat&2) == 0)
			vline(bx,by,y2-y1,0L,0L,p);
		else
			mvline(bx,by,y2-y1,0L,0L,p);
		return;
	}

	//xinc = -globm;
	//if (xinc < 0)
	//   xinc = -xinc, x1 ^= 65535;
	//x1 &= 65535;
	//if (xinc >= 65536)
	//   xinc = 65535, x1 = 65535;
	//asm4 = (xinc<<16);

	if ((globstat&2) == 0)
		dline(bx,by,y2-y1,y1<<2,0L,p);
	else
		mdline(bx,by,y2-y1,y1<<2,0L,p);

	//for(y=y1;y<=y2;y++)
	//{
	//   ptr = (char *)(picoffs+(((bx>>26)&63)<<6)+((by>>26)&63));
	//   drawpixel(p,palookup[shade][*ptr]);
	//   x += xinc;
	//   p += xdim(+carry); (NOTE:  This is adc / sbb with the x += xinc)
	//   bx += bxinc;
	//   by += byinc;
	//}
}

drawslowtextureplane(long npoints)
{
	long i, z, zz, zzz, d1, d2, dotspan, mindot, maxdot, newdot;
	long x, xinc, x1, x2, index, bx, by, shval1, shval2, dat;
	long col, p, lastp, dapicnum, dashade, bufplc, maskval1, maskval2;
	char *ptr;

	setupgetreadpos();

	if ((getdxinc == 0) || (getdyinc == 0) || (klabs(getdxinc) == klabs(getdyinc)))
	{
		drawfasttextureplane(npoints);
		return;
	}

	mindot = 0x7fffffff; maxdot = 0x80000000;

	for (z=0;z<npoints;z++)
	{
		newdot = ((sy[z]+1)>>16);
		thedot[z] = newdot;
		if (newdot < mindot) mindot = newdot;
		if (newdot > maxdot) maxdot = newdot;
	}
	dotspan = maxdot-mindot;
	clearbuf(FP_OFF(dotc),(dotspan+2)>>1,0L);

	for(z=0;z<npoints;z++)
		thedot[z] -= mindot;

	for(z=0;z<npoints;z++)
	{
		zz = point2[z];
		d1 = thedot[z]; d2 = thedot[zz];

		if (d1 != d2)
		{
			xinc = (sx[zz]-sx[z])/(d2-d1);

			if (d2 > d1)
			{
				x = sx[z];
				for(index=d1;index<d2;index++)
				{
					doti[index][dotc[index]++] = ((x+1)>>16);
					x += xinc;
				}
			}
			else
			{
				x = sx[zz];
				for(index=d2;index<d1;index++)
				{
					doti[index][dotc[index]++] = ((x+1)>>16);
					x += xinc;
				}
			}
		}
	}

	dapicnum = picnum[globplane]; dapicnum += animateoffs(dapicnum);
	if (waloff[dapicnum] < 0) loadtile(dapicnum);

	dashade = shade[globplane]+4;
	if (dashade < 0) dashade = 0;
	if (dashade >= numpalookups) dashade = numpalookups-1;

	ptr = (char *)(waloff[dapicnum]); col = *ptr;
	for(z=0;z<8;z++)
	{
		ptr++;
		i = *ptr;
		if (palette[(i<<1)+i]+palette[(i<<1)+i+1]+palette[(i<<1)+i+2] > palette[(col<<1)+col]+palette[(col<<1)+col+1]+palette[(col<<1)+col+2])
			col = i;
	}
	col = palookup[dashade][col];
	col += (col<<8);
	col += (col<<16);

	bufplc = waloff[dapicnum];

	maskval1 = ((1<<(picsiz[picnum[globplane]]&15))-1);
	maskval2 = ((1<<(picsiz[picnum[globplane]]>>4))-1);
	shval1 = 26-(picsiz[picnum[globplane]]&15);
	shval2 = 26-(picsiz[picnum[globplane]]>>4);
	maskval1 <<= 8;
	shval1 -= 8;

	for(z=0;z<=dotspan;z++)
		if (dotc[z] > 0)
		{
			if (dotc[z] > 2)
				for(zz=1;zz<dotc[z];zz++)
					for(zzz=0;zzz<zz;zzz++)
						if (doti[z][zz] < doti[z][zzz])
							i = doti[z][zz], doti[z][zz] = doti[z][zzz], doti[z][zzz] = i;
			for(zz=0;zz<dotc[z];zz+=2)
			{
				x1 = doti[z][zz]; x2 = doti[z][zz+1];
				if (x1 > x2) i = x1, x1 = x2, x2 = i;

				zzz = ((z+mindot)<<16);
				p = frameplace + ylookup[z+mindot] + x1;
				if ((globstat&2) == 0)
				{
					for(lastp=x1;lastp<x2;lastp++)
					{
						getreadpos(lastp<<16,zzz,&bx,&by);
						ptr = (char *)(bufplc+((bx>>shval1)&maskval1)+((by>>shval2)&maskval2));
						dat = *ptr;
						drawpixel(p,dat);
						p++;
					}
				}
				else
				{
					for(lastp=x1;lastp<x2;lastp++)
					{
						getreadpos(lastp<<16,zzz,&bx,&by);
						ptr = (char *)(bufplc+((bx>>shval1)&maskval1)+((by>>shval2)&maskval2));
						dat = *ptr;
						if (dat != 255)
							drawpixel(p,dat);
						p++;
					}
				}
			}
		}
}

getmousevalues(short *damousx, short *damousy, short *dabstatus)
{
	if (moustat == 1)
	{
		readmousexy(damousx,damousy);
		readmousebstatus(dabstatus);
	}
}

overwritemousecursor()
{
	long i;

	for(i=1;i<=4;i++)
	{
		drawpixel(frameplace+ylookup[searchy+i]+(searchx+0),whitecol);
		drawpixel(frameplace+ylookup[searchy-i]+(searchx+0),whitecol);
		drawpixel(frameplace+ylookup[searchy+0]+(searchx+i),whitecol);
		drawpixel(frameplace+ylookup[searchy+0]+(searchx-i),whitecol);
	}
}

xormousecursor()
{
	long i, p;

	for(i=1;i<=4;i++)
	{
		p = ylookup[searchy-i] + (searchx+0);
		vesasetpage(p>>16); p = 0xa0000+(p&65535); drawpixel(p,getpixel(p)^144);
		p = ylookup[searchy+0] + (searchx-i);
		vesasetpage(p>>16); p = 0xa0000+(p&65535); drawpixel(p,getpixel(p)^144);
		p = ylookup[searchy+0] + (searchx+i);
		vesasetpage(p>>16); p = 0xa0000+(p&65535); drawpixel(p,getpixel(p)^144);
		p = ylookup[searchy+i] + (searchx+0);
		vesasetpage(p>>16); p = 0xa0000+(p&65535); drawpixel(p,getpixel(p)^144);
	}
}

setvesamode(short vesamode)
{
	static struct rminfo
	{
		long EDI, ESI, EBP, ESP, EBX, EDX, ECX, EAX;
		short flags, ES, DS, FS, GS, IP, CS, SP, SS;
	} RMI;

	union REGS regs;
	struct SREGS sregs;
	short selector, segment;
	short *ptr, gran;

	memset(&sregs,0,sizeof(sregs));
	regs.w.ax = 0x0100;
	regs.w.bx = (256>>4);             //Number of paragraphs
	int386x(0x31,&regs,&regs,&sregs); // DPMI call 100h allocates DOS memory
	segment = regs.w.ax;
	selector = regs.w.dx;

	memset(&RMI,0,sizeof(RMI));    // Set up real-mode call structure
	RMI.EAX = 0x00004f01;
	RMI.ECX = (long)vesamode;
	RMI.ES = segment;
	RMI.EDI = 0;

	regs.w.ax = 0x0300;            // Use DMPI call 300h to issue the DOS interrupt
	regs.h.bl = 0x10;
	regs.h.bh = 0;
	regs.w.cx = 0;
	sregs.es = FP_SEG(&RMI);
	regs.x.edi = FP_OFF(&RMI);
	int386x(0x31,&regs,&regs,&sregs);

	if ((RMI.EAX&65535) != 0x004f)
		return(-1);

	ptr = (short *)((((long)segment)<<4)+4);
	gran = 64 / (*ptr);
	vesapageshift = 0;
	while (gran > 1)
	{
		vesapageshift++;
		gran >>= 1;
	}

	if (vesasetmode(vesamode) != 0x4f)
		return(-1);

	return(0);
}

draw3dclipline(long rx0, long ry0, long rz0, long rx1, long ry1, long rz1, char col)
{
	long sx0, sy0, sx1, sy1;

	rotate3d(&rx0,&ry0,&rz0);
	rotate3d(&rx1,&ry1,&rz1);

	if ((rz0 > 0) || (rz1 > 0))
	{
		if (clipline(&rx0,&ry0,&rz0,&rx1,&ry1,&rz1) == 1)
		{
			if ((rz0 > 16) && (rz1 > 16))
			{
				project(rx0,ry0,rz0,&sx0,&sy0);
				if (sx0 < 0) sx0 = 0;
				if (sx0 >= (xdim<<16)) sx0 = (xdim<<16)-1;
				if (sy0 < 0) sy0 = 0;
				if (sy0 >= (ydim<<16)) sy0 = (ydim<<16)-1;
				project(rx1,ry1,rz1,&sx1,&sy1);
				if (sx1 < 0) sx1 = 0;
				if (sx1 >= (xdim<<16)) sx1 = (xdim<<16)-1;
				if (sy1 < 0) sy1 = 0;
				if (sy1 >= (ydim<<16)) sy1 = (ydim<<16)-1;
				drawline(sx0,sy0,sx1,sy1,col);
			}
		}
	}
}

drawgrid()
{
	long x, y, z, xs, ys, zs, xe, ye, ze, xo, yo, zo, col;

	xs = posx-3072; xe = xs+6144;
	ys = posy-3072; ye = ys+6144;
	zs = posz-3072; ze = zs+6144;

	xo = (posx&511^511);
	yo = (posy&511^511);
	zo = (posz&511^511);

	for(x=xs+xo;x<=xe;x+=512)
	{
		draw3dclipline(x-posx,ys-posy,zs-posz,x-posx,ys-posy,ze-posz,6);
		draw3dclipline(x-posx,ys-posy,ze-posz,x-posx,ye-posy,ze-posz,6);
		draw3dclipline(x-posx,ye-posy,ze-posz,x-posx,ye-posy,zs-posz,6);
		draw3dclipline(x-posx,ye-posy,zs-posz,x-posx,ys-posy,zs-posz,6);
	}
	for(y=ys+yo;y<=ye;y+=512)
	{
		draw3dclipline(xs-posx,y-posy,zs-posz,xs-posx,y-posy,ze-posz,5);
		draw3dclipline(xs-posx,y-posy,ze-posz,xe-posx,y-posy,ze-posz,5);
		draw3dclipline(xe-posx,y-posy,ze-posz,xe-posx,y-posy,zs-posz,5);
		draw3dclipline(xe-posx,y-posy,zs-posz,xs-posx,y-posy,zs-posz,5);
	}
	for(z=zs+zo;z<=ze;z+=512)
	{
		draw3dclipline(xs-posx,ys-posy,z-posz,xs-posx,ye-posy,z-posz,4);
		draw3dclipline(xs-posx,ye-posy,z-posz,xe-posx,ye-posy,z-posz,4);
		draw3dclipline(xe-posx,ye-posy,z-posz,xe-posx,ys-posy,z-posz,4);
		draw3dclipline(xe-posx,ys-posy,z-posz,xs-posx,ys-posy,z-posz,4);
	}
}

gettile(short dapicnum)
{
	char *ptr;
	long xpicnum, ypicnum, xypicnum;
	short topleftpic;

	xpicnum = (xdim>>6);
	ypicnum = (ydim>>6);
	xypicnum = xpicnum*ypicnum;

	topleftpic = (dapicnum/xypicnum)*xypicnum;
	drawtilescreen(topleftpic);

	xormousecursor();
	while (1)
	{
		obstatus = bstatus, getmousevalues(&mousx,&mousy,&bstatus);
		if ((mousx|mousy) != 0)
		{
			xormousecursor();
			searchx += mousx;
			searchy += mousy;
			if (searchx < 4) searchx = 4;
			if (searchx >= xdim-4) searchx = xdim-4-1;
			if (searchy < 4) searchy = 4;
			if (searchy >= ydim-4) searchy = ydim-4-1;
			xormousecursor();
		}

		if (keystatus[0xcb] > 0)
		{
			keystatus[0xcb] = 0;
			xormousecursor();
			searchx -= 64;
			if (searchx < 4) searchx = 4;
			xormousecursor();
		}
		if (keystatus[0xcd] > 0)
		{
			keystatus[0xcd] = 0;
			xormousecursor();
			searchx += 64;
			if (searchx >= xdim-4) searchx = xdim-4-1;
			xormousecursor();
		}
		if (keystatus[0xc8] > 0)
		{
			keystatus[0xc8] = 0;
			xormousecursor();
			searchy -= 64;
			if (searchy < 4) searchy = 4;
			xormousecursor();
		}
		if (keystatus[0xd0] > 0)
		{
			keystatus[0xd0] = 0;
			xormousecursor();
			searchy += 64;
			if (searchx >= xdim-4) searchx = xdim-4-1;
			xormousecursor();
		}
		if (keystatus[0xc9] > 0)  //PGUP
		{
			if (topleftpic > 0)
			{
				topleftpic -= xypicnum;
				drawtilescreen(topleftpic);
				xormousecursor();
			}
			keystatus[0xc9] = 0;
		}
		if (keystatus[0xd1] > 0)  //PGDN
		{
			if (topleftpic < MAXTILES-xypicnum)
			{
				topleftpic += xypicnum;
				drawtilescreen(topleftpic);
				xormousecursor();
			}
			keystatus[0xd1] = 0;
		}

		if (keystatus[1] > 0)
		{
			keystatus[1] = 0;
			return(dapicnum);
		}
		if (keystatus[0x1c] > 0)
		{
			keystatus[0x1c] = 0;
			if ((searchx < (xpicnum<<6)) && (searchy < (ypicnum<<6)))
				dapicnum = topleftpic+(searchy>>6)*xpicnum+(searchx>>6);
			return(dapicnum);
		}

	}
}

drawtilescreen(short topleftpic)
{
	char *ptr;
	long i, j, xsiz, ysiz, ptrinc, x, y, p;
	short dapic;


	clearbuf(frameplace,(xdim*ydim)>>2,0x06070807);

		//Draw tiles on screen
	dapic = topleftpic;
	for(j=0;j<=ydim-64;j+=64)
		for(i=0;i<=xdim-64;i+=64)
		{
			xsiz = (long)tilesizx[dapic];
			ysiz = (long)tilesizy[dapic];
			if (waloff[dapic] < 0) loadtile(dapic);

			if ((xsiz > 0) && (ysiz > 0))
			{
				ptrinc = 1;
				while (((xsiz > 64) || (ysiz > 64)) && (ptrinc < 32))
				{
					xsiz >>= 1;
					ysiz >>= 1;
					ptrinc <<= 1;
				}
				if (xsiz > 64) xsiz = 64;
				if (ysiz > 64) ysiz = 64;

				for(x=0;x<xsiz;x++)
				{
					ptr = (char *)(waloff[dapic]+((x*ptrinc)<<8));
					p = frameplace+ylookup[j]+(x+i);
					for(y=0;y<ysiz;y++)
					{
						drawpixel(p,*ptr);
						p += xdim;
						ptr += ptrinc;
					}
				}
			}
			dapic++;
		}

	nextpage();

	if (option[0] >= 2)
	{
		page = ((page+3)&3);
		if (option[0] == 2) koutp(0x3cd,page+(page<<4));
		if (option[0] == 3) koutp(0x3ce,0x9), koutp(0x3cf,page<<4);
		if (option[0] == 4) koutp(0x3d4,0x35), koutp(0x3d5,page);
	}
}

animateoffs(long tilenum)
{
	long i, k, offs;

	offs = 0;
	i = (lockclock>>((picanm[tilenum]>>24)&15));
	if ((picanm[tilenum]&63) > 0)
	{
		switch(picanm[tilenum]&192)
		{
			case 64:
				k = (i%((picanm[tilenum]&63)<<1));
				if (k < (picanm[tilenum]&63))
					offs = k;
				else
					offs = (((picanm[tilenum]&63)<<1)-k);
				break;
			case 128:
				offs = (i%((picanm[tilenum]&63)+1));
					break;
			case 192:
				offs = -(i%((picanm[tilenum]&63)+1));
		}
	}
	return(offs);
}

screencapture()
{
	char filename[15], *ptr;
	long fil, i, bufplc, p, col, ncol, leng, numbytes, xres;

	strcpy(filename,"capturxx.pcx");
	filename[6] = (capturecount/10)+48;
	filename[7] = (capturecount%10)+48;
	capturecount++;
	if ((fil=open(filename,O_BINARY|O_CREAT|O_TRUNC|O_WRONLY,S_IWRITE))==-1)
	{
		capturecount--;
		return(-1);
	}

	pcxheader[8] = ((xdim-1)&255); pcxheader[9] = (((xdim-1)>>8)&255);
	pcxheader[10] = ((ydim-1)&255); pcxheader[11] = (((ydim-1)>>8)&255);
	pcxheader[12] = (xdim&255); pcxheader[13] = ((xdim>>8)&255);
	pcxheader[14] = (ydim&255); pcxheader[15] = ((ydim>>8)&255);
	pcxheader[66] = (xdim&255); pcxheader[67] = ((xdim>>8)&255);

	write(fil,&pcxheader[0],128);

	if (option[0] == 0)
		ptr = (char *)(0xa0000 + (((long)page)<<14));
	else
		ptr = (char *)frameplace;
	numbytes = xdim*ydim;
	xres = xdim;

	bufplc = 0;
	p = 0;
	while (p < numbytes)
	{
		koutp(97,kinp(97)|3);

		if (option[0] == 0)
		{
			koutp(0x3ce,4); koutp(0x3cf,p&3); col = *ptr;
			p++; if ((p&3) == 0) ptr++;
		}
		else
		{
			col = *ptr;
			p++; ptr++;
		}

		leng = 1;

		if (option[0] == 0)
		{
			koutp(0x3ce,4); koutp(0x3cf,p&3); ncol = *ptr;
		}
		else
		{
			ncol = *ptr;
		}

		while ((ncol == col) && (p < numbytes) && (leng < 63) && ((p%xres) != 0))
		{
			leng++;

			if (option[0] == 0)
			{
				p++; if ((p&3) == 0) ptr++;
				koutp(0x3ce,4); koutp(0x3cf,p&3); ncol = *ptr;
			}
			else
			{
				p++; ptr++;
				ncol = *ptr;
			}
		}

		koutp(97,kinp(97)&252);

		if ((leng > 1) || (col >= 0xc0))
		{
			tempbuf[bufplc++] = (leng|0xc0);
			if (bufplc == 256)
			{
				if (write(fil,&tempbuf[0],256) < 256)
				{
					close(fil); capturecount--; return(-1);
				}
				bufplc = 0;
			}
		}
		tempbuf[bufplc++] = col;
		if (bufplc == 256)
		{
			if (write(fil,&tempbuf[0],256) < 256)
			{
				close(fil); capturecount--; return(-1);
			}
			bufplc = 0;
		}
	}

	tempbuf[bufplc++] = 0xc;
	if (bufplc == 256)
	{
		if (write(fil,&tempbuf[0],256) < 256)
		{
			close(fil); capturecount--; return(-1);
		}
		bufplc = 0;
	}

	koutp(0x3c7,0);
	for(i=0;i<768;i++)
	{
		tempbuf[bufplc++] = (kinp(0x3c9)<<2);
		if (bufplc == 256)
		{
			if (write(fil,&tempbuf[0],256) < 256)
			{
				close(fil); capturecount--; return(-1);
			}
			bufplc = 0;
		}
	}

	if (bufplc > 0)
		if (write(fil,&tempbuf[0],bufplc) < bufplc)
		{
			close(fil); capturecount--; return(-1);
		}
	close(fil);
	return(0);
}

/*inside(long x, long y, long npoints)
{
	long z, zz, x1, y1, x2, y2, templong;
	short cnt;

	cnt = 0;
	for(z=0;z<npoints;z++)
	{
		zz = point2[z];

		x1 = (sx[z]>>16); x2 = (sx[zz]>>16);
		if ((x1 >= x) || (x2 >= x))
		{
			y1 = (sy[z]>>16); y2 = (sy[zz]>>16);
			if (y1 > y2)
			{
				templong = x1, x1 = x2, x2 = templong;
				templong = y1, y1 = y2, y2 = templong;
			}
			if ((y1 <= y) && (y2 > y))
				if (x1*(y-y2)+x2*(y1-y) <= x*(y1-y2))
					cnt ^= 1;
		}
	}
	return(cnt);
}*/

/*planefront(long plane1, long plane2)
{
	long i, z, z1, z2;

	z1 = 0; z2 = 0;                                 //Z averaging (temporary)
	for(i=0;i<linnum[plane1];i++)
	{
		z = rzi[lin[linoff[plane1]+i]];
		if (z > 0) z1 += z;
	}
	for(i=0;i<linnum[plane2];i++)
	{
		z = rzi[lin[linoff[plane2]+i]];
		if (z > 0) z2 += z;
	}
	if (z1*linnum[plane2] < z2*linnum[plane1])
		return(plane1);
	else
		return(plane2);
}*/

resizelinnum(long daplane, long dapoints)
{
	long i, j, dachange;

	dachange = dapoints-linnum[daplane];
	if (dachange == 0) return;

	if (dachange < 0)
	{
		for(i=linoff[daplane];i<numlines;i++)
		{
			lin[i+dapoints] = lin[i+linnum[daplane]];
			lin2[i+dapoints] = lin2[i+linnum[daplane]]+dachange;
		}
	}
	else
	{
		for(i=numlines-1;i>=linoff[daplane];i--)
		{
			lin[i+dapoints] = lin[i+linnum[daplane]];
			lin2[i+dapoints] = lin2[i+linnum[daplane]]+dachange;
		}
	}
	numlines += dachange;

	linnum[daplane] = dapoints;
	for(i=daplane;i<numplanes;i++)
		linoff[i+1] = linoff[i]+linnum[i];
}

drawsolidplane(long npoints)
{
	long i, z, zz, zzz, d1, d2, dotspan, mindot, maxdot, newdot;
	long x, xinc, x1, x2, index;
	long col, p, lastp, dapicnum, dashade;
	char *ptr;

	mindot = 0x7fffffff; maxdot = 0x80000000;

	for (z=0;z<npoints;z++)
	{
		newdot = ((sy[z]+1)>>16);
		thedot[z] = newdot;
		if (newdot < mindot) mindot = newdot;
		if (newdot > maxdot) maxdot = newdot;
	}
	dotspan = maxdot-mindot;
	clearbuf(FP_OFF(dotc),(dotspan+2)>>1,0L);

	for(z=0;z<npoints;z++)
		thedot[z] -= mindot;

	for(z=0;z<npoints;z++)
	{
		zz = point2[z];
		d1 = thedot[z]; d2 = thedot[zz];

		if (d1 != d2)
		{
			xinc = (sx[zz]-sx[z])/(d2-d1);

			if (d2 > d1)
			{
				x = sx[z];
				for(index=d1;index<d2;index++)
				{
					doti[index][dotc[index]++] = ((x+1)>>16);
					x += xinc;
				}
			}
			else
			{
				x = sx[zz];
				for(index=d2;index<d1;index++)
				{
					doti[index][dotc[index]++] = ((x+1)>>16);
					x += xinc;
				}
			}
		}
	}

	dapicnum = picnum[globplane]; dapicnum += animateoffs(dapicnum);
	if (waloff[dapicnum] < 0) loadtile(dapicnum);

	dashade = shade[globplane]+4;
	if (dashade < 0) dashade = 0;
	if (dashade >= numpalookups) dashade = numpalookups-1;

	ptr = (char *)(waloff[dapicnum]); col = *ptr;
	for(z=0;z<8;z++)
	{
		ptr++;
		i = *ptr;
		if (palette[(i<<1)+i]+palette[(i<<1)+i+1]+palette[(i<<1)+i+2] > palette[(col<<1)+col]+palette[(col<<1)+col+1]+palette[(col<<1)+col+2])
			col = i;
	}
	col = palookup[dashade][col];
	col += (col<<8);
	col += (col<<16);

	for(z=0;z<=dotspan;z++)
		if (dotc[z] > 0)
		{
			if (dotc[z] > 2)
				for(zz=1;zz<dotc[z];zz++)
					for(zzz=0;zzz<zz;zzz++)
						if (doti[z][zz] < doti[z][zzz])
							i = doti[z][zz], doti[z][zz] = doti[z][zzz], doti[z][zzz] = i;
			for(zz=0;zz<dotc[z];zz+=2)
			{
				x1 = doti[z][zz]; x2 = doti[z][zz+1];
				if (x1 > x2) i = x1, x1 = x2, x2 = i;

				p = frameplace + ylookup[z+mindot] + x1;
				lastp = p+(x2-x1);

				while (p < lastp-3) drawpixelses(p,col), p += 4;
				if (p < lastp-1) drawpixels(p,col), p += 2;
				if (p < lastp) drawpixel(p,col);
			}
		}
}

drawwireplane(long npoints)
{
	long z, zz;

	for(z=0;z<npoints;z++)
	{
		zz = point2[z];
		drawline(sx[z],sy[z],sx[zz],sy[zz],79);
	}
}

clipmove (long *x, long *y, long *z, long xv, long yv, long zv)
{
	long i, cnt, xvect[4], yvect[4], zvect[4];

	getcliplanes (*x,*y,*z,xv,yv,zv);

	cnt = 3;
	do
	{
		xvect[cnt] = xv; yvect[cnt] = yv; zvect[cnt] = zv;

		raytrace(x,y,z,&xv,&yv,&zv);
		cnt--;

		for(i=cnt+1;i<3;i++)
			if (mulscale(xv,xvect[i],6)+mulscale(yv,yvect[i],6)+mulscale(zv,zvect[i],6) < -1024)
				return;

	} while (((xv|yv|zv) != 0) && (cnt >= 0));
}

raytrace (long *x, long *y, long *z, long *xv, long *yv, long *zv)
{
	long goalx, goaly, goalz, intx, inty, intz, nintx, ninty, nintz;
	long clipplane, nx, ny, nz, x0, y0, z0, x1, y1, x2, y2, zx, zxx;
	long top, bot, plane, topoverbot, cnt, templong, bad;
	long clipxoffs, clipyoffs, *ptr, closesttopoverbot;

	goalx = (*x) + (*xv); goaly = (*y) + (*yv); goalz = (*z) + (*zv);
	intx = goalx; inty = goaly; intz = goalz;

	clipplane = -1;
	closesttopoverbot = 0x7fffffff;

	for(plane=cliplanes-1;plane>=0;plane--)
	{
		templong = cliplinoff[plane];
		x0 = (*x)-clipx[templong]; nx = nxi[cliplanum[plane]];
		y0 = (*y)-clipy[templong]; ny = nyi[cliplanum[plane]];
		z0 = (*z)-clipz[templong]; nz = nzi[cliplanum[plane]];
		bot = -(mulscale(nx,*xv,12)+mulscale(ny,*yv,12)+mulscale(nz,*zv,12));
		top = mulscale(nx,x0,12)+mulscale(ny,y0,12)+mulscale(nz,z0,12);

		if ((top <= 0) && (bot < 0) && (top >= bot-256))
		{
			topoverbot = divscale(top,bot,24);
			if (topoverbot-262144 < closesttopoverbot)
			{
				nintx = (*x) + mulscale(*xv,topoverbot,24);
				ninty = (*y) + mulscale(*yv,topoverbot,24);
				nintz = (*z) + mulscale(*zv,topoverbot,24);

				//if (klabs(nintx-(*x))+klabs(ninty-(*y))+klabs(nintz-(*z)) < klabs(intx-(*x))+klabs(inty-(*y))+klabs(intz-(*z)))
				//{

				clipxoffs = (cliplinoff[plane]<<2); clipyoffs = clipxoffs;
				if ((klabs(nz) > klabs(nx)) && (klabs(nz) > klabs(ny)))
				{
					x0 = nintx; clipxoffs += (long)clipx;
					y0 = ninty; clipyoffs += (long)clipy;
				}
				else if (klabs(ny) > klabs(nx))
				{
					x0 = nintx; clipxoffs += (long)clipx;
					y0 = nintz; clipyoffs += (long)clipz;
				}
				else
				{
					x0 = ninty; clipxoffs += (long)clipy;
					y0 = nintz; clipyoffs += (long)clipz;
				}

				cnt = 0;
				ptr = (long *)clipxoffs; x2 = *ptr;
				ptr = (long *)clipyoffs; y2 = *ptr;
				for(zx=0;zx<cliplinnum[plane];zx++)
				{
					zxx = zx+1; if (zxx >= cliplinnum[plane]) zxx = 0;
					zxx <<= 2;
					x1 = x2; ptr = (long *)(clipxoffs+zxx); x2 = *ptr;
					y1 = y2; ptr = (long *)(clipyoffs+zxx); y2 = *ptr;

					if ((x1 >= x0) || (x2 >= x0))
					{
						if (y1 <= y2)
						{
							if ((y1 <= y0) && (y2 > y0))
								if (x1*(y0-y2)+x2*(y1-y0) <= x0*(y1-y2))
									cnt ^= 1;
						}
						else
						{
							if ((y2 <= y0) && (y1 > y0))
								if (x2*(y0-y1)+x1*(y2-y0) <= x0*(y2-y1))
									cnt ^= 1;
						}
					}
				}

				if (cnt == 1)
				{
					templong = cliplinoff[plane];
					x0 = clipx[templong]; y0 = clipy[templong]; z0 = clipz[templong];
					cnt = 256;
					do
					{
						bad = mulscale(nx,nintx-x0,4)+mulscale(ny,ninty-y0,4)+mulscale(nz,nintz-z0,4);
						if (bad >= 0)
						{
							topoverbot -= 16384;
							nintx = (*x) + mulscale(*xv,topoverbot,24);
							ninty = (*y) + mulscale(*yv,topoverbot,24);
							nintz = (*z) + mulscale(*zv,topoverbot,24);
							cnt--;
						}
					} while ((bad >= 0) && (cnt >= 0));

					if (topoverbot < closesttopoverbot)
					{
						closesttopoverbot = topoverbot;
						intx = nintx; inty = ninty; intz = nintz;
						clipplane = plane;
					}
				}
			}
		}
	}

	if (clipplane == -1)
	{
		*xv = 0;
		*yv = 0;
		*zv = 0;
	}
	else
	{
		nx = nxi[cliplanum[clipplane]];
		ny = nyi[cliplanum[clipplane]];
		nz = nzi[cliplanum[clipplane]];
		bot = mulscale(nx,nx,16)+mulscale(ny,ny,16)+mulscale(nz,nz,16);
		if (bot != 0)
		{
			x0 = goalx-intx; y0 = goaly-inty; z0 = goalz-intz;
			top = mulscale(nx,x0,16)+mulscale(ny,y0,16)+mulscale(nz,z0,16);
			*xv = x0-scale(nx,top,bot)-ksgn(nx);
			*yv = y0-scale(ny,top,bot)-ksgn(ny);
			*zv = z0-scale(nz,top,bot)-ksgn(nz);
		}
	}

	*x = intx;
	*y = inty;
	*z = intz;
}

getcliplanes (long x1, long y1, long z1, long xv, long yv, long zv)
{
	long i, j, k, x2, y2, z2, templong, xmin, ymin, zmin, xmax, ymax, zmax;
	long nx, ny, nz, plane, zx, i0, i1, i2, nx1, ny1, nz1, nx2, ny2, nz2;

	x2 = x1+xv; y2 = y1+yv; z2 = z1+zv;

	if (x1 > x2) templong = x1, x1 = x2, x2 = templong;
	if (y1 > y2) templong = y1, y1 = y2, y2 = templong;
	if (z1 > z2) templong = z1, z1 = z2, z2 = templong;

	x1 -= (128+8); x2 += (128+8);
	y1 -= (128+8); y2 += (128+8);
	z1 -= (128+8); z2 += (128+8);

	clipoints = 0;
	cliplanes = 0;

	for(plane=numplanes-1;plane>=0;plane--)
		if ((planestat[plane]&4) == 0)
			if ((pxmin[plane] <= x2) && (x1 <= pxmax[plane]))
				if ((pzmin[plane] <= z2) && (z1 <= pzmax[plane]))
					if ((pymin[plane] <= y2) && (y1 <= pymax[plane]))
					{
						cliplanum[cliplanes] = plane;
						cliplinoff[cliplanes] = clipoints;
						cliplinnum[cliplanes] = linnum[plane];
						j = linoff[plane];
						for(zx=j;zx<j+linnum[plane];zx++)
						{
							k = lin[zx];
							clipx[clipoints] = px[k] - (ksgn(nxi[plane])<<7);
							clipy[clipoints] = py[k] - (ksgn(nyi[plane])<<7);
							clipz[clipoints] = pz[k] - (ksgn(nzi[plane])<<7);
							clipoints++;
						}
						cliplanes++;
					}
}

hitscan (long *x, long *y, long *z, long xv, long yv, long zv)
{
	long startx, starty, startz, nintx, ninty, nintz, i0, i1, i2;
	long clipplane, nx, ny, nz, x0, y0, z0, x1, y1, z1, x2, y2, z2, zx, zxx;
	long top, bot, plane, topoverbot, closesttopoverbot, cnt;
	long clipxoffs, clipyoffs, *ptr;

	startx = *x; starty = *y; startz = *z;

	clipplane = -1; closesttopoverbot = 0x7fffffff;
	for(plane=0;plane<numplanes;plane++)
	{
		i0 = lin[linoff[plane]+0];
		x0 = (startx-px[i0]); y0 = (starty-py[i0]); z0 = (startz-pz[i0]);

		nx = nxi[plane]; ny = nyi[plane]; nz = nzi[plane];
		bot = -(mulscale(nx,xv,12)+mulscale(ny,yv,12)+mulscale(nz,zv,12));
		top = mulscale(nx,x0,12)+mulscale(ny,y0,12)+mulscale(nz,z0,12);

		if ((top <= 0) && (bot < 0) && (top >= bot))
		{
			topoverbot = divscale(top,bot,24);
			if (topoverbot < closesttopoverbot)
			{
				nintx = startx + mulscale(xv,topoverbot,24);
				ninty = starty + mulscale(yv,topoverbot,24);
				nintz = startz + mulscale(zv,topoverbot,24);

				if ((klabs(nz) > klabs(nx)) && (klabs(nz) > klabs(ny)))
				{
					x0 = nintx; clipxoffs = (long)px;
					y0 = ninty; clipyoffs = (long)py;
				}
				else if (klabs(ny) > klabs(nx))
				{
					x0 = nintx; clipxoffs = (long)px;
					y0 = nintz; clipyoffs = (long)pz;
				}
				else
				{
					x0 = ninty; clipxoffs = (long)py;
					y0 = nintz; clipyoffs = (long)pz;
				}

				cnt = 0;
				zxx = (lin[linoff[plane]]<<2);
				ptr = (long *)(clipxoffs+zxx); x2 = (*ptr)-x0;
				ptr = (long *)(clipyoffs+zxx); y2 = (*ptr)-y0;
				for(zx=0;zx<linnum[plane];zx++)
				{
					zxx = zx+1; if (zxx >= linnum[plane]) zxx = 0;
					zxx = (lin[linoff[plane]+zxx]<<2);
					x1 = x2; ptr = (long *)(clipxoffs+zxx); x2 = (*ptr)-x0;
					y1 = y2; ptr = (long *)(clipyoffs+zxx); y2 = (*ptr)-y0;
					if ((y1^y2) < 0)
					{
						if ((x1^x2) < 0)
							cnt ^= (x1*y2<x2*y1)^(y1<y2);
						else if (x1 >= 0)
							cnt ^= 1;
					}
				}

				if (cnt == 1)
				{
					closesttopoverbot = topoverbot;
					*x = nintx; *y = ninty; *z = nintz;
					clipplane = plane;
				}
			}
		}
	}

	return(clipplane);
}

searchit()
{
	long xvect, yvect, zvect, i, j, mindist, dist, dax, day, daz, dasx, dasy;

	searchitx = posx;
	searchity = posy;
	searchitz = posz;

	xvect = ((searchx-(xdim>>1))<<8);
	yvect = ((searchy-(ydim>>1))<<8);
	zvect = (xdim<<7);
	backrotate3d(&xvect,&yvect,&zvect);

	searchplane = hitscan(&searchitx,&searchity,&searchitz,xvect,yvect,zvect);
	searchpoint = -1;
	if (searchplane >= 0)
	{
		mindist = 384;
		for(i=linoff[searchplane];i<linoff[searchplane]+linnum[searchplane];i++)
		{
			dist = klabs(px[lin[i]]-searchitx)+klabs(py[lin[i]]-searchity)+klabs(pz[lin[i]]-searchitz);
			if (dist < mindist) mindist = dist, searchpoint = lin[i];
		}
		if (searchpoint >= 0)
		{
			dax = px[searchpoint]-posx;
			day = py[searchpoint]-posy;
			daz = pz[searchpoint]-posz;
			rotate3d(&dax,&day,&daz);
			project(dax,day,daz,&dasx,&dasy);

			dasx >>= 16, dasy >>= 16;
			if ((dasx >= 4) && (dasy >= 4) && (dasx < xdim-4) && (dasy < ydim-4))
				for(j=-4;j<=4;j++)
				{
					drawpixel(frameplace+ylookup[dasy+j]+(dasx-4),55);
					drawpixel(frameplace+ylookup[dasy+j]+(dasx+4),55);
					drawpixel(frameplace+ylookup[dasy-4]+(dasx+j),55);
					drawpixel(frameplace+ylookup[dasy+4]+(dasx+j),55);
				}
		}
	}
}

recurseit(long z)
{
	g = lin[linoff[z]]; g2 = 0;
	if (nxi[z] != 0) g2 += mulscale(nxi[z],posx-px[g],8);
	if (nyi[z] != 0) g2 += mulscale(nyi[z],posy-py[g],8);
	if (nzi[z] != 0) g2 += mulscale(nzi[z],posz-pz[g],8);
	if (g2 <= 0)
	{
		if (back[z] >= 0) recurseit(back[z]);
		else if (headspritesect[z+MAXPLANES] >= 0) drawsprisect(z+MAXPLANES);
		if (reversemode == 0) drawplane(z);
		if (front[z] >= 0) recurseit(front[z]);
		else if (headspritesect[z] >= 0) drawsprisect(z);
	}
	else
	{
		if (front[z] >= 0) recurseit(front[z]);
		else if (headspritesect[z] >= 0) drawsprisect(z);
		if (reversemode == 1) drawplane(z);
		if (back[z] >= 0) recurseit(back[z]);
		else if (headspritesect[z+MAXPLANES] >= 0) drawsprisect(z+MAXPLANES);
	}
}

checkinside(long x, long y, long z)
{
	long i, j, k;

	i = head;
	while(i >= 0)
	{
		j = lin[linoff[i]]; k = 0;
		if (nxi[i] != 0) k += mulscale(nxi[i],x-px[j],8);
		if (nyi[i] != 0) k += mulscale(nyi[i],y-py[j],8);
		if (nzi[i] != 0) k += mulscale(nzi[i],z-pz[j],8);
		if (k <= 0) i = front[i]; else i = back[i];
	}
	return(k<=0);
}

drawbspbox(long planum, char dacol)
{
	long p, dat;

	if (planum >= 4000) return;
	p = ((planum%80)<<2)+((planum/80)*320*4)+0xa0000;

	dat = dacol+(dacol<<8);
	dat += (dat<<16);

	drawpixelses(p+0,dat^0x04040404);
	drawpixelses(p+320,dat^0x04000004);
	drawpixelses(p+640,dat^0x04000004);
	drawpixelses(p+960,dat^0x04040404);
}

makebsp()
{
	long i, j, k, z, zz, zx, zzx, bestz, minsplitnum, maxcnt;
	long splitnum, daplistindex, parentz, pnumplanes, xpnumplanes;
	long plane, xmin, xmax, ymin, ymax, zmin, zmax;

	for(i=0;i<numplanes;i++)
	{
		parent[i] = -1, plist[i] = i;
		drawbspbox(i,whitecol);
	}

	pnumplanes = numplanes;
	while (pnumplanes > 0)
	{
		minsplitnum = 0x7fff; parentz = parent[plist[rand()%pnumplanes]];

		xpnumplanes = 0;
		for(zx=pnumplanes-1;zx>=0;zx--)
			if (parent[plist[zx]] == parentz)
				front[xpnumplanes++] = zx;          //Use front as temp here

		maxcnt = 32;
		while (maxcnt > 0)
		{
			zzx = front[rand()%xpnumplanes]; z = plist[zzx];

			splitnum = 0;
			for(zx=xpnumplanes-1;zx>=0;zx--)
			{
				zz = plist[front[zx]];
				if (z != zz)
					if (planeside(z,zz) == 0)
					{
						splitnum++;
						if (splitnum >= minsplitnum) break;
					}
			}
			if (splitnum < minsplitnum)
			{
				bestz = z; daplistindex = zzx;
				if (splitnum == 0) break;
				minsplitnum = splitnum;
			}

			maxcnt--;
			if (maxcnt < 0) break;
		}
		z = bestz;

		koutp(67,182); koutp(66,2); koutp(66,pnumplanes);
		koutp(97,kinp(97)|3);

		for(zx=xpnumplanes-1;zx>=0;zx--)
		{
			zz = plist[front[zx]];
			if (z != zz)
			{
				k = planeside(z,zz);
				if (k != 0)
					parent[zz] = z+((k<0)<<14);
				else
				{
					splitpoly(z,zz);   //Split plane segment zz by plane z
					plist[pnumplanes++] = numplanes-1;
					parent[zz] = z+16384; parent[numplanes-1] = z;
					drawbspbox((long)numplanes-1,whitecol^128);
				}
			}
		}

		if (keystatus[1] > 0)
		{
			setvmode(0x3);
			printf("Quit while making 3D bsp\n");
			exit(0);
		}

		drawbspbox(z,blackcol);

		pnumplanes--;
		plist[daplistindex] = plist[pnumplanes];
		koutp(97,kinp(97)&252);
	}

	for(z=0;z<numplanes;z++)
		front[z] = -1, back[z] = -1;

	head = -1;
	for(z=0;z<numplanes;z++)
	{
		if (parent[z] == -1)
			head = z;
		else if ((parent[z]&16384) != 0)
			back[parent[z]&16383] = z;
		else
			front[parent[z]] = z;
	}

	for(plane=numplanes-1;plane>=0;plane--)
	{
		xmin = 0x7fffffff; xmax = 0x80000000;
		ymin = 0x7fffffff; ymax = 0x80000000;
		zmin = 0x7fffffff; zmax = 0x80000000;

		j = linoff[plane];
		for(i=j+linnum[plane]-1;i>=j;i--)
		{
			k = lin[i];
			if (px[k] < xmin) xmin = px[k];
			if (py[k] < ymin) ymin = py[k];
			if (pz[k] < zmin) zmin = pz[k];
			if (px[k] > xmax) xmax = px[k];
			if (py[k] > ymax) ymax = py[k];
			if (pz[k] > zmax) zmax = pz[k];
		}
		pxmin[plane] = xmin; pxmax[plane] = xmax;
		pymin[plane] = ymin; pymax[plane] = ymax;
		pzmin[plane] = zmin; pzmax[plane] = zmax;
	}
}

planeside (long z1, long z2)
{
	char sid;
	long z, zx, offsz1, zend;

	sid = 0; offsz1 = lin[linoff[z1]]; zend = linoff[z2]+linnum[z2];

	if (nyi[z1] == 0)                 //Optimal cases for walls
	{
		if (nxi[z1] == 0)
		{
			if (nzi[z1] == 0)           //EVIL CASE!
				return(1);

			offsz1 = pz[offsz1];
			if (nzi[z1] > 0)
			{
				for(z=linoff[z2];z<zend;z++)
				{
					zx = pz[lin[z]]-offsz1;
					sid |= ((zx<0)+((zx>0)<<1));
				}
				if (sid == 3) return(0);
				if (sid == 2) return(-1);
				return(1);
			}
			for(z=linoff[z2];z<zend;z++)
			{
				zx = offsz1-pz[lin[z]];
				sid |= ((zx<0)+((zx>0)<<1));
			}
			if (sid == 3) return(0);
			if (sid == 2) return(-1);
			return(1);
		}
		if (nzi[z1] == 0)
		{
			offsz1 = px[offsz1];
			if (nxi[z1] > 0)
			{
				for(z=linoff[z2];z<zend;z++)
				{
					zx = px[lin[z]]-offsz1;
					sid |= ((zx<0)+((zx>0)<<1));
				}
				if (sid == 3) return(0);
				if (sid == 2) return(-1);
				return(1);
			}
			for(z=linoff[z2];z<zend;z++)
			{
				zx = offsz1-px[lin[z]];
				sid |= ((zx<0)+((zx>0)<<1));
			}
			if (sid == 3) return(0);
			if (sid == 2) return(-1);
			return(1);
		}
		for(z=linoff[z2];z<zend;z++)
		{
			zx = mulscale(nxi[z1],px[lin[z]]-px[offsz1],8) +
				  mulscale(nzi[z1],pz[lin[z]]-pz[offsz1],8);
			sid |= ((zx<0)+((zx>0)<<1));
			if (sid == 3) return(0);
		}
		if (sid == 2) return(-1);
		return(1);
	}

	if ((nxi[z1]|nzi[z1]) == 0)            //Optimal case for ceilings/floors
	{
		offsz1 = py[offsz1];
		if (nyi[z1] > 0)
		{
			for(z=linoff[z2];z<zend;z++)
			{
				zx = py[lin[z]]-offsz1;
				sid |= ((zx<0)+((zx>0)<<1));
			}
			if (sid == 3) return(0);
			if (sid == 2) return(-1);
			return(1);
		}
		for(z=linoff[z2];z<zend;z++)
		{
			zx = offsz1-py[lin[z]];
			sid |= ((zx<0)+((zx>0)<<1));
		}
		if (sid == 3) return(0);
		if (sid == 2) return(-1);
		return(1);
	}

		//General case (use this if rewrite needed)
	for(z=linoff[z2];z<zend;z++)
	{
		zx = mulscale(nxi[z1],px[lin[z]]-px[offsz1],8) +
			  mulscale(nyi[z1],py[lin[z]]-py[offsz1],8) +
			  mulscale(nzi[z1],pz[lin[z]]-pz[offsz1],8);
		sid |= ((zx<0)+((zx>0)<<1));
		//if (zx < 0) sid |= 1;
		//if (zx > 0) sid |= 2;
		if (sid == 3) return(0);
	}
	if (sid == 2) return(-1);
	return(1);
}

splitpoly(long z, long zz)    //Split plane segment zz by plane z
{
	long i, j, k, s1, s2, t, intx, inty, intz, start2;
	long cx, cy, cz, nx, ny, nz, z1, z2, z3, z4;

		//Copy plane attributes
	for(i=0;i<3;i++)
	{
		bxv[numplanes][i] = bxv[zz][i];
		byv[numplanes][i] = byv[zz][i];
		bzv[numplanes][i] = bzv[zz][i];
	}
	picnum[numplanes] = picnum[zz];
	shade[numplanes] = shade[zz];
	planestat[numplanes] = planestat[zz];
	nxi[numplanes] = nxi[zz];
	nyi[numplanes] = nyi[zz];
	nzi[numplanes] = nzi[zz];

	t = lin[linoff[z]];
	cx = px[t]; nx = nxi[z];
	cy = py[t]; ny = nyi[z];
	cz = pz[t]; nz = nzi[z];

		//Calculate front plane segment
	linoff[numplanes] = numlines;
	start2 = numlines; splitcnt = 0; i = linoff[zz];
	do
	{
		t = lin[i]; s2 = mulscale(nx,px[t]-cx,8)+mulscale(ny,py[t]-cy,8)+mulscale(nz,pz[t]-cz,8);
		do
		{
			j = lin2[i]; lin2[i] |= 0x8000;

			s1 = s2;
			t = lin[j]; s2 = mulscale(nx,px[t]-cx,8)+mulscale(ny,py[t]-cy,8)+mulscale(nz,pz[t]-cz,8);

			if (s1 < 0)
			{
				lin2[numlines] = numlines+1;
				lin[numlines++] = lin[i];
			}
			if ((s1^s2) < 0)
			{
				intx = px[lin[i]] + scale(px[lin[j]]-px[lin[i]],s1,s1-s2);
				inty = py[lin[i]] + scale(py[lin[j]]-py[lin[i]],s1,s1-s2);
				intz = pz[lin[i]] + scale(pz[lin[j]]-pz[lin[i]],s1,s1-s2);
				lin2[numlines] = numlines+1;
				lin[numlines] = newpoint(intx,inty,intz);
				if (s1 < 0) split[splitcnt++] = numlines;
				numlines++;
			}
			i = j;
		} while ((lin2[i]&0x8000) == 0);

		if (numlines >= start2+3)
			lin2[numlines-1] = start2, start2 = numlines;
		else
			numlines = start2;

		i = linoff[zz]+linnum[zz]-1;
		while ((i > linoff[zz]) && ((lin2[i]&0x8000) > 0)) i--;
	} while (i > linoff[zz]);

	for(i=1;i<splitcnt;i++)
		for(j=0;j<i;j++)
		{
			z1 = split[i]; z2 = lin2[z1]; z3 = split[j]; z4 = lin2[z3];
			z1 = lin[z1]; z2 = lin[z2]; z3 = lin[z3]; z4 = lin[z4];
			s1  = klabs(px[z1]-px[z2])+klabs(py[z1]-py[z2])+klabs(pz[z1]-pz[z2]);
			s1 += klabs(px[z3]-px[z4])+klabs(py[z3]-py[z4])+klabs(pz[z3]-pz[z4]);
			s2  = klabs(px[z1]-px[z4])+klabs(py[z1]-py[z4])+klabs(pz[z1]-pz[z4]);
			s2 += klabs(px[z3]-px[z2])+klabs(py[z3]-py[z2])+klabs(pz[z3]-pz[z2]);
			if (s2 < s1)
			{ t = lin2[split[i]]; lin2[split[i]] = lin2[split[j]]; lin2[split[j]] = t; }
		}

		linnum[numplanes] = numlines-linoff[numplanes];
		numplanes++;


		//Fix lin2's
	for(i=linoff[zz]+linnum[zz]-1;i>=linoff[zz];i--)
		lin2[i] &= ~0x8000;


		//Calculate back plane segment
	linoff[numplanes] = numlines;
	start2 = numlines; splitcnt = 0; i = linoff[zz];
	do
	{
		t = lin[i]; s2 = mulscale(nx,cx-px[t],8)+mulscale(ny,cy-py[t],8)+mulscale(nz,cz-pz[t],8);
		do
		{
			j = lin2[i]; lin2[i] |= 0x8000;

			s1 = s2;
			t = lin[j]; s2 = mulscale(nx,cx-px[t],8)+mulscale(ny,cy-py[t],8)+mulscale(nz,cz-pz[t],8);

			if (s1 < 0)
			{
				lin2[numlines] = numlines+1;
				lin[numlines++] = lin[i];
			}
			if ((s1^s2) < 0)
			{
				intx = px[lin[i]] + scale(px[lin[j]]-px[lin[i]],s1,s1-s2);
				inty = py[lin[i]] + scale(py[lin[j]]-py[lin[i]],s1,s1-s2);
				intz = pz[lin[i]] + scale(pz[lin[j]]-pz[lin[i]],s1,s1-s2);
				lin2[numlines] = numlines+1;
				lin[numlines] = newpoint(intx,inty,intz);
				if (s1 < 0) split[splitcnt++] = numlines;
				numlines++;
			}

			i = j;
		} while ((lin2[i]&0x8000) == 0);

		if (numlines >= start2+3)
			lin2[numlines-1] = start2, start2 = numlines;
		else
			numlines = start2;

		i = linoff[zz]+linnum[zz]-1;
		while ((i > linoff[zz]) && ((lin2[i]&0x8000) > 0)) i--;
	} while (i > linoff[zz]);

	for(i=1;i<splitcnt;i++)
		for(j=0;j<i;j++)
		{
			z1 = split[i]; z2 = lin2[z1]; z3 = split[j]; z4 = lin2[z3];
			z1 = lin[z1]; z2 = lin[z2]; z3 = lin[z3]; z4 = lin[z4];
			s1  = klabs(px[z1]-px[z2])+klabs(py[z1]-py[z2])+klabs(pz[z1]-pz[z2]);
			s1 += klabs(px[z3]-px[z4])+klabs(py[z3]-py[z4])+klabs(pz[z3]-pz[z4]);
			s2  = klabs(px[z1]-px[z4])+klabs(py[z1]-py[z4])+klabs(pz[z1]-pz[z4]);
			s2 += klabs(px[z3]-px[z2])+klabs(py[z3]-py[z2])+klabs(pz[z3]-pz[z2]);
			if (s2 < s1)
				{ t = lin2[split[i]]; lin2[split[i]] = lin2[split[j]]; lin2[split[j]] = t; }
		}

	linnum[numplanes] = numlines-linoff[numplanes];
	numplanes++;

		//Resize for copying back plane into original
	resizelinnum(zz,(long)linnum[numplanes-1]);

		//Delete back plane
	numplanes--; numlines -= linnum[numplanes];

		//Copy back plane into original spot
	j = linoff[zz]-linoff[numplanes];
	for(i=linnum[numplanes]-1;i>=0;i--)
	{
		lin[i+linoff[zz]] = lin[i+linoff[numplanes]];
		lin2[i+linoff[zz]] = lin2[i+linoff[numplanes]]+j;
	}
}

setpalette(long dabrightness)
{
	char *ptr;
	long i, j, dist, daval;

	curbrightness = (long)dabrightness;
	if (curbrightness < 0) curbrightness = 0;
	if (curbrightness > 15) curbrightness = 15;

	if (option[0] == 6)
	{
		for(i=0;i<256;i++)
		{
			dist = palette[i*3]+palette[i*3+1]+palette[i*3+2];
			dist >>= 1;
			ptr = (char *)(FP_OFF(palookup[0])+(i<<1));
			for(j=0;j<32;j++)
			{
				daval = ((dist*(31-j))>>7);
				if (daval > 15) daval = 15;
				*ptr = daval;
				ptr += 256;
			}
		}

		koutp(0x3c8,0);
		for(j=0;j<16;j++)
			for(i=0;i<16;i++)
			{
				koutp(0x3c9,britable[curbrightness][i<<2]);
				koutp(0x3c9,0);
				koutp(0x3c9,britable[curbrightness][j<<2]);
			}
	}
	else
	{
		koutp(0x3c8,0);
		for(i=0;i<768;i++)
			koutp(0x3c9,britable[curbrightness][palette[i]]);
	}
}

calcscreenconsts()
{
	long i, j;

	xreszoom = xres*zoom;
	xdimscale = divscale32(xdim,xdim*zoom);
	ydimscale = divscale32(ydim,xdim*zoom);
	xdimscale1 = mulscale(xdimscale,65536-xpan,16);
	xdimscale2 = mulscale(xdimscale,65536+xpan,16);
	ydimscale1 = mulscale(ydimscale,65536-ypan,16);
	ydimscale2 = mulscale(ydimscale,65536+ypan,16);
	xdimup15 = (xdim<<15)-xpan*xres;
	ydimup15 = (ydim<<15)-ypan*xres;

	j = 0;
	for(i=0;i<MAXYDIM;i++)
	{
		ylookup[i] = j;
		j += xdim;
	}
}

stereoinit()
{
	long i;

	setverts(510L,487L,0L,479L,12L,479L);

	outp(0x3d4,0x38); outp(0x3d5,0x48);           //S3 Extensions enable #1
	outp(0x3d4,0x39); outp(0x3d5,0xa5);           //S3 Extensions enable #2
	outp(0x3d4,0x31); outp(0x3d5,inp(0x3d5)|9);   //Init for start address

	outp(0x3d4,0x9); outp(0x3d5,inp(0x3d5)&(255-1));

	for(i=0;i<4;i++)
	{
		outp(0x3d4,0x35); outp(0x3d5,i); clearbuf(0xa0000,65536>>2,0L);
	}
}

setverts(long vertotal, long startverblank, long endverblank,
			long veretstart, long veretend, long verdispenab)
{
	char oflow;

	outp(0x3d4,0x11); outp(0x3d5,inp(0x3d5)&127);
	outp(0x3d4,0x6); outp(0x3d5,vertotal&255);

	oflow = 0;
	oflow += ((vertotal&256)>>8);
	oflow += ((verdispenab&256)>>7);
	oflow += ((veretstart&256)>>6);
	oflow += ((startverblank&256)>>5);
	oflow += ((vertotal&512)>>4);
	oflow += ((verdispenab&512)>>3);
	oflow += ((veretstart&512)>>2);
	outp(0x3d4,0x7); outp(0x3d5,(inp(0x3d5)&0x10)|oflow);

	outp(0x3d4,0x9); outp(0x3d5,(inp(0x3d5)&(255-32))|((startverblank&512)>>4));
	outp(0x3d4,0x10); outp(0x3d5,veretstart&255);
	outp(0x3d4,0x11); outp(0x3d5,(inp(0x3d5)&0xf0)+(veretend&15));
	outp(0x3d4,0x12); outp(0x3d5,verdispenab&255);
	outp(0x3d4,0x15); outp(0x3d5,startverblank&255);
	outp(0x3d4,0x16); outp(0x3d5,endverblank&255);

	outp(0x3d4,0x9); outp(0x3d5,inp(0x3d5)&(255-1));
}

stereoblit(long stereobuf1, long stereobuf2)
{
	outp(0x3d4,0x35); outp(0x3d5,page+0); copybuf(stereobuf1,0xa0000,64000>>2);
	outp(0x3d4,0x35); outp(0x3d5,page+1); copybuf(stereobuf2,0xa4000,49152>>2);
	outp(0x3d4,0x35); outp(0x3d5,page+2); copybuf(stereobuf2+49152,0xa0000,14848>>2);
}

printext256(long xpos, long ypos, short col, short backcol, char name[82], char fontsize)
{
	char *fontptr;
	short dat, dacol;
	long stx, zx, stupidbitoffset, x, y, charxsiz;

	stx = xpos;

	if (fontsize == 1)
	{
		fontptr = smalltextfont;
		charxsiz = 4;
		stupidbitoffset = 1;
	}
	else
	{
		fontptr = textfont;
		charxsiz = 8;
		stupidbitoffset = 0;
	}

	for(zx=0;zx<strlen(name);zx++)
	{
		dat = name[zx];

		for(y=0;y<8;y++)
			for(x=0;x<charxsiz;x++)
			{
				if ((fontptr[(dat<<3)+y]&(1<<(7-stupidbitoffset-x))) > 0)
					dacol = (short)col;
				else if (backcol >= 0)
					dacol = (short)backcol;
				else
					dacol = -1;

				if (dacol >= 0)
					drawpixel(ylookup[y+ypos]+(x+stx-stupidbitoffset)+frameplace,(long)dacol);
			}

		stx += charxsiz;
	}
}

load3d(char *filename)
{
	long i, j, k, m, fil, x, y, z, minval, maxval, scaleval, oval;
	long x1, y1, z1, x2, y2, z2;
	long startsnumpoints, startsnumplanes, startsnumobjects, tempbufcnt;
	short danumpoints, danumplanes, danumobjects, dashort;
	signed char ch;

	if ((fil = open(filename,O_BINARY|O_RDONLY,S_IREAD)) == -1) return(-1);

	startsnumpoints = snumpoints;
	startsnumplanes = snumplanes;
	startsnumobjects = snumobjects;

	tempbufcnt = 0; read(fil,tempbuf,4096);

	danumobjects = tempbuf[tempbufcnt++];
	if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
	danumobjects += (tempbuf[tempbufcnt++]<<8);
	if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }

	for(k=0;k<danumobjects;k++)
	{
		danumpoints = tempbuf[tempbufcnt++];
		if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
		danumpoints += (tempbuf[tempbufcnt++]<<8);
		if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }

		dashort = 0;
		for(i=0;i<danumpoints;i++)
		{
			oval = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			oval += (tempbuf[tempbufcnt++]<<8);
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			dashort += oval;
			spx[i+snumpoints] = (long)dashort;
		}

		dashort = 0;
		for(i=0;i<danumpoints;i++)
		{
			oval = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			oval += (tempbuf[tempbufcnt++]<<8);
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			dashort += oval;
			spy[i+snumpoints] = (long)dashort;
		}

		dashort = 0;
		for(i=0;i<danumpoints;i++)
		{
			oval = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			oval += (tempbuf[tempbufcnt++]<<8);
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			dashort += oval;
			spz[i+snumpoints] = (long)dashort;
		}

		danumplanes = tempbuf[tempbufcnt++];
		if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
		danumplanes += (tempbuf[tempbufcnt++]<<8);
		if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }

		oval = 0;
		for(i=0;i<danumplanes;i++)
		{
			ch = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			if (ch == 0x80)
			{
				oval = tempbuf[tempbufcnt++];
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
				oval += (tempbuf[tempbufcnt++]<<8);
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			}
			else
				oval += (long)ch;
			sp1[i+snumplanes] = oval;
		}

		oval = 0;
		for(i=0;i<danumplanes;i++)
		{
			ch = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			if (ch == 0x80)
			{
				oval = tempbuf[tempbufcnt++];
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
				oval += (tempbuf[tempbufcnt++]<<8);
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			}
			else
				oval += (long)ch;
			sp2[i+snumplanes] = oval;
		}

		oval = 0;
		for(i=0;i<danumplanes;i++)
		{
			ch = tempbuf[tempbufcnt++];
			if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			if (ch == 0x80)
			{
				oval = tempbuf[tempbufcnt++];
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
				oval += (tempbuf[tempbufcnt++]<<8);
				if (tempbufcnt >= 4096) { read(fil,tempbuf,4096); tempbufcnt = 0; }
			}
			else
				oval += (long)ch;
			sp3[i+snumplanes] = oval;
		}

		if ((danumpoints > 0) && (danumplanes > 0))
		{
			planeoff[snumobjects] = snumplanes;
			pointoff[snumobjects] = snumpoints;
			snumplanes += danumplanes;
			snumpoints += danumpoints;
			snumobjects++;
		}
	}
	planeoff[snumobjects] = snumplanes;
	pointoff[snumobjects] = snumpoints;

	if (snumpoints > startsnumpoints)
	{
		x = 0; y = 0; z = 0;
		for(i=startsnumpoints;i<snumpoints;i++) x += spx[i], y += spy[i], z += spz[i];
		x /= snumpoints, y /= snumpoints, z /= snumpoints;

		for(i=startsnumpoints;i<snumpoints;i++)
			spx[i] -= x, spy[i] -= y, spz[i] -= z;

		minval = 0x7fffffff; maxval = 0x80000000;
		for(i=startsnumpoints;i<snumpoints;i++)
		{
			if (spx[i] < minval) minval = spx[i];
			if (spx[i] > maxval) maxval = spx[i];
			if (spy[i] < minval) minval = spy[i];
			if (spy[i] > maxval) maxval = spy[i];
			if (spz[i] < minval) minval = spz[i];
			if (spz[i] > maxval) maxval = spz[i];
		}

		scaleval = (65536*2048)/(maxval-minval);
		for(i=startsnumpoints;i<snumpoints;i++)
		{
			spx[i] = mulscale(spx[i],scaleval,16);
			spy[i] = mulscale(spy[i],scaleval,16);
			spz[i] = mulscale(spz[i],scaleval,16);
		}
	}

	for(i=startsnumobjects;i<snumobjects;i++)
	{
		x = 0, y = 0, z = 0;
		for(j=pointoff[i];j<pointoff[i+1];j++)
			x += spx[j], y += spy[j], z += spz[j];

		scaleval = pointoff[i+1]-pointoff[i];
		sox[i] = x/scaleval;
		soy[i] = y/scaleval;
		soz[i] = z/scaleval;

		for(j=pointoff[i];j<pointoff[i+1];j++)
			spx[j] -= sox[i], spy[j] -= soy[i], spz[j] -= soz[i];

		for(j=planeoff[i];j<planeoff[i+1];j++)
		{
			k = pointoff[i];
			m = sp1[j]+k; x = spx[m]; y = spy[m]; z = spz[m];
			m = sp2[j]+k; x1 = spx[m]-x; y1 = spy[m]-y; z1 = spz[m]-z;
			m = sp3[j]+k; x2 = spx[m]-x; y2 = spy[m]-y; z2 = spz[m]-z;
			x = doublemulscale(y1,z2,-y2,z1,2);
			y = doublemulscale(x2,z1,-x1,z2,2);
			z = doublemulscale(x1,y2,-x2,y1,2);
			while (klabs(x) >= 32768) x >>= 1, y >>= 1, z >>= 1;
			while (klabs(y) >= 32768) x >>= 1, y >>= 1, z >>= 1;
			while (klabs(z) >= 32768) x >>= 1, y >>= 1, z >>= 1;
			k = ksqrtasm(x*x+y*y+z*z);
			if (k > 0)
			{
				k = (65536*8192*3)/k;
				snx[j] = mulscale(x,k,16);
				sny[j] = mulscale(y,k,16);
				snz[j] = mulscale(z,k,16);
			}
		}
	}

	groupoff[snumgroups++] = startsnumobjects;
	groupoff[snumgroups] = snumobjects;

	close(fil);
	return(0);
}

drawsprisect(long sectnum)
{
	long i, ii, j, k, m, o, templong, temp, gap, dasprinum;
	long x, y, z, dax, day, daz, xvect, yvect, zvect, dashade;
	long x1, y1, z1, x2, y2, z2;

	switch(spritedrawmode)
	{
		case 0:
			for(i=headspritesect[sectnum];i>=0;i=nextspritesect[i])
				if ((spri[i].xscale != 0) && (spri[i].yscale != 0) && (spri[i].zscale != 0))
				{
					o = spri[i].objnum;
					gxx = hxx; gxy = hxy; gxz = hxz;
					gyx = hyx; gyy = hyy; gyz = hyz;
					gzx = hzx; gzy = hzy; gzz = hzz;
					xvect = sox[o]+spri[i].x-posx;
					yvect = soy[o]+spri[i].y-posy;
					zvect = soz[o]+spri[i].z-posz;
					rotate3d(&xvect,&yvect,&zvect);
					addrotate3d(spri[i].a1,spri[i].a2,spri[i].a3,i);
					for(j=pointoff[o];j<pointoff[o+1];j++)
					{
						dax = spx[j]; day = spy[j]; daz = spz[j];
						rotate3d(&dax,&day,&daz);
						dax += xvect; day += yvect; daz += zvect;
						if (daz > 4)
						{
							projectlo(dax,day,daz,&x,&y);
							if ((x|y) >= 0)
								if ((x < xdim) && (y < ydim))
									drawpixel(ylookup[y]+x+frameplace,palookup[0][15]);
						}
					}
				}
			break;
		case 1:
			for(i=headspritesect[sectnum];i>=0;i=nextspritesect[i])
				if ((spri[i].xscale != 0) && (spri[i].yscale != 0) && (spri[i].zscale != 0))
				{
					o = spri[i].objnum;
					gxx = hxx; gxy = hxy; gxz = hxz;
					gyx = hyx; gyy = hyy; gyz = hyz;
					gzx = hzx; gzy = hzy; gzz = hzz;
					xvect = sox[o]+spri[i].x-posx;
					yvect = soy[o]+spri[i].y-posy;
					zvect = soz[o]+spri[i].z-posz;
					rotate3d(&xvect,&yvect,&zvect);
					addrotate3d(spri[i].a1,spri[i].a2,spri[i].a3,i);
					k = 0;
					for(j=pointoff[o];j<pointoff[o+1];j++)
					{
						dax = spx[j]; day = spy[j]; daz = spz[j];
						rotate3d(&dax,&day,&daz);
						dax += xvect; day += yvect; daz += zvect;
						if (daz > 4)
						{
							projectlo(dax,day,daz,&sx[k],&sy[k]);
							if (sx[k] < 0) sx[k] = 0;
							if (sx[k] >= xdim) sx[k] = xdim-1;
							if (sy[k] < 0) sy[k] = 0;
							if (sy[k] >= ydim) sy[k] = ydim-1;
						}
						else
							sx[k] = 0x80000000;
						k++;
					}
					for(j=planeoff[o];j<planeoff[o+1];j++)
						if ((sx[sp1[j]] != 0x80000000) && (sx[sp2[j]] != 0x80000000) && (sx[sp3[j]] != 0x80000000))
						{
							templong = palookup[0][((i&7)<<5)+23];
							drawline(sx[sp1[j]]<<16,sy[sp1[j]]<<16,sx[sp2[j]]<<16,sy[sp2[j]]<<16,(char)templong);
							drawline(sx[sp2[j]]<<16,sy[sp2[j]]<<16,sx[sp3[j]]<<16,sy[sp3[j]]<<16,(char)templong);
							drawline(sx[sp3[j]]<<16,sy[sp3[j]]<<16,sx[sp1[j]]<<16,sy[sp1[j]]<<16,(char)templong);
						}
				}
			break;
		case 2:
			dasprinum = 0;
			for(i=headspritesect[sectnum];i>=0;i=nextspritesect[i])
			{
				o = spri[i].objnum;
				dax = sox[o]+spri[i].x-posx;
				day = soy[o]+spri[i].y-posy;
				daz = soz[o]+spri[i].z-posz;
					//Perpendicular distance
				if (triplemulscale28(dax,gxz,day,gyz,daz,gzz) >= 0)
				{
						//Euclidian distance
					objdist[dasprinum] = triplemulscale8(dax,dax,day,day,daz,daz);
					objsort[dasprinum] = i;
					dasprinum++;
				}
			}

			gap = (dasprinum>>1);
			do
			{
				for(i=0;i<dasprinum-gap;i++)
				{
					temp = i;
					while ((objdist[temp] > objdist[temp+gap]) && (temp >= 0))
					{
						templong = objsort[temp]; objsort[temp] = objsort[temp+gap]; objsort[temp+gap] = templong;
						templong = objdist[temp]; objdist[temp] = objdist[temp+gap]; objdist[temp+gap] = templong;
						temp -= gap;
					}
				}
				gap >>= 1;
			} while (gap > 0);

			for(ii=dasprinum-1;ii>=0;ii--)
			{
				i = objsort[ii];
				if ((spri[i].xscale != 0) && (spri[i].yscale != 0) && (spri[i].zscale != 0))
				{
					o = spri[i].objnum;
					gxx = hxx; gxy = hxy; gxz = hxz;
					gyx = hyx; gyy = hyy; gyz = hyz;
					gzx = hzx; gzy = hzy; gzz = hzz;
					xvect = sox[o]+spri[i].x-posx;
					yvect = soy[o]+spri[i].y-posy;
					zvect = soz[o]+spri[i].z-posz;
					rotate3d(&xvect,&yvect,&zvect);
					addrotate3d(spri[i].a1,spri[i].a2,spri[i].a3,i);
					k = 0;
					for(j=pointoff[o];j<pointoff[o+1];j++)
					{
						dax = spx[j]; day = spy[j]; daz = spz[j];
						rotate3d(&dax,&day,&daz);
						dax += xvect; day += yvect; daz += zvect;
						if (daz > 4)
							projectlo(dax,day,daz,&sx[k],&sy[k]);
						else
							sx[k] = 0x80000000;
						k++;
					}
					setuprotate3d(spri[i].a1,spri[i].a2,spri[i].a3);
					for(j=planeoff[o];j<planeoff[o+1];j++)
						if (sx[sp1[j]] != 0x80000000)
							if (sx[sp2[j]] != 0x80000000)
								if (sx[sp3[j]] != 0x80000000)
								{
									dashade = triplemulscale28(snx[j],gxx,sny[j],gxy,snz[j],gxz);
									dashade = (dashade>>11)+(objdist[ii]>>18)+(j&1)+12;
									if (dashade < 0) dashade = 0;
									if (dashade > 31) dashade = 31;
									drawsolidtri(j,(long)palookup[dashade][((i&7)<<5)+31]);
								}
				}
			}
			break;
	}
	gxx = hxx; gxy = hxy; gxz = hxz;
	gyx = hyx; gyy = hyy; gyz = hyz;
	gzx = hzx; gzy = hzy; gzz = hzz;
}

projectlo (long x, long y, long z, long *dasx, long *dasy)
{
	long t;

	t = divscale(xreszoom,z,8);
	*dasx = mulscale(x,t,24) + (xdimup15>>16);
	*dasy = mulscale(y,t,24) + (ydimup15>>16);
}

drawsolidtri(long i, long col)
{
	long p, lastp, v, nextv, z[3], upoint, dpoint, uval, dval;
	long x1, y1, x2, y2, dax1, day1, dax2, day2;
	long x, y, xplc, xinc;

	z[0] = sp3[i]; z[1] = sp2[i]; z[2] = sp1[i];

	if ((sx[z[1]]-sx[z[0]])*(sy[z[2]]-sy[z[0]]) >= (sy[z[1]]-sy[z[0]])*(sx[z[2]]-sx[z[0]]))
	{
		col &= 255;
		col += (col<<8);
		col += (col<<16);

		upoint = 0; dpoint = 0;
		uval = sy[z[0]]; dval = uval;
		if (sy[z[1]] < uval) upoint = 1, uval = sy[z[1]];
		if (sy[z[1]] > dval) dpoint = 1, dval = sy[z[1]];
		if (sy[z[2]] < uval) upoint = 2, uval = sy[z[2]];
		if (sy[z[2]] > dval) dpoint = 2, dval = sy[z[2]];

		if ((dval < 0) || (uval > ydim)) return;

		v = dpoint;

		x2 = sx[z[v]];
		y2 = sy[z[v]];
		do
		{
			nextv = v+1; if (nextv == 3) nextv = 0;

			x1 = x2; y1 = y2;
			x2 = sx[z[nextv]]; y2 = sy[z[nextv]];

			day1 = y1; if (day1 > ydim) day1 = ydim;
			day2 = y2; if (day2 < 0) day2 = 0;
			if (day1 > day2)
			{
				xinc = divscale(x1-x2,y1-y2,16);
				xplc = (x2<<16);
				if (day2 != y2) xplc += (day2-y2)*xinc;
				qinterpolatedown16((long)(&lastx[day2]),day1-day2,xplc,xinc);
			}

			v = nextv;
		} while (v != upoint);

		do
		{
			nextv = v+1; if (nextv == 3) nextv = 0;

			x1 = x2; y1 = y2;
			x2 = sx[z[nextv]]; y2 = sy[z[nextv]];

			day1 = y1; if (day1 < 0) day1 = 0;
			day2 = y2; if (day2 > ydim) day2 = ydim;
			if (y2 > y1)
			{
				xinc = divscale(x2-x1,y2-y1,16);
				xplc = (x1<<16);
				if (day1 != y1) xplc += (day1-y1)*xinc;
				for(y=day1;y<day2;y++)
				{
					dax1 = lastx[y]; if (dax1 < 0) dax1 = 0;
					dax2 = (xplc>>16); if (dax2 > xdim) dax2 = xdim;
					if (dax2 > dax1)
					{
						p = frameplace + ylookup[y]+dax1;
						lastp = p+(dax2-dax1);

						while (p < lastp-3) drawpixelses(p,col), p += 4;
						if (p < lastp-1) drawpixels(p,col), p += 2;
						if (p < lastp) drawpixel(p,col);
					}
					xplc += xinc;
				}
			}

			v = nextv;
		} while (v != dpoint);
	}
}

getregion(long x, long y, long z)
{
	long i, j, k;

	i = head;
	while (1)
	{
		j = lin[linoff[i]]; k = 0;
		if (nxi[i] != 0) k += mulscale(nxi[i],x-px[j],8);
		if (nyi[i] != 0) k += mulscale(nyi[i],y-py[j],8);
		if (nzi[i] != 0) k += mulscale(nzi[i],z-pz[j],8);
		if (k <= 0)
		{
			j = front[i];
			if (j < 0) return(i);
			i = j;
		}
		else
		{
			j = back[i];
			if (j < 0) return(i+MAXPLANES);
			i = j;
		}
	}
	return(-1);
}

initspritelists()
{
	long i;

	for (i=0;i<MAXREGIONS;i++)     //Init doubly-linked sprite sector lists
		headspritesect[i] = -1;
	headspritesect[MAXREGIONS] = 0;
	for(i=0;i<MAXSPRITES;i++)
	{
		prevspritesect[i] = i-1;
		nextspritesect[i] = i+1;
		spri[i].sectnum = MAXREGIONS;
	}
	prevspritesect[0] = -1;
	nextspritesect[MAXSPRITES-1] = -1;


	for(i=0;i<MAXSTATUS;i++)      //Init doubly-linked sprite status lists
		headspritestat[i] = -1;
	headspritestat[MAXSTATUS] = 0;
	for(i=0;i<MAXSPRITES;i++)
	{
		prevspritestat[i] = i-1;
		nextspritestat[i] = i+1;
		spri[i].statnum = MAXSTATUS;
	}
	prevspritestat[0] = -1;
	nextspritestat[MAXSPRITES-1] = -1;
}

insertsprite(short sectnum, short statnum)
{
	insertspritestat(statnum);
	return(insertspritesect(sectnum));
}

insertspritesect(short sectnum)
{
	short blanktouse;

	if ((sectnum < 0) || (sectnum >= MAXREGIONS) || (headspritesect[MAXREGIONS] == -1))
		return(-1);  //list full

	blanktouse = headspritesect[MAXREGIONS];

	headspritesect[MAXREGIONS] = nextspritesect[blanktouse];
	if (headspritesect[MAXREGIONS] >= 0)
		prevspritesect[headspritesect[MAXREGIONS]] = -1;

	prevspritesect[blanktouse] = -1;
	nextspritesect[blanktouse] = headspritesect[sectnum];
	if (headspritesect[sectnum] >= 0)
		prevspritesect[headspritesect[sectnum]] = blanktouse;
	headspritesect[sectnum] = blanktouse;

	spri[blanktouse].sectnum = sectnum;

	return(blanktouse);
}

insertspritestat(short statnum)
{
	short blanktouse;

	if ((statnum < 0) || (statnum >= MAXSTATUS) || (headspritestat[MAXSTATUS] == -1))
		return(-1);  //list full

	blanktouse = headspritestat[MAXSTATUS];

	headspritestat[MAXSTATUS] = nextspritestat[blanktouse];
	if (headspritestat[MAXSTATUS] >= 0)
		prevspritestat[headspritestat[MAXSTATUS]] = -1;

	prevspritestat[blanktouse] = -1;
	nextspritestat[blanktouse] = headspritestat[statnum];
	if (headspritestat[statnum] >= 0)
		prevspritestat[headspritestat[statnum]] = blanktouse;
	headspritestat[statnum] = blanktouse;

	spri[blanktouse].statnum = statnum;

	return(blanktouse);
}

deletesprite(short spritenum)
{
	deletespritestat(spritenum);
	return(deletespritesect(spritenum));
}

deletespritesect(short deleteme)
{
	if (spri[deleteme].sectnum == MAXREGIONS)
		return(-1);

	if (headspritesect[spri[deleteme].sectnum] == deleteme)
		headspritesect[spri[deleteme].sectnum] = nextspritesect[deleteme];

	if (prevspritesect[deleteme] >= 0) nextspritesect[prevspritesect[deleteme]] = nextspritesect[deleteme];
	if (nextspritesect[deleteme] >= 0) prevspritesect[nextspritesect[deleteme]] = prevspritesect[deleteme];

	if (headspritesect[MAXREGIONS] >= 0) prevspritesect[headspritesect[MAXREGIONS]] = deleteme;
	prevspritesect[deleteme] = -1;
	nextspritesect[deleteme] = headspritesect[MAXREGIONS];
	headspritesect[MAXREGIONS] = deleteme;

	spri[deleteme].sectnum = MAXREGIONS;
	return(0);
}

deletespritestat (short deleteme)
{
	if (spri[deleteme].statnum == MAXSTATUS)
		return(-1);

	if (headspritestat[spri[deleteme].statnum] == deleteme)
		headspritestat[spri[deleteme].statnum] = nextspritestat[deleteme];

	if (prevspritestat[deleteme] >= 0) nextspritestat[prevspritestat[deleteme]] = nextspritestat[deleteme];
	if (nextspritestat[deleteme] >= 0) prevspritestat[nextspritestat[deleteme]] = prevspritestat[deleteme];

	if (headspritestat[MAXSTATUS] >= 0) prevspritestat[headspritestat[MAXSTATUS]] = deleteme;
	prevspritestat[deleteme] = -1;
	nextspritestat[deleteme] = headspritestat[MAXSTATUS];
	headspritestat[MAXSTATUS] = deleteme;

	spri[deleteme].statnum = MAXSTATUS;
	return(0);
}

changespritesect(short spritenum, short newsectnum)
{
	if ((newsectnum < 0) || (newsectnum > MAXREGIONS)) return(-1);
	if (spri[spritenum].sectnum == newsectnum) return(-1);
	if (spri[spritenum].sectnum == MAXREGIONS) return(-1);
	if (deletespritesect(spritenum) < 0) return(-1);
	insertspritesect(newsectnum);
	return(0);
}

changespritestat(short spritenum, short newstatnum)
{
	if ((newstatnum < 0) || (newstatnum > MAXSTATUS)) return(-1);
	if (spri[spritenum].statnum == newstatnum) return(-1);
	if (spri[spritenum].statnum == MAXSTATUS) return(-1);
	if (deletespritestat(spritenum) < 0) return(-1);
	insertspritestat(newstatnum);
	return(0);
}
